function [ ijtor ] = gatherIJTOR( pth, opt )
%% gatherIJTOR( path, opt )
%
% Opens all files specified by the opt.regex (default: 'ijtor.*') as binaries.
% Then tries to read them as ijtor files:
% i: disturbed node, j: reacting node, t: theta/phase value
% o: omega/frequency value, r: return time value
%
% For further processing of ijtor arrays see getSurfs( ijtor) or surfIJTOR
%
% * Parameters
%   * pth : Path to the ijtor files
%   * opt : option struct
%       * opt.regex = 'ijtor.*': regex to find them
%       * opt.mat = false       if a mat file should be searched and loaded
%       * opt.mreg = '.*.mat'   the regexpression to find mat file
%       * opt.msav = ''         if and how to save mat file (empty means no
%                               saving)
%       * opt.wb = [];          handle of a waitbar. 'def' for default bar
%
%  * Returns
%   * ijtor: 5xN matrix
%
% 
% V1.01 mawolff@uos.de 26-May-2019


% V1.01 mawolff@uos.de 26-May-2019
% improved memory allocation
% V1.0 mawolff@uos.de 14-Aug-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt = struct(); 
end
 
 opt = readOptions( opt );
 
 %% code
 if( opt.mat )                                  % if requested by user
     fls = getFilesByRegex( pth, opt.mreg);     % look for .mat files
     if ( numel(fls) ~= 0)                      % if there are any
         try                                    % try to read the 
             S = load([pth fls{1}],'-regexp','ijtor.*');    % first matching one
             ijtor = S.ijtor;
             if( ~isempty( opt.wb ) )
                 close(opt.wb);
             end
             return;
         catch ME
             switch ME.identifier
                 case 'MATLAB:load:couldNotReadFile'
                     warning(['File ' fls{1} ...
                         ' could not be read.']);
                 case 'MATLAB:FileIO:InvalidFid'
                     warning(['File ' fls{1}  ' could not be find.']);
                 otherwise
                     rethrow( ME );
             end%switch
         end%try
     end%if
 end

 fls = getFilesByRegex( pth, opt.regex);
%  ijtor = zeros(5,1);
 prealloc = false;          % will become true after the array is preallocated
 
 if( ~isempty( opt.wb ) )
     if( ischar( opt.wb ) )
         wbo.name = ['reading ' num2str(numel(fls)) ' files'];
         wbo.barcolor = 'leafgreen';
         opt.wb= [];
         opt.wb = cWaitbar(wbo);
     end
 end
 
 fileIdx = 1;
 lastCol = 0;
 for i = 1:numel(fls)
     
     if( ~isempty( opt.wb ) )
         waitbar(i/numel(fls), opt.wb)
     end
     
     try
         if( strcmp(fls{i}(end-3:end),'.mat') )
             continue;
         end
         fid = fopen([pth fls{i}]);
         fin = fread( fid, inf, 'double' );
         cols = numel(fin)/5;   
         % during second run, one line is omitted
         if( ~prealloc )
            ijtor = nan(5,cols);    
            prealloc = true;
%             N1 = numel(fin);
%             N2 = N1 - 62*5*89;
         end
%          ijtor = [ijtor reshape(fin,5,numel(fin)/5)]; %#ok<AGROW>
%          fprintf('%d %d %d',numel(fin),numel(fin)/N1, numel(fin)/N2 );
%          disp(' ');
         ijtor(:,(1:cols)+lastCol) = reshape( fin,5,cols);
         fclose(fid);
         clear fin;
         lastCol = lastCol + cols;
         fileIdx = fileIdx + 1;     % iterate file index only if loop continued until here
     catch ME
         switch ME.identifier
             case 'MATLAB:load:couldNotReadFile'
                 warning(['File ' fls{i} ...
                     ' could not be read.']);
             case 'MATLAB:FileIO:InvalidFid'
                 warning(['File ' fls{i}  ' could not be find.']);
             otherwise
                 rethrow( ME );
         end%switch
     end%try
     
 end%for
 
 
 if( ~isempty( opt.wb ) )
     close(opt.wb);
 end
 
 ijtor = ijtor(:,2:end);
 if( ~strcmp(opt.msav,'') )
     save([pth opt.msav '.mat'],'ijtor');
 end
 
 return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'regex') )
    opt.regex = 'ijtor.*';
 end
 if( ~isfield( opt, 'mat') )
    opt.mat = false;
 end
 if( ~isfield( opt, 'mreg') )
    opt.mreg = '.*.mat';
 end
 if( ~isfield( opt, 'msav') )
    opt.msav = '';
 end
 if( ~isfield( opt, 'wb') )
     opt.wb = [];
 end
end
