function [ tb, limc ] = getMeanRT( s, opt )
%% getMeanRT( surf, opt )
% Calculates the average return time of the given surface
% using only times being finite (not belonging to limit cycle
% as defined by opt.lim)
%
% * Parameters
%   * s: the surface to get the mean return time from
%   * opt: option struct
%       * opt.eq = true; if not set false it is assumed that the surface
%               parts (rectangles) are equal in size. This simplifies the
%               sum and the speed of calculation. In this not equal case
%         opt.sx and opt.sy must be given
%       * opt.reminf = true; removes the values that are inf before
%           determining the mean (otherwise it would be inf too).
%       * opt.lim = inf; if there is a time to say this is limit cycle and
%                           no real return time
%
% * Returns
%   * tb: t^bar the mean return time for the given surface
%   * limc: if a limit cycle (inf return time occured)
%
% * See also: getAllMeanRT, getSurfs, surfGridRT, surfNodeRT
%
% V1.0 mawolff@uos.de 13-Dec-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code

limc = any(any(s >= opt.lim));

[sx, sy] = size( s );           % get number of surface elements
surfsize = sx*sy;
if( opt.eq )                    % if surface is divided into equal rectangles
    s = reshape(s,numel(s),1);  % put all return times in a single vector
    if( opt.reminf )            % if requested
        s(s>=opt.lim) = [];     % remove detected limit cycle initial conditions
        surfsize = numel(s);    % there are less elements now for calculating the mean
    end
    tb = sum(s)/surfsize;       % mean calculation ( sum divided by number)
else
    tb = 0;
    I = 0;
    for x = 1:sx
        for y = 1:sy
            tb = tb + s(x,y)*opt.sx(x,y)*opt.sy(x,y);   % TODO: there is an error
            I = I + opt.sx(x,y)*opt.sy(x,y);            % muss delta(sx)*delta(sy) sein (kleine flaechen aufsummieren)
        end
    end
    tb = tb/I;
end

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'reminf') )
    opt.reminf = true;
 end
 if( ~isfield( opt, 'lim') )
    opt.lim = inf;
 end
 if( ~isfield( opt, 'eq') )
    opt.eq = true;
 end
 if( ~opt.eq )
     if( ~isfield(opt, 'sx') || ~isfield(opt,'sy'))
         error(['when surface parts are not equally spaced'...
             ' surface x (sx) and sy must be specified.']);
     end
 end
end
