function [ surfs ] = getSurfs( ijtor, opt )
%% getSurfs( ijtor, opt )
%
% Processes the IJTOR raw data to obtain a cell array containing surfaces 
% where the x-y-plane spans all possible perturbations 
% (Delta voltage phase, Delta voltage frequency) 
% and z-values are the corresponding return times
%
%
% For further processing of ijtor-surfaces see:
%   * surfGridRT (get grid return time (maximum over all respond. nodes j))
%   * surfNodeRT (get node return time (maximum over all perturb. nodes k))
%   * getMeanRT  (get mean return time of single surface)
%   * getMaxRT   (get maximum return time of single surface)
%
%
% * Parameters
%   * ijtor : 5xN double matrix in the IJTOR-format
%               ( perturbed node I, responding node J, 
%               voltage phase perturbation T(heta),
%               voltage frequency perturbation O(mega), 
%               Return time)
%   * opt: option struct 
%       opt.iseq = 1:1 : the disturbed "i" nodes
%       opt.jseq = 1:1 : the responding "j" nodes
%       opt.wb = [] : waitbar 'def' for default     
%
%  * Returns
%   * surfs : cell array containing the surfaces as structs which again 
%               contain double matrices x,y and z
%
% * See also: ClusterRrtEG
%
% V1.0 mawolff@uos.de 22-Aug-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );

 
if( size(ijtor, 1) == 1)
    ijtor = reshape(ijtor, 5, numel(ijtor)/5);
end

%% code
 
Ni = numel(opt.iseq);
Nj = numel(opt.jseq);
 if( ~isempty( opt.wb ) )
     if( ischar( opt.wb ) )
         wbo.name = ['processing ' ...
             num2str( Ni*Nj ) ...
             ' surfaces'];
         wbo.barcolor = 'leafgreen';
         opt.wb= [];
         opt.wb = cWaitbar(wbo);
     end
 end
 
surfs = cell(numel(opt.iseq),numel(opt.jseq));
for ii = 1:numel(opt.iseq)
    for ji = 1:numel(opt.jseq)
        
        if( ~isempty( opt.wb ) )
            waitbar(  ((ii-1)*Nj+ji)/(Ni*Nj), opt.wb);
        end
        
        i = opt.iseq(ii);
        j = opt.jseq(ji);
        [rtsurf, xsurf, ysurf] = rtSurf(ijtor,i-1,j-1);
%         [rtsurf, xsurf, ysurf, ijtor] = rtSurf2(ijtor,i-1,j-1);
% only faster for veeeeery large systems
        
        ra = 1:size(xsurf,1);
        sf.x = xsurf(ra,:);
        sf.y = ysurf(ra,:);
        sf.z = rtsurf(ra,:);
        surfs{ii,ji} = sf;
    end
end

 if( ~isempty( opt.wb ) )
     close(opt.wb);
 end

return;
 
end
 
 
function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'iseq') )
    opt.iseq = 1;
 end
 if( ~isfield( opt, 'jseq') )
    opt.jseq = 1;
 end
 if( ~isfield( opt, 'wb') )
     opt.wb = [];
 end
end
