function [ surfmax ] = surfNodeRT( ijtor, opt )
%% surfNodeRT( ijtor, opt )
% surfNodeRT( surfs, opt )
%
% Calculates the return time surface t_j for one node j maximised over all 
% return time surfaces t_ij obtained from consecutively perturbing 
% all nodes i (including node j)
% Mathematically:
% t^node_j (theta, omega ) = max_i ( t_ij ( theta, omega ) )
%
% * Parameters
%   * ijtor : if a double matrix it is assumed to be in ijtor-format
%               (ij-nodes, theta, omega, return time) then the t_ij
%               surfaces are calculated from it.
%             if a cell matrix is given it is assmued that ijtor{i,j}.x
%             (and y and z respectively) are already the surfaces of t_ij.
%   * opt: option struct
%       * opt.iseq = 1:1 : the disturbed "i" nodes
%       * opt.jseq = 1:1 : the responding "j" nodes
%
%  * Returns
%   * surfmax : cell containing all the requested t^node_j surfaces
%
%
% V1.0 mawolff@uos.de 11-Dec-2017
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt );
 
%% code
 if( iscell( ijtor) )       % if input is a cell array
    surfs = ijtor;          % its entries are assumed to be the surfaces
    cellin = true;          % mark that input is cell array
 else                       % otherwise it is assumed data is ijtor-double matrix
    surfs = getSurfs(ijtor,opt);    % then surfaces are extracted from ijtor-matrix
    cellin = false;                 % mark that input was not cell
end

surfmax = cell(numel(opt.iseq),1);      % preallocate resulting matrix

for ji = 1:numel(opt.jseq)              % for every reacting node
    j = opt.iseq(ji);                   %
    for ii = 1:numel(opt.iseq)
        i = opt.iseq(ii);
        sx = inf;
        sy = inf;
        if( cellin )                        % if input was cell
            [sxs,sys] = size(surfs{i,j}.z);   % calc surface size from it
        else                                % otherwise
            [sxs,sys] = size(surfs{i,j});     % calc it from new surfaces
        end
        sx = min( sx, sxs );
        sy = min( sy, sys );
    end
    smaxj = zeros(sx,sy);
    for ii = 1:numel(opt.iseq)          % get all disturbed nodes
%         disp([ji ii]);
        i = opt.jseq(ii);               % get indice of that node
        if( cellin )
            csurf = surfs{i,j}.z;
        else
            csurf = surfs{i,j};
        end
        
        disp([ji ii ])
        for x = 1:sx
            for y = 1:sy
                smaxj(x,y) = max( smaxj(x,y), csurf(x,y));  % get maximum for every phase,freq pair
            end
        end
    end
    surfmax{ji}=smaxj;
end


return;
 
end
 
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
if( ~isfield( opt, 'iseq') )
    opt.iseq = 1;
end
if( ~isfield( opt, 'jseq') )
    opt.jseq = 1;
end
end
