/*
 * RasterReturnTime.cpp
 *
 *  Created on: Feb 25, 2019
 *      Author: mawolff
 */



#include "CodeUtils.h"

#include "RasterReturnTime.h"


// =================================================== constructors =================================================== //
RasterReturnTime::RasterReturnTime(PowerGrid* pg) {
	this->pgrid = pg;

	// all uncommented variables should be set by user in the parameter file
	firstPertNode = 0;
	curPertNode = 0;
	nodeIterator = 0;	// first node stays first node when this is initialized as zero
	lastPertNode = 0;

	pertDelay = 0;

	freqPertMin = 0;
	freqPertMax = 0;
	freqStepw = 0;
	phasePertMin = 0;
	phasePertMax = 0;
	phaseStepw = 0;

	thresholdHigh = 0;
	thresholdLow = 0;
	threshSpread = 0;
	numOfThresh = 0;
	fixedThreshNum = false;

	trconv = new bool*[1];
	trconv[0] = new bool[1];
	rtmat = new double*[1];
	rtmat[0] = new double[1];
	rtmatsize = 1;


}

RasterReturnTime::~RasterReturnTime() {

	for(int i = 0; i < rtmatsize; i++){
		delete[] rtmat[i];
	}
	delete[] rtmat;

	for(int i = 0; i < rtmatsize; i++){
		delete[] trconv[i];
	}
	delete[] trconv;

}

// ====================================================== methods ===================================================== //
// ------------------------------------------------------ read methods ------------------------------------------------ //
bool RasterReturnTime::readFromFile(std::string file){

	std::ifstream ins;
	ins.open(CodeUtils::Str2Char(file));
	if(ins.is_open()){
		std::vector<double> dv;
		std::vector<int> iv;
		std::string line = "0";
		std::string linepar = "";
		bool waspar;
		while(line.size() > 0){
			getline(ins,line);
			linepar = line;
			waspar = true;
			if( line.substr(0,2) == "//")
				continue;
			else if( CodeUtils::lineIs(line, "Freq pert minimum") ){	// todo table with new names (latex)
				freqPertMin = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line, "Freq pert maximum") ){
				freqPertMax = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line, "Phase pert minimum") ){
				phasePertMin = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line, "Phase pert maximum") ){
				phasePertMax = CodeUtils::readDouble(ins);
			}
			else if ( CodeUtils::lineIs(line, "Freq pert step") ){
				freqStepw = CodeUtils::readDouble(ins);
			}
			else if ( CodeUtils::lineIs(line, "Phase pert step") ){
				phaseStepw = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line, "Pertub delay") ){
				pertDelay = CodeUtils::readint(ins);
			}
			else if( CodeUtils::lineIs(line,"RT threshold high") ){
				thresholdHigh = CodeUtils::readDouble(ins);
				if( !isFixedThreshNum() )
					numOfThresh++;
				resetThresh();
			}
			else if( CodeUtils::lineIs(line,"RT threshold low") ){
				thresholdLow = CodeUtils::readDouble(ins);
				if( !isFixedThreshNum() )
					numOfThresh++;
				resetThresh();
			}
			else if( CodeUtils::lineIs(line,"Thresh spread") ){
//				threshSpr = readDouble(ins);	// todo compatibility
//				thresholds = linspace(threshold/threshSpr,threshold*threshSpr,
//						numOfThresh);
			}
			else if( CodeUtils::lineIs(line,"Num of thresh") ){
				numOfThresh = CodeUtils::readint(ins);
				setFixedThreshNum(true);
				resetThresh();
			}
			else if( CodeUtils::lineIs(line, "First pert node") ){
				firstPertNode = CodeUtils::readint(ins);
			}
			else if( CodeUtils::lineIs(line, "Last pert node") ){
				lastPertNode = CodeUtils::readint(ins);
			}
			else{
				waspar = false;
			}
			if( waspar )		// if the line was a parameter read by raster return time class
				std::cout << "read " << linepar << std::endl;
		}
	}

	return true;
}

// ------------------------------------------------------ simulation -------------------------------------------------- //
// TODO: constructor with param. other methods without param
// TODO: check for all instances of par.len... are they necessary and set it to numnodes


void RasterReturnTime::resetThresh(){
	thresholds.clear();
	thresholds = CodeUtils::darr2vec(
			CodeUtils::linspace(thresholdLow, thresholdHigh, numOfThresh), numOfThresh);

	for(int i = 0; i < rtmatsize; i++){
		delete[] rtmat[i];
	}
	delete[] rtmat;


	rtmat = new double*[numOfThresh];
	for(int i = 0; i < numOfThresh; i++){
		rtmat[i] = new double[pgrid->getNumNodes()];
	}

	for(int i = 0; i < rtmatsize; i++){
		delete[] trconv[i];
	}
	delete[] trconv;

	rtmatsize = numOfThresh;		// adjust rtmatsize for further resets

	trconv = new bool*[numOfThresh];
	for(int i = 0; i < numOfThresh; i++){
		trconv[i] = new bool[pgrid->getNumNodes()];
		for( int j = 0; j < pgrid->getNumNodes(); j++){
			trconv[i][j] = false;
		}
	}
}

/**
 * Determines the mean value and stores it in rtv for every node n.
 * If it does not converge the return time is infinity.
 * @param double* freq: The frequencies of all nodes at the given s
 * @param int s: current step. Together with par.rkseptwidth the current time is
 * 		determined
 * @param double* rtv: the return time vector/array where the mean return times
 * 		are saved.
 */
void RasterReturnTime::setMeanRet(double* rtv){

	double cm;
	for(int n = 0; n < pgrid->getNumNodes(); n++ ){	// for every node
		cm = 0;										// initialize current mean
		for(int thr = 0; thr < numOfThresh; thr++){ // for every threshold
			cm += rtmat[thr][n];					// add up the return times for calculating the mean
		}
		rtv[n] = cm/numOfThresh;					// and store that mean in given array for node n
	}

}

double RasterReturnTime::getThresh(int i){
		return thresholds.at(i);
}

/**
 * calculates if the return time has to be changed and stores the
 * new result in the par.rtmat array.
 * @param double* freq: the current frequency of all nodes
 * @param double t: simulation time of current step.
 * @param double thr: number of threshold to use
 */
void RasterReturnTime::setRetTime(double* freq, double t, int thr){

	for(int node = 0; node < pgrid->getNumNodes(); node++){
		if( std::fabs(freq[node]) < getThresh(thr) ){
			if( !trconv[thr][node] ){				// if frequency of node is not already converged
				rtmat[thr][node] = t;				// note current time as converging/return time
				trconv[thr][node] = true;			// and note that it is now converged
			}
		} else { 									// if nodes frequency is above threshold
			trconv[thr][node] = false;				// mark it as not converged
			rtmat[thr][node] = INFINITY;			// and set return time to infinity
		}
	}
}

/**
 * If the frequency of a node j is under threshold i rtmat[i][j] is set to t
 * @param freq: double array containing current frequencies.
 * @param t   : current time of simulation (double)
 */
void RasterReturnTime::setRetAllThresh(double* freq, double t){
	for(int thr = 0; thr < numOfThresh; thr++){			// for every threshold
		setRetTime(freq,t,thr);						// set the return time in the matrix par.rtmat
	}
}

bool RasterReturnTime::nextPertNode(){
	curPertNode = firstPertNode + nodeIterator;
	nodeIterator++;
	return curPertNode < lastPertNode;
}

// todo: in MRT und ENRT integrieren  // todo was passiert hier eigentlich?

/**
 * Stores the maximum perturbation for frequency and phase respectively
 */
//void PowerPerturbation::setMaxPert(Param& par, double* freq, double* phas, double t){
//
//	for( int i = 0; i < par.len; i++){
//		if( std::fabs(freq[i]) > std::fabs(par.getFreqmax().at(i)) ){
//			par.getFreqmax().at(i) = freq[i];
//			par.getFreqmaxTime().at(i) = t;
//		}
//		if( std::fabs(phas[i]) > std::fabs(par.getPhasmax().at(i)) ){
//			par.getPhasmax().at(i) = phas[i];
//			par.getPhasmaxTime().at(i) = t;
//		}
//	}
//
//}
/**
 * Stores the maximum elongation and the corresponding time while calculating the
 * trajectory
 */
void RasterReturnTime::maxPIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC){


//	double** freqTraj = new double*[par.RKsteps];	// storage for the frequency trajectory
//	double** phasTraj = new double*[par.RKsteps];	// storage for the phase trajectory
//	for(int i = 0; i < par.RKsteps; i++){			// initialize storage arrays
//		freqTraj[i] = new double[par.len];		// for frequency
//		phasTraj[i] = new double[par.len];		// and for phase
//	}

	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint;// = new double[par.len]; dont need own space	// temporary array for switching in1 and in2

//	double* rtv = new double[par.len];
	std::vector<double> ipm;						// vector storing dnode, rnode, dtheta, domega, maxFreq, maxFreqT, maxPhase, maxPhaseT
											// i = indices, p = perturbation, m = max response
//	int node = par.firstDistNode;				// investigate given node

	srand(par.getRandSeed());				// set random seed

	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = omegaIC[i];			// disturb node's frequency
		thetaIn1[i] = thetaIC[i];			// disturb node's phase
		thetaIn2[i] = 0;
		omegaIn2[i] = 0;
	}

//	par.resetMaxPert(); // todo revive

	for(int s = 1; s < par.RKsteps; s++){									// for all requested runge kutta steps

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

//		setMaxPert(par,omegaIn2,thetaIn2,s*par.rkStepWidth);				// store the perturbation if its bigger than former ones	// TODO needed here?
//		setRetAllThresh(par,omegaIn2,s);									// calculate all return time and set them in par.rtmat

//		std::copy(&omegaIn2[0],&omegaIn2[0]+par.len,						// store current frequencies
//				&freqTraj[s-1][0]);
//		std::copy(&thetaIn2[0],&thetaIn2[0]+par.len,						// store current phases
//				&phasTraj[s-1][0]);

		tempPoint = omegaIn1;		// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

//	setMeanRet(par,rtv);							// calculate the mean from the return times
	for(int n = 0; n < par.len; n++){				// for every reacting node
		ipm.push_back(curPertNode);					// push back disturbed node
		ipm.push_back(n);							// push back reacting node
		ipm.push_back(thetaIC[curPertNode]);		// push back phase disturbance at disturbed node
		ipm.push_back(omegaIC[curPertNode]);		// push back frequency disturbance at disturbed node
		ipm.push_back(par.getFreqmax().at(n));		// maximum frequency elongation at node n
		ipm.push_back(par.getFreqmaxTime().at(n));	// time of maximum frequency elongation at node n
		ipm.push_back(par.getPhasmax().at(n));		// maximum phase elongation at node n
		ipm.push_back(par.getPhasmaxTime().at(n));	// time of maximum phase elongation at node n
	}
	CodeUtils::vecToBinFileFast(ipm,par.getOfile("path")+"ipm/ipm"+par.getOfile("file"));	// write the vector to a binary file

//	par.writeMaxP();
// todo: put node number in name // todo: let them here?
//	arrayToBinFile(omegaIC,par.len, (par.getOfile("path")+"frequency/frequency"
//			+par.getOfile("file")));							// write the initial condition of frequency first
//	arrayToBinFile(omegaIn1,par.len, (par.getOfile("path")+"frequency/frequency"
//			+par.getOfile("file")));							// and then the resulting frequency after simulation time
//	arrayToBinFile(thetaIC,par.len, (par.getOfile("path")+"phase/phase"
//			+par.getOfile("file")));							// do same for phase
//	arrayToBinFile(thetaIn1,par.len, (par.getOfile("path")+"phase/phase"
//			+par.getOfile("file")));

//	for( int s = 0; s < par.RKsteps-1; s++){
//		arrayToBinFile(freqTraj[s],par.len,									// write stored frequency to binary file
//				par.getOfile("path")+"frequency/freqTraj"
//					+par.getOfile("file"));
//		arrayToBinFile(phasTraj[s],par.len,
//				par.getOfile("path")+"phase/phasTraj"						// write stored phase to binary file
//					+par.getOfile("file"));
//		delete[] freqTraj[s];												// clear frequency storage
//		delete[] phasTraj[s];												// clear phase storage
//	}

	ipm.clear();		// clear vector
	delete[] thetaIn1;	// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
//	delete[] rtv;
}


double RasterReturnTime::currentEnergy(double* freq, double* phas){		// todo move to power grid?
//	double T = 0; 		// todo revive
//	double V = 0;
//	linkline tmp;
//	for( int i = 0; i <len; i++){
//		T += (1.0)/2.0*Hpar.at(i)*pow(freq[i],2);
//		V -= P[i]*phas[i];	// or + ?
//		for( uint j = 0; j < K.at(i).size(); j++){
//			tmp = K.at(i).at(j);	// todo see harmonic energy ii,ji
//			V += tmp.couplingStrength*
//					(1 - cos(phas[i]-phas[j]));	// -tmp.angle
//		}
//	}
////		delete &tmp;
//	return T+V;	// Energy = kinetic + potential
	return 0;
}


double RasterReturnTime::harmonicEnergy(double* freq, double* phas){ 		// todo move to power grid?
//	double T = 0;
//	double V = 0;
//	double Vzero = 0;
//	int ii = 0;
//	int ji = 0;
//	linkline tmp;
//	for( int i = 0; i < len; i++){
//		T += Hpar.at(i)*pow(freq[i],2);	// times one half later
////			V -= P[i]*phas[i];
//		for( uint j = 0; j < K.at(i).size(); j++){
//			tmp  = K.at(i).at(j);
//			ii = tmp.thisNode;
//			ji = tmp.conNode;
////				V += 0.25*tmp.couplingStrength*cos(thstable[ii]-thstable[ji])*
////						pow( (phas[ii]-phas[ji]), 2);
////				Vzero += 0.25*tmp.couplingStrength*cos(thstable[ii]-thstable[ji])*
////						pow( (thstable[ii]-thstable[ji]), 2);
//			V += 0.25*tmp.couplingStrength*cos(thstable[i]-thstable[j]-tmp.angle)*
//					pow(( smallerAngle(phas[i]-phas[j])
//							-smallerAngle(thstable[i]-thstable[j]) ),2);
//			// if cos(th*) = 0 the potential is zero and the power grid
//			// is close to collapse even in steady state?
//		}
//	}
//	T *= 1.0/2.0;
////		delete &tmp;
//	return T+V-Vzero;
	return 0;	// todo revive
}

double RasterReturnTime::enT(double* freq, double* phas){ 	// todo move to power grid?
//	double T = 0;
//	double V = 0;
//	double Vzero = 0;
//	int ii = 0;
//	int ji = 0;
//	linkline tmp;
//	for( int i = 0; i < len; i++){
//		T += Hpar.at(i)*pow(freq[i],2);	// times one half later
//	}
//	T *= 1.0/2.0;
//	return T;//+V-Vzero;
	return 0; // todo revive
}

double RasterReturnTime::enV(double* freq, double* phas){ 	// todo move to power grid?
//	double T = 0;
//	double V = 0;
//	double Vzero = 0;
//	int ii = 0;
//	int ji = 0;
//	linkline tmp;
//	for( int i = 0; i < len; i++){
//		for( uint j = 0; j < K.at(i).size(); j++){
//			tmp  = K.at(i).at(j);
//			ii = tmp.thisNode;
//			ji = tmp.conNode;
//			V += 0.25*tmp.couplingStrength*cos(thstable[i]-thstable[j]-tmp.angle)*
//					pow(( smallerAngle(phas[i]-phas[j])
//							-smallerAngle(thstable[i]-thstable[j]) ),2);
//			// if cos(th*) = 0 the potential is zero and the power grid
//			// is close to collapse even in steady state?
//		}
//	}
////		T *= 1.0/2.0;
////		delete &tmp;
//	return V;//T+V-Vzero;
	return 0; 	// todo revive
}



// <<<<<<<<<<<<<<<<<<<<<<<<<<<<< todo energy stuff >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
void RasterReturnTime::setEnergyRet(Param& par, double* freq, double* phas, int s){

	double t = ((double)s) * par.rkStepWidth;

	//	if( std::fabs(par.currentEnergy(freq,phas)) < par.getEnThresh() ){
	//		if( !par.enconv ){
	//			par.enconv = true;
	//			par.enrt = t;
	//		}
	//	} else {
	//		par.enconv = false;
	//		par.enrt = INFINITY;
	//	}

	if( std::fabs(harmonicEnergy(freq,phas)) < par.getEnThresh() ){
		if( !par.henconv ){
			par.henconv = true;
			par.henrt = t;
		}
	} else {
		par.henconv = false;
		par.henrt = INFINITY;
	}
}


/**
 * checks if all elements of vec are lower than konvN
 */
bool RasterReturnTime::vecConvergence(double* vec, double konvN, int len){

	for(int i = 0; i < len; i++){
		if( fabs(vec[i]) > konvN || std::isnan(vec[i]) ){
			return false;
		}
	}
	return true;
}

void RasterReturnTime::countVecConv(double* vec, int pertNode, Param* par){

	for(int i = 0; i < par->len; i++){
//		if((  fabs(vec[i]) < par->convNum && !( std::isnan(vec[i]) )  )){
			par->konvOut[pertNode][i] += (  fabs(vec[i]) < par->convNum && !( std::isnan(vec[i]) )  );
//		}
	}
}


/** // todo refresh this method
 * Monte Carlo sampling of initial conditions with return time method.
 * calculates the return time of a trajectory as an average
 * over a number of thresholds given in par. If the trajectory
 * does not converge -1 is stored.
 * @param Param par: The parameter object
 * @param RungeKutta rk: The RungeKutta object which has the
 * 		calculation method of the swing equation
 */ /*
void RasterReturnTime::MonteCarloRetTimes(Param& par, RungeKutta& rk){

	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint = new double[par.len];	// temporary array for switching in1 and in2

	double* rtv = new double[par.len];
	std::vector<double> ijr;
	std::vector<double> energyf;

										// ==== initialize fixed point vector ====
	delete[] par.thstable;				// clear fixed point vector
	par.thstable = new double[par.len];	// create a new one with the length given in param
	for(int i = 0; i < par.len ; i++){	// and fill every entry
		par.thstable[i] = 0;			// initially with zero
		thetaIn1[i] = 0;
		thetaIn2[i] = 0;
		omegaIn1[i] = 0;
		omegaIn2[i] = 0;
		tempPoint[i] = 0;
	}

	int node = par.firstDistNode;						// investigate given node
//	double t = 0;										// simulation time rk step width * rk step

	srand(par.getRandSeed());							// set random seed

//	while( generateNet(par) ){							// For all nets which should be investigated;
														// in this method the fixed point is calculated.
		for(int draw = 0; draw < par.draws; draw++){	// For every Monte Carlo draw

			for(int i = 0; i < par.len; i++){			// start at fixed point
				omegaIn1[i] = 0;						// which is at frequency equal to zero
				thetaIn1[i] = par.thstable[i];			// and phases calculated by Newton raphson method
			}

			for(int s = 1; s < par.RKsteps; s++){
				if( (s-1) == pertDelay){								// after given delay time 	//disturb system so oder so
//					if( par.isPertVecGiven()){
//						for(int pnode = 0; pnode < par.len; pnode++){
//							omegaIn1[pnode] = par.pertVecOm.at(pnode);
//							thetaIn1[pnode] = par.pertVecTh.at(pnode);
//						}
//					} else {
						omegaIn1[node] = CodeUtils::randD(freqPertMin,freqPertMax);	// disturb node's frequency
						thetaIn1[node] = par.thstable[node] +
								CodeUtils::randD(phasePertMin,phasePertMax);		// disturb node's phase
//					}
				}

				rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

				setRetAllThresh(par,omegaIn2,s);									// calculate all return time and set them in par.rtmat

				tempPoint = omegaIn1;		// switch in and out
				omegaIn1 = omegaIn2;
				omegaIn2 = tempPoint;
				tempPoint = thetaIn1;
				thetaIn1 = thetaIn2;
				thetaIn2 = tempPoint;
			}	// after all runge kutta steps are made per draw and per network

			setMeanRet(par,rtv);
			for(int n = 0; n < par.len; n++){
				ijr.push_back(node);		// push back disturbed node
				ijr.push_back(n);			// push back reacting node
				ijr.push_back(rtv[n]);		// push back return time for that constellation
			}
			CodeUtils::vecToBinFile(ijr,par.getOfile("path")+"ijr"+par.getOfile("file"));
			ijr.clear();

		}	// after all draws
//	}		// after all networks

	delete[] thetaIn1;	// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
	delete[] tempPoint;
	delete[] rtv;
}
*/
void RasterReturnTime::enrtIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC){

	bool savetraj = par.saveStepTraj<par.RKsteps;

	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint;							// temporary array for switching in1 and in2;  does not need own space

	std::vector<double> freqtr;
	std::vector<double> phastr;

	double* rtv = new double[par.len];
	std::vector<double> eijtor;					// vector storing dnode, dtheta, domega and energy return time alternating
	std::vector<double> heijtor;				// vector storing dnode, dtheta, domega and harmonic energy return time alternating
	std::vector<double> energyTraj;				// vector storing the trajectory of the energy
//	int node = par.firstDistNode;				// investigate given node

	srand(par.getRandSeed());					// set random seed


	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = omegaIC[i];				// disturb node's frequency
		thetaIn1[i] = thetaIC[i];				// disturb node's phase
		thetaIn2[i] = 0;
		omegaIn2[i] = 0;
	}

	for(int s = 1; s < par.RKsteps; s++){									// for all requested runge kutta steps

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

//		setRetAllThresh(par,omegaIn2,s);									// calculate all return time and set them in par.rtmat
		setEnergyRet(par,omegaIn2,thetaIn2,s);
		energyTraj.push_back(harmonicEnergy(omegaIn2,thetaIn2));

		if ( (s-1) % par.saveStepTraj == 0 && savetraj){	// todo remove?
			for( int i = 0 ; i < par.len; i ++){
				freqtr.push_back(omegaIn2[i]);
				phastr.push_back(thetaIn2[i]);
			}
		}

		tempPoint = omegaIn1;		// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;
	}

	heijtor.push_back(curPertNode);				// push back disturbed node
	heijtor.push_back(0);						// push back dummy zero to keep ijtor-format
	heijtor.push_back(thetaIC[curPertNode]);	// push back phase disturbance at disturbed node
	heijtor.push_back(omegaIC[curPertNode]);	// push back frequency disturbance at disturbed node
	heijtor.push_back(par.henrt);				// push back energy return time for that constellation

	CodeUtils::vecToBinFile(heijtor,par.getOfile("path")+"heijtor/heijtor"+par.getOfile("file"));	// write the vector to a binary file
	CodeUtils::vecToBinFile(energyTraj,par.getOfile("path")+"etraj/etraj"+par.getOfile("file"));	// write the vector to a binary file



	if( savetraj){
		CodeUtils::vecToBinFile(freqtr, (par.getOfile("path")+"frequency/frequencyTraj"
				+par.getOfile("file")));					// and then the resulting frequency after simulation time
		CodeUtils::vecToBinFile(phastr, (par.getOfile("path")+"phase/phaseTraj"
				+par.getOfile("file")));					// do same for phase
	}

// todo: put node number in name
	CodeUtils::arrayToBinFile(omegaIC,par.len, (par.getOfile("path")+"frequency/frequency"
			+par.getOfile("file")));					// write the initial condition of frequency first
	CodeUtils::arrayToBinFile(omegaIn1,par.len, (par.getOfile("path")+"frequency/frequency"
			+par.getOfile("file")));					// and then the resulting frequency after simulation time
	CodeUtils::arrayToBinFile(thetaIC,par.len, (par.getOfile("path")+"phase/phase"
			+par.getOfile("file")));					// do same for phase
	CodeUtils::arrayToBinFile(thetaIn1,par.len, (par.getOfile("path")+"phase/phase"
			+par.getOfile("file")));

	heijtor.clear();	// clear vector
	energyTraj.clear();
	delete[] thetaIn1;	// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
	delete[] rtv;
}

void RasterReturnTime::mrtIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC){


	double* thetaIn1 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t
	double* thetaIn2 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t+dt
	double* omegaIn1 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t
	double* omegaIn2 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t+dt
	double* tempPoint;										// temporary array for switching in1 and in2 // does not need own memory

	double* rtv = new double[pgrid->getNumNodes()]; 		// return time vector storing averaged end results
	std::vector<double> ijtor;								// vector storing disturbed node, reacting node, phase and frequency perturbation and return time alternating(?)

	for(int i = 0; i < pgrid->getNumNodes(); i++){
		omegaIn1[i] = omegaIC[i];						// disturb node's frequency
		thetaIn1[i] = thetaIC[i];						// disturb node's phase
		thetaIn2[i] = 0;
		omegaIn2[i] = 0;
	}

	for(int s = 1; s < par.RKsteps; s++){				// for all requested Runge Kutta steps

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1,
				omegaIn2, thetaIn2);					// calculate the swing equation

		setRetAllThresh(omegaIn2,
				((double)s) * par.rkStepWidth);			// calculate all return times and set them in par.rtmat

		tempPoint = omegaIn1;							// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

	}
	setMeanRet(rtv);								// calculate the mean from the return times
	for(int n = 0; n < pgrid->getNumNodes(); n++){	// for every reacting node
		ijtor.push_back(curPertNode);				// push back disturbed node
		ijtor.push_back(n);				        	// push back reacting node
		ijtor.push_back(thetaIC[curPertNode]);		// push back phase disturbance at disturbed node
		ijtor.push_back(omegaIC[curPertNode]);		// push back frequency disturbance at disturbed node
		ijtor.push_back(rtv[n]);		        	// push back return time for that constellation
	}
	CodeUtils::vecToBinFileFast(ijtor,par.getOfile("path")
			+"ijtor/ijtor"+par.getOfile("file"));		// write the vector to a binary file

// to do: put node number in name + "-" + CodeUtils::i2Str(curPertNode) ?? why? special case?
	CodeUtils::arrayToBinFile(  omegaIC,pgrid->getNumNodes(),
			(par.getOfile("path")+"frequency/frequency"
			+par.getOfile("file")  )  );					// write the initial condition of frequency first
	CodeUtils::arrayToBinFile(omegaIn1,pgrid->getNumNodes(),
			(par.getOfile("path")+"frequency/frequency"
			+par.getOfile("file")));					// and then the resulting frequency after simulation time
	CodeUtils::arrayToBinFile(thetaIC,pgrid->getNumNodes(),
			(par.getOfile("path")+"phase/phase"
			+par.getOfile("file")));					// do same for phase
	CodeUtils::arrayToBinFile(thetaIn1,pgrid->getNumNodes(),
			(par.getOfile("path")+"phase/phase"
			+par.getOfile("file")));

	ijtor.clear();								// clear vector
	delete[] thetaIn1;							// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
	delete[] rtv;
}

/**
 * calculates and stores the trajectory for a given initial condition
 */
void RasterReturnTime::trajIC(Param& par, RungeKutta& rk, double* thetaIC, double* omegaIC){


	double** freqTraj = new double*[par.RKsteps];	// storage for the frequency trajectory
	double** phasTraj = new double*[par.RKsteps];	// storage for the phase trajectory


	double* thetaIn1 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t
	double* omegaIn2 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t+dt
	double* tempPoint; 										// temporary array for switching in1 and in2

	for(int i = 0; i < pgrid->getNumNodes() ; i++){			// and fill every entry
		thetaIn2[i] = 0;									// of the first output array (is later switched with input)
		omegaIn2[i] = 0;									//
	}
	for(int i = 0; i < par.RKsteps; i++){					// initialize storage arrays
		freqTraj[i] = new double[pgrid->getNumNodes()];		// for frequency
		phasTraj[i] = new double[pgrid->getNumNodes()];		// and for phase
	}

	srand(par.getRandSeed());					// set random seed

	for(int i = 0; i < pgrid->getNumNodes(); i++){
		omegaIn1[i] = omegaIC[i];				// disturb node's frequency (in input array)
		thetaIn1[i] = thetaIC[i];				// disturb node's phase
	}

	for(int s = 1; s < par.RKsteps; s++){									// for all requested runge kutta steps

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		std::copy(&omegaIn2[0],&omegaIn2[0]+pgrid->getNumNodes(),			// store current frequencies
				&freqTraj[s-1][0]);
		std::copy(&thetaIn2[0],&thetaIn2[0]+pgrid->getNumNodes(),			// store current phases
				&phasTraj[s-1][0]);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for( int s = 0; s < par.RKsteps-1; s++){
		CodeUtils::arrayToBinFile(freqTraj[s],pgrid->getNumNodes(),			// write stored frequency to binary file
				par.getOfile("path")+"frequency/freqTraj"
					+par.getOfile("file"));
		CodeUtils::arrayToBinFile(phasTraj[s],pgrid->getNumNodes(),
				par.getOfile("path")+"phase/phasTraj"						// write stored phase to binary file
					+par.getOfile("file"));
		delete[] freqTraj[s];												// clear frequency storage
		delete[] phasTraj[s];												// clear phase storage
	}

	delete[] thetaIn1;														// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;

}

/**
 * iterates through all requested initial conditions and decides
 * dependent on the mode what to calculate with them.
 * Currently the following modes exist:
 * * ENRT : return time for current IC determined with the "energy" function
 * * RMRT : return time for current IC determined by the frequency
 * * MaxP : stores maximum elongation for phase and frequency and their times
 * 			of occurrence after a perturbation with current IC
 * * Traj : stores the trajectory
 *
 */
void RasterReturnTime::rasterRT(Param& par, RungeKutta& rk){

	// todo create method creating necessary files and consider folder RRT/ijtor etc
	CodeUtils::cdir("heijtor");		// create a folder heijtor for corresponding data if it not already exist
	CodeUtils::cdir("frequency");
	CodeUtils::cdir("ijtor");
	CodeUtils::cdir("phase");
	CodeUtils::cdir("etraj");
	CodeUtils::cdir("ipm");

	// prepare grid // take grid here todo grid is in constructor what to do here?

	resetThresh();											// initialize the thresholds and the arrays storing return times

	double* voltPhaseSS = pgrid->getVoltAngSS()->data();	// get steady state voltage angles from power grid
	double* freqIC = new double[pgrid->getNumNodes()];		// allocate double array for initial conditions for voltage frequency
	double* phaseIC = new double[pgrid->getNumNodes()]; 	// and phase
	double curFreq = freqPertMin; 							// current initial condition of frequency
	double curPhase = phasePertMin;							// and phase


	while( nextPertNode() ){								// for all nodes of interest
		while(curFreq < freqPertMax){ 						// for all frequencies (omega) and hence all initial conditions


			for(int i = 0; i < pgrid->getNumNodes(); i++){	// for every node
				freqIC[i] = 0; 								// reset frequency to steady state (all zero)
				phaseIC[i] = voltPhaseSS[i];				// reset voltage phases to steady state
				if( i == curPertNode ){						// for perturbed node
					freqIC[i] = curFreq;					// set requested perturbance voltage frequency
					phaseIC[i] += curPhase;					// and angle
				}		// todo include disturbance vector here
			}

			if(par.isModeERT()){							// energy return times	// todo check new location
				enrtIC(par,rk,phaseIC,freqIC);
			}
			if(par.isModeRMRT()){							// if raster return times with multiple thresholds is chosen
				mrtIC(par,rk,phaseIC,freqIC);				// calculate the return time for current initial condition
			}
			if(par.isModeMaxp()){						// todo check
				maxPIC(par,rk,phaseIC,freqIC);
			}
			if(par.isModeStraj()){							// if a calculation of the trajectory is requested	// todo check
				trajIC(par,rk,phaseIC,freqIC);
			}

			curPhase += phaseStepw;
			if( curPhase >= phasePertMax){
				curPhase = phasePertMin;
				curFreq += freqStepw;
			}
		}
	}

}

// ------------------------------------------------------- get&set ---------------------------------------------------- //
double RasterReturnTime::getFreqPertMax() const {
	return freqPertMax;
}

void RasterReturnTime::setFreqPertMax(double freqPertMax) {
	this->freqPertMax = freqPertMax;
}

double RasterReturnTime::getFreqPertMin() const {
	return freqPertMin;
}

void RasterReturnTime::setFreqPertMin(double freqPertMin) {
	this->freqPertMin = freqPertMin;
}

double RasterReturnTime::getFreqStepw() const {
	return freqStepw;
}

void RasterReturnTime::setFreqStepw(double freqStepw) {
	this->freqStepw = freqStepw;
}

int RasterReturnTime::getNumOfThresh() const {
	return numOfThresh;
}

void RasterReturnTime::setNumOfThresh(int numOfThresh) {
	this->numOfThresh = numOfThresh;
}

double RasterReturnTime::getPertDelay() const {
	return pertDelay;
}

void RasterReturnTime::setPertDelay(double pertDelay) {
	this->pertDelay = pertDelay;
}

double RasterReturnTime::getPhasePertMax() const {
	return phasePertMax;
}

void RasterReturnTime::setPhasePertMax(double phasePertMax) {
	this->phasePertMax = phasePertMax;
}

double RasterReturnTime::getPhasePertMin() const {
	return phasePertMin;
}

void RasterReturnTime::setPhasePertMin(double phasePertMin) {
	this->phasePertMin = phasePertMin;
}

double RasterReturnTime::getPhaseStepw() const {
	return phaseStepw;
}

void RasterReturnTime::setPhaseStepw(double phaseStepw) {
	this->phaseStepw = phaseStepw;
}

double RasterReturnTime::getThresholdHigh() const {
	return thresholdHigh;
}

void RasterReturnTime::setThresholdHigh(double threshold) {
	this->thresholdHigh = threshold;
}

double RasterReturnTime::getThresholdLow() const {
	return thresholdLow;
}

void RasterReturnTime::setThresholdLow(double threshold) {
	this->thresholdLow = threshold;
}

double RasterReturnTime::getThreshSpread() const {
	return threshSpread;
}

void RasterReturnTime::setThreshSpread(double threshSpread) {
	this->threshSpread = threshSpread;
}

bool RasterReturnTime::isFixedThreshNum() const {
	return fixedThreshNum;
}

void RasterReturnTime::setFixedThreshNum(bool fixedThreshNum) {
	this->fixedThreshNum = fixedThreshNum;
}

// ----------------------------------------------------- persistence -------------------------------------------------- //
std::string RasterReturnTime::toString(){
	std::stringstream outfile;
	outfile << "=============== Raster return time Input ================" 	<< std::endl;
	outfile << "----------------- Algorithm Parameters ------------------" 	<< endl;
	outfile << "FirstPertNode: " 		<< firstPertNode  	<< endl;
	outfile << "LastPertNode: " 		<< lastPertNode		<< endl;
	outfile << "FreqPertMin: " 			<< freqPertMin		<< endl;
	outfile << "FreqPertStep: " 		<< freqStepw		<< endl;
	outfile << "FreqPertMax: " 			<< freqPertMax		<< endl;
	outfile << "PhasePertMin: " 		<< phasePertMin		<< endl;
	outfile << "PhasePertStep: " 		<< phaseStepw		<< endl;
	outfile << "PhasePertMax: " 		<< phasePertMax		<< endl;
	outfile << "PertDelay: " 			<< pertDelay		<< endl;
	outfile << "ThresholdHigh: " 		<< thresholdHigh	<< endl;
	outfile << "ThresholdLow: " 		<< thresholdLow		<< endl;
	outfile << "ThresholdNumber: " 		<< numOfThresh		<< endl;
//	outfile << "TurbineNodes: " 							<< endl;
//	outfile << CodeUtils::ivec2str(turNodes);
//	outfile 												<< endl;
//	outfile << "------------------ Turbine Parameters -------------------" 	<< endl;
//	outfile << "Prmratio: " 			<< turbinePrmratio	<< endl;
//	outfile << "PowerCurveExponent: " 	<< turbineExp 		<< endl;
//	outfile 												<< endl;
//	outfile << "-------------------- Surviveability ---------------------" 	<< endl;
//	outfile << "Threshold: " 			<< survthresh		<< endl;
//	outfile << "InitializationSteps: "	<< survInitSteps	<< endl;
//	outfile << "Interval: "				<< survInterval		<< endl;
//	outfile 												<< endl;

	outfile << endl;
	return outfile.str();
}



// ==================================================== variables ===================================================== //
// =================================================== constructors =================================================== //
// ====================================================== methods ===================================================== //
// --------------------------------------------------- array methods -------------------------------------------------- //
// ------------------------------------------------- directory methods ------------------------------------------------ //
// ------------------------------------------------ mathematical methods ---------------------------------------------- //
// --------------------------------------------------- string methods ------------------------------------------------- //
// ------------------------------------------------------- get&set ---------------------------------------------------- //
// ------------------------------------------------------ simulation -------------------------------------------------- //




//			else if ( CodeUtils::lineIs(line, "Omega start") ){
////					getline(ins,line);
//				delete[] om0;
//				om0 = new double[1];
//				om0[0] = readDouble(ins);
//			}
//			else if ( CodeUtils::lineIs(line, "Omega end") ){
//				delete[] ome;
////					getline(ins,line);
//				ome = new double[1];
//				ome[0] = readDouble(ins);
//			}
//			else if ( CodeUtils::lineIs(line, "Theta start") ){
////					getline(ins,line);	// to do change in other version
//				delete[] th0;
//				th0 = new double[1];
//				th0[0] = readDouble(ins);
//			}
//			else if ( CodeUtils::lineIs(line, "Theta end") ){
//				delete[] the;
//				getline(ins,line);
//				the = new double[1];
//				the[0] = readDouble(ins);
//			}
