/*
 * StochasticInput.cpp
 *
 *  Created on: Feb 27, 2019
 *      Author: mawolff
 */

#include "StochasticInput.h"
#include "CodeUtils.h"

#include <tgmath.h> 	// sqrt, fabs

// =================================================== constructors =================================================== //
StochasticInput::StochasticInput(PowerGrid* pg, Param* pars) {
	this->pgrid = pg; 			// the power grid for which the stochastic analysis is made
	this->par = pars;			// the parameters and mode for analysis

	MODE_MULTI_TURB = false;	// assume only one turbine if nothing else requested. see readFromFile/Turbine nodes
	writeStoch = false;			// do not write anything to hard drive unasked
	pratedGiven = false;		// as long as no rated power is given, calculate it
	stochFileName = "Pseries";	// a default name to understand what is the meaning of the file
	multipleWindSer = false;	// if not given otherwise, use the wind series as one part
//	turIndepWS = 1;				// as default, use same given wind series for all turbine nodes

	exceedthresh = 0.01;		// meaningful guess
	survall = 0;				// initialize zero
	survInitSteps = 0;			// assume no initialization time (begin in stationary regime)
	survInterval = 100;			// guess
	balanceFactor = 1;			// assume no balance needed at the beginning (should be set later anyway before used)

	vrated = 12.5;				// typical rated wind speed
	vmean = 8;					// mean wind speed in central Europe

	freqHistBinWidth = 0.005;	// resolution of frequency histogram. Default: 5mHz angular frequency (about 1mHz real frequency)
	setFreqHistBinNum(5000);	// set number of bins and corresponding maximum and minimum frequency for histogram

	fluc_share = false;			// false: nodes are either stochastic or deterministic. true: can be both
	balance_mean = true;		// hard coded: true: the missing power at mean wind speed is compensated by all conventional gens

	freqIncrHistBinWidth = 0.005;	// save resolution as freqHistBinWidth

	windD1 = 0.054;				// default value from M. Brune (Bachelor thesis)

}

/**
 * it is not StochasticInput s concern to delete the power grid or the function
 */
StochasticInput::~StochasticInput() {
	for(uint i = 0; i < turNodeMat.size(); i++){
		turNodeMat.at(i).clear();
	}
	turNodeMat.clear();
	clearPSeries();
}

// ====================================================== methods ===================================================== //
// -------------------------------------------------------- read ------------------------------------------------------ //

/**
 * lines are generated by coupling mag or admittance mag
 * all other function just use these lines
 */
bool StochasticInput::readFromFile(std::string pfile){

	std::ifstream ins; 						// allocate input file stream
	ins.open(CodeUtils::Str2Char(pfile));	// open file
	if(ins.is_open()){ 						// if instream is working
		std::cout << "-------------------- StochasticInput reading "
				  << " --------------------" << std::endl;
		std::vector<double> dv; 			// a double vector for temporary storing doubles
//		std::vector<int>* iv;				// a double vector for temporary storing integers
		bool waspar;						// indicates whether the current line was a parameter read by power grid class
		std::string line = "0";				// the file is loaded line by line into this string
		std::string linepar = "";			// storing the parameter name to inform user that parameter was read
		bool skipline = false;				// if line was read by method and must not be read here again
		while(line.size() > 0){ 			// as long the file has still lines left
			if( !skipline ){
				getline(ins,line); 			// read next line from stream
			} else { 						// if line was skipped
				skipline = false;			// do not skip the next line
			}
			linepar = line; 				// save line for informing user later
			waspar = true;					// if not negated later, the line was a parameter of interest
			if( line.substr(0,2) == "//") 	// if line is a comment from user
				continue; 					// skip this line
			// --------------------------------------------------------- turbines -------------------------------------------------------- //
			else if( CodeUtils::lineIs(line , "Turbine nodes" ) ){		// explicit definition of turbine nodes. It is possible to have
				turNodeMat = CodeUtils::readIntMatArb(ins, line);		// read turbine node configurations
				skipline = true;


			}
			else if( CodeUtils::lineIs(line,"Turbine prated") ){		// maximum stochastic power at this node reached at rated wind speed
				CodeUtils::readDVec(ins, &Prated);
				setPratedGiven(true);									// use given maximum power instead of produced power at this node
			}
			else if( CodeUtils::lineIs(line,"Turbine vrated") ){ 		// rated wind speed
				vrated = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line,"Renewable share") ){ 		// share of renewable power at all nodes
				CodeUtils::readDVec(ins, &powerRenewShareInit);
				setFlucShare(true); 									// set flag and ...
			}
			else if( CodeUtils::lineIs(line,"Turbine vmean") ){ 		// mean annual wind speed (or at least over long time >month)
				vmean = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line,"Survive threshold") ){		// threshold for measuring exceedance probability
				exceedthresh = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line,"Survive init steps") ){	//
				survInitSteps = CodeUtils::readint(ins);
			}
			else if( CodeUtils::lineIs(line,"Survive interval") ){ 		// measure exceedance probability in intervals separately
				survInterval = CodeUtils::readint(ins);
			}
			else if( CodeUtils::lineIs(line, "Write stoch")){			// if the stochastic power series should be printed to a file
				int yn = CodeUtils::readint(ins);
				if(yn == 0){
					writeStoch = false;
				} else {
					writeStoch = true;
				}
			}
			else if( CodeUtils::lineIs(line,"Freq hist bin width") ){	// resolution of frequency histogram
				freqHistBinWidth = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line,"Freq hist bin num") ){	 	// set number of bins and corresponding maximum and minimum frequency for histogram
				setFreqHistBinNum(CodeUtils::readint(ins));
			}
			else if( CodeUtils::lineIs(line,
					"Freq incr hist bin width") ){						// resolution of frequency increment histogram
				freqIncrHistBinWidth = CodeUtils::readDouble(ins);
			}
			else if( CodeUtils::lineIs(line,"Freq time lags") ){ 		// a vector of time lags in seconds for the frequency increment histogram
				CodeUtils::readDVec(ins, &timeLags);
				calcTLagSteps(timeLags);
			}
			else if ( CodeUtils::lineIs(line, "Stoch filename") ){ 		// the name of the file to print the stochastic power series in
				getline(ins,line);
				stochFileName = line;
			}
			else if( CodeUtils::lineIs(line,"Wind series") ){ 			// a series of wind velocities to interpolate when corresponding mode is active
				windSpeedMeasured = CodeUtils::readDMatArb(ins, line);
				skipline = true;
			}
			else if( CodeUtils::lineIs(line,"Wind series ext" )){
				getline(ins,line);
				windSpeedMeasured = CodeUtils::readExtDMat(line);		// copy (constructor) external values

			}
//			else if( CodeUtils::lineIs(line,"Turbine ind ws") ){ 		//	number of turbine nodes with independent wind series inputs
//				turIndepWS = CodeUtils::readint(ins);
//			}
			else if( CodeUtils::lineIs(line,"Wind mult") ){ 			// the wind series should be divided into parts of RK steps*RK width
				setMultipleWindSer(true);
			}
			else if( CodeUtils::lineIs(line,"Wind D1") ){ 				// Drift coefficient for stochastic interpolation process (Ornstein Uhlenbeck)
				windD1 = CodeUtils::readDouble(ins);
			}
			else { 														// if none of the indicator names was present
				waspar = false; 										// the line did not contain a parameter to be read by this class
			}

			if( waspar )				// if the line was a parameter read by stochastic input class
				std::cout << "read " << linepar << std::endl;			// print that it was read
			dv.clear();													// clear temporary vector
		}
	}
	return true;
}


// ---------------------------------------------------- stochastic main ----------------------------------------------- //
/**
 * initializes the stochastic input simulation and finally calls evaluation method.
 */
void StochasticInput::stochSwing(Param& par, RungeKutta& rk){


	int windParts = 1;													// number of parts the wind series is divided into
	int windPartLen = windSpeedMeasured.at(0).size();							// length of every single part
	if( multipleWindSer ){												// if multiple parts are requested
		getPartsAndLen(windParts,windPartLen,
				&(windSpeedMeasured.at(0)) ); 							// determine in how many parts the wind series can be divided
	}
	std::string filepure = par.filenum;									// filename as given from input file
	std::string filew = par.filenum;									// filename when only one wind series analyzed
	for(int w = 0; w < windParts; w++){ 								// for every part

		getCurrentWindPart(w, windPartLen,
				&windSpeedMeasured, &windSpeedCur); 					// get the current series used in stochasticPseries()
		if(multipleWindSer) 											// if multiple parts are taken
			filew = filepure + "w" + CodeUtils::d2Str(w);				// attach the number of wind series part to output files

		for(uint tnc = 0; tnc < turNodeMat.size(); tnc++){				// for every turbine node configuration
			par.filenum = filew + "tn" + CodeUtils::d2Str(tnc);			// attach the number of the turbine node configuration to output files
			prepareTurbines(par, rk, turNodeMat.at(tnc), w);			// creates the power input series for the turbine nodes configuration

			calcBalanceFactor(turNodeMat.at(tnc)); 						// tunes all conventional generators corresponding to the current wind situation

			stochEvaluation(par,rk,turNodeMat.at(tnc)); 				// run evaluation of stochastic input depending on the mode
		}

	}


}
// checked (v2.004): Multi turbine and single turbine same results when using one turbine (the same of course)

// ---------------------------------------------------- process data -------------------------------------------------- //

/**
 * depending on single or multi turbine mode prepare the stochastic input of the wind
 * turbines and run the evaluation with this input
 *
 * @param Param& par: parameter struct
 * @param RungeKutta& rk: the RungeKutta solver
 * @param vector<int> turNodes : the numbers of the turbines for current simulation
 * @param int w: the current wind part number
 */
void StochasticInput::prepareTurbines(Param& par, RungeKutta& rk, std::vector<int> turNodes, int w){

	pgrid->resetPowerMech();
	powerRenewShare.clear();
	for(uint i = 0; i < powerRenewShareInit.size(); i++){
		powerRenewShare.push_back(powerRenewShareInit.at(i));
	}

	if( fluc_share){ 												// if nodes are not completely renewable (but shared conventional/renewable)
		if( powerRenewShare.size() == turNodes.size() ){			// if the fluctuating share is given for every renewable energy source
			powerRenewShare.clear(); 								//
			for(int i = 0; i < pgrid->getNumNodes(); i++){			// fill the vector with zeros
				powerRenewShare.push_back(0);						// the renewable share is zero
			}														//
			for(uint t = 0; t < turNodes.size(); t++){ 				// except for turbine nodes where it is given
				powerRenewShare.at(turNodes.at(t)) = powerRenewShareInit.at(t);
			}
		} else if( powerRenewShare.size() < (uint)pgrid->getNumNodes() ){	// if renewable share is given for most nodes
			for(uint i = powerRenewShare.size(); i < (uint)pgrid->getNumNodes(); i++){ 	// assume that these are those labelled from one
				powerRenewShare.push_back(0);						// and add the rest as zero
			}
		}
	} else { 													// if generators are either completely renewable or conventional
		powerRenewShare.clear();								//
		for(int i = 0; i < pgrid->getNumNodes(); i++){			//
			powerRenewShare.push_back(pgrid->getPowerMech(i));	// the renewable share is the total power at that node
		}
	}

	// todo set the reference power for all not-turbine nodes!!! BUG and crash DANGER!!
	//	(if given size < size(turnodes)) do method checkRated or similar
	if( ! isPratedGiven() ){ 										// if rated power not given
		Prated.clear();
		double Prat;												// it is constructed
		for( int i = 0; i < pgrid->getNumNodes(); i++){				// in such a way that it produces at average wind speed the same power
			Prat = pow( vrated / vmean, 3) * powerRenewShare.at(i);	// as the replaced conventional generator
			Prated.push_back(Prat);
		}

	}

	clearPSeries();													// clear stochastic power series for every node
	for( uint tn = 0; tn < turNodes.size(); tn++){ 					// generate a power series for every stochastic node
		std::vector<double> *np = new std::vector<double>;			// create new vector
		Pseries.push_back(*np); 									// copy new vector
		delete np; 													// delete local instance
		stochasticPseries(turNodes.at(tn), &(windSpeedCur.at(tn)),	// create corresponding time series using
				&(Pseries.at(Pseries.size()-1)) , par);				// stochastic node turNodes.at(tn) and windSpeedCur.at(tn)
		if( writeStoch ){ 											// if user wants the time series printed
			CodeUtils::cpthdir(par.getOfile("path"),"pseries");		// make directory for power input series
			std::string num = CodeUtils::d2Str(turNodes.at(tn));	// generate fitting name
			CodeUtils::vecToBinFile(Pseries.at(Pseries.size()-1), 	// and print them to file
					par.getOfile("path") + "pseries/"
					+ stochFileName + "tn" + num + "w"
					+ CodeUtils::d2Str(w));
		}
	}

}

/**
 * starts the simulation (rk iteration) depending on mode
 */
void StochasticInput::stochEvaluation(Param& par, RungeKutta& rk, std::vector<int> turNodes){
	if( par.isModeStochCASC() ){ 									// if corresponding mode activated
		CodeUtils::cpthdir(par.getOfile("path"), "stochcas");
		stochCasc(par, rk, turNodes); 								// run simulation and check if transmission lines are overloaded
	}
	if( par.isModeStochLf() ){
		CodeUtils::cpthdir(par.getOfile("path"), "flow");
		stochFlows(par,rk,turNodes);
	}
	if( par.isModeSurvivall() ){ 									// if exceedance probability of the whole grid is requested (all nodes below threshold)
		CodeUtils::cpthdir(par.getOfile("path"), "surv");
		CodeUtils::cpthdir(par.getOfile("path"), "hist");
		CodeUtils::cpthdir(par.getOfile("path"), "flanks");
		ExceedProbAll(par,*pgrid,rk,turNodes);						// run corresponding simulation
	}
	if( par.isModeSurv() ){ 										// if exceedance probability  should be measured for every node individually
		CodeUtils::cpthdir(par.getOfile("path"), "surv");
		CodeUtils::cpthdir(par.getOfile("path"), "hist");
		CodeUtils::cpthdir(par.getOfile("path"), "flanks");
		ExceedProb(par,rk,turNodes);								// run corresponding simulation
	}
	if( par.isModeStraj() ){										// if trajectory should be captured
		CodeUtils::cpthdir(par.getOfile("path"), "frequency"); 		// create corresponding folders
		CodeUtils::cpthdir(par.getOfile("path"), "phase");
		traj(par,rk,turNodes); 										// and generate trajectory
	}
}




//std::vector<double> windKamps(Param& par, double dt){
//	// todo fourier trafo?
//}


/**
 * clears all values from the Pseries vector matrix and sets
 * it to 0x0
 */
void StochasticInput::clearPSeries(){
	for( uint i = 0; i < Pseries.size(); i++){	// every row
		Pseries.at(i).clear();					// is cleared
	}
	Pseries.clear(); 							// and the remaining column
}


void StochasticInput::prepareInitialState(double* &voltPhase, double* &voltFreq,
		double* &voltPhase2, double* &voltFreq2, std::vector<int> turbineNodes){

	voltPhase2 = new double[pgrid->getNumNodes()];		// array storing the phase angle at time t+dt (output)
	voltFreq   = new double[pgrid->getNumNodes()];		// array storing the frequency at time t
	voltFreq2  = new double[pgrid->getNumNodes()];		// array storing the frequency at time t+dt

	for(int i = 0; i < pgrid->getNumNodes() ; i++){		// and fill every entry
		voltPhase2[i] = 0;								// of the first output array (is later switched with input)
		voltFreq2[i]  = 0;								//
	}

	for(int i = 0; i < pgrid->getNumNodes(); i++){
		voltFreq[i] = 0;								// node's initial frequency
	}
	voltPhase = pgrid->getVoltAngSS()->data();			// node's initial phase

	setPowerMechStationary(turbineNodes);				// set balanced power initially
}


/**
 *
 * Depending on settings a stochastic time series of wind powers is generated.
 * Then it is transformed to a series of power values using the wind speed power curve.
 * The reference power is adjusted to the turbineNode at reference wind speed.
 *
 * When mode_STEP_FUN is set, only the given wind data is used and taken constant until the next value.
 *
 * When mode_SIF (Stochastic Interpolation of FINO-data) is used the data
 * points are interpolated by a (Ohrnstein-Uhlenbeck) stochastic process matching the ends.
 *
 * When mode_GIVEN_SERIES is set, it is assumed that wind velocities are given
 * on simulation time scale and can be used directly
 *
 * @parameters:
 * int turbineNode : used for rated power and average adjusting, when fluc_share is set
 * vector<double>* windSeries : the vector containing the wind series for
 * 								transformation into power
 * vector<double>* powerSeries: the vector to store the power time series into
 * Param& par : a reference to the Param instance to check modes and the step width
 */
void StochasticInput::stochasticPseries(int turbineNode, std::vector<double> *windSeries,
		std::vector<double> *powerSeries, Param& par){

	// todo Kamps model

// -------------------------------------- mode step function ---------------------------------- //
	if( par.isModeStepFun() ){

		powerSeries->clear();									// prepare empty vector for results

		for( uint i = 0; i < windSeries->size()-1; i++){ 		//
			for(uint j = 0; j < 1.0f/par.rkStepWidth; j++){ 		// fill the wind series with constant data for every RK step
				powerSeries->push_back(windSeries->at(i)); 	// between two given wind data (assumed to be in seconds)
			}
		}

		if( writeStoch ){ 										// if requested
			CodeUtils::vecToBinFile(*powerSeries, 				// print wind speed data to file
					par.getOfile("path") + "windspeed"+stochFileName);
		}

		speedToPower(turbineNode, powerSeries, powerSeries);	// convert the wind data (stored in powerSeries) to power (overwriting wind data)

	}

// --------------------- mode stochastic interpolation of FINO data ----------------------------- //
	if( par.isModeSif() ){

		powerSeries->clear();			// prepare empty vector for results

		std::vector<double> windInterpSer;
		std::vector<double> velemp;
		std::vector<double> tw;
		tw.clear();
		tw.push_back(0);
		tw.push_back(1);


		for( uint i = 0; i < windSeries->size()-1; i++){
			velemp.clear();
			velemp.push_back(windSeries->at(i)); 			// velocity at one instant i
			velemp.push_back(windSeries->at(i+1));			// velocity one second (instant) later

			windInterpSer.clear();
			windInterpSer = stochInterp(par,tw,velemp,par.rkStepWidth);	// interpolate between measured wind data
			for(uint j = 0; j < windInterpSer.size(); j++){ 			// and append them to complete time series powerSeries
				powerSeries->push_back( windInterpSer.at(j) );
			}

		}

		if( writeStoch ){ 												// if requested
			CodeUtils::vecToBinFile(*powerSeries, 						// print wind speed data to file
					par.getOfile("path") + "windspeed"+stochFileName);
		}

		speedToPower(turbineNode, powerSeries, powerSeries);	// convert the wind data (stored in powerSeries) to power (overwriting wind data)

		windInterpSer.clear();
		velemp.clear();
		tw.clear();
	}

// ------------------------------------- mode given wind series ------------------------------------- //
	if( par.isModeGivenSeries() ){
		powerSeries->clear();								// prepare empty vector for results
		for( uint i = 0; i < windSeries->size(); i++){	// make sure powerSeries is as long as wind series
			powerSeries->push_back(0);
		}
		speedToPower(turbineNode, windSeries, powerSeries);	// convert the given wind data to power
	}

	 return;

}

/*
 * Returns a value for power corresponding to the "power curve" with parameters for
 * a wind speed v.
 *
 * @param v : wind speed
 * @param vrate : the rated wind speed for which the power curve saturates
 * @param pfac : prefactor of curve including the reference wind speed ( Pref / vr^3 )
 *
 */
double StochasticInput::powerCurveFunc(double v, double pfac){
	return pfac						// power curve is P_rated at V_rated,
			*pow(std::min(			// constant for v > V_rated
					std::abs(v),	// and cubic for all other v
					vrated),3);		//
}

/*
 * Calculates the prefactor of the power curve from rated wind speed and rated power for the node n
 * (both stored in StochasticInput class)
 *
 *
 * @parameters:
 * int n : the node for which the factor is calculated
 *
 * Also uses Prated and vrated (from StochasticInput class)
 */
double StochasticInput::powerCurvePfac(int n){	// double& pfac

	return Prated.at(n) / pow( vrated , 3);

}

/**
 * Converts all wind speeds in *windspeed into power values using the power curve
 * The two vectors MUST HAVE the same length (powerSeries may be longer)
 *
 * @parameters
 * 	int turbineNode : the node for which the power should be calculated
 * 	*vector<double> windspeed  : the windspeed data
 * 	*vector<double> powerSeries: the resulting power data (may be the same as windspeed data)
 */
void StochasticInput::speedToPower(int turbineNode, std::vector<double> *windspeed, std::vector<double> *powerSeries){

	double Pfac = powerCurvePfac(turbineNode);						// calculate prefactor of power curve

	if( fluc_share ){											// if a share of fluctuating power is given
		for( uint i = 0; i < windspeed->size(); i++){
			powerSeries->at(i) = powerCurveFunc(windspeed->at(i), Pfac);	// and replace with fluctuation	// todo comment
			powerSeries->at(i) += pgrid->getPowerMech(turbineNode) - powerRenewShare.at(turbineNode);
		}
	}
	else {
		for( uint i = 0; i < windspeed->size(); i++){
			powerSeries->at(i) = powerCurveFunc(windspeed->at(i),Pfac);
		}
	}

}

/*
 * Calculates balance factor alpha such that alpha*Ptotconv + Ptotfluc = Ptot.
 *
 * Make sure that the stochastic power series (Pseries) are already calculated
 * for all nodes given in tnodes
 *
 * Note: should work with fluc_share due to the offset mean in Pseries (see speedToPower)
 *
 * @parameters:
 *  std::vector<int> tnodes : indices of all nodes having stochastic power input
 */
void StochasticInput::calcBalanceFactor(std::vector<int> tnodes){
	pgrid->resetPowerMech();
	std::vector<Node*> gens(pgrid->getGens());				// get all generator nodes
	double Ptot = 0;
	for( uint i = 0; i < gens.size(); i++){ 				// get total power of the grid (as if every generator was conventional)
		Ptot += gens.at(i)->getPowerMech();					// get mechanical powers of all generators (as if every generator was conventional)
	}

//	std::vector<double>* pall = pgrid->getPowerMech();
//	double Ptot = CodeUtils::dVecSum(pall);
	double Ptotfluc = 0,									// to store sum of average powers produced by stochastic generators over the simulation time
			Ptotconv = Ptot; 								// to store sum of all powers produced by actual conventional generators
	for( uint i = 0; i < tnodes.size(); i++){ 				// for all stochastic turbine nodes
		Ptotfluc += CodeUtils::dvecMean(Pseries.at(i)); 	// add their average power to total fluctuating power
		Ptotconv -= pgrid->getPowerMech(tnodes.at(i));		// subtract conventional power from a generator that is actually stochastic turbine
	}
	balanceFactor = (Ptot - Ptotfluc) / ( Ptotconv );		// calculate balance factor alpha such that alpha*Ptotconv + Ptotfluc = Ptot

//	pall->clear();
//	delete pall;	// get power mech creates a vector
}

void StochasticInput::getPartsAndLen(int& parts, int& partlen, std::vector<double> *wind){
	if( par->isModeSif() || par->isModeStepFun() ){			// if wind data is given in seconds
		partlen = std::ceil(par->RKsteps					// calculate the part length in seconds too
				* par->rkStepWidth) + 1; 					// add one since the series is interpolated between seconds yielding N-1 interpolations
		parts = std::floor(wind->size() 					// floor to prevent truncated part at the end
				/ partlen);
	}
	if( par->isModeGivenSeries() ){							// if wind data is given on time scale of RK steps
		partlen = par->RKsteps; 							// the parts are that long accordingly
		parts = std::floor(wind->size() 					//
						/ partlen);
	}
}

void StochasticInput::getCurrentWindPart(int part, int partlen,
		std::vector<std::vector<double> > *windSeries, std::vector<std::vector<double> > *windPart){
	for(uint i = 0; i < windPart->size(); i++){
		windPart->at(i).clear();		// is there a memory leak possible? todo
	}
	windPart->clear();
	for(uint tn = 0; tn < windSeries->size(); tn++ ){
		windPart->push_back(std::vector<double>() );
		for(int i = part*partlen; i < (part+1)*partlen; i++){
			windPart->at(tn).push_back(windSeries->at(tn).at(i));
		}
	}
}


// todo: power mean values to file
/*
 * Sets the power of all generators in the grid to the value in the stationary state.
 * if balance_mean is true
 * the power of conventional generators is set to initial value times balance factor.
 * The power of uncontrollable renewable energy sources is set to their mean value.
 * make sure that the balance factor is already calculated (calcBalanceFactor) in that case
 *
 * Otherwise it is set to initial value only.
 */
void StochasticInput::setPowerMechStationary(std::vector<int> tnodes){
	pgrid->resetPowerMech(); 								// if no power balancing needed just set grid to initial values (as read from file)
	if( balance_mean ){ 									// if balancing required
		std::vector<Node*> gens(pgrid->getGens());			// get all generator nodes
		for( uint i = 0; i < gens.size(); i++){ 	 			// for all conventional generators
			gens.at(i)->setPowerMech( 						// set the power for calculating the power flow
					gens.at(i)->getPowerMech()*balanceFactor);	// to their given value adjusted with the balance factor
//			pgrid->setPowerMech(i,
//					pgrid->getPowerMech(i)*balanceFactor);
		}
		for( uint i = 0; i < tnodes.size(); i++){ 			// for all (stochastic) turbine nodes
			pgrid->setPowerMech(tnodes.at(i), 				// set the reference initial power for calculating the power flow
					CodeUtils::dvecMean(Pseries.at(i)) ); 	// to the average of the stochastic produced power
		}
		pgrid->NRPF(); 										// calculate the steady state for the new configuration
	}
}


/**
 *
 * par     : Parameter struct
 * tbegend : starting and end time
 * velemp  : measured (empirical) wind velocities
 * dt      : smallest time step
 */
std::vector<double> StochasticInput::stochInterp(Param& par, std::vector<double> tbegend, std::vector<double> velemp, double dt){


	std::vector<double> veloint;	// allocate double vector
	veloint.clear(); 				// ensure its empty

	 MTRand mt;						// Mersenne twister random generator
	 par.randSeed++; 				// increase random seed
	 mt.seed(par.randSeed);			// set random seed

	 MTRand mt2;					// Mersenne twister random generator for second series
	 mt2.seed(2*par.randSeed);		// set different random seed

	 double mm1,mm2,bb1,bb2; 					// predefine stochastic model parameters
	 	 	 	 	 	 	 	 	 	 	 	// mm mean of corresponding auxiliary process and bb ordinate intersection
	 double tlen = tbegend.at(1)-tbegend.at(0);	// length of first time interval
	 int tsteps = floor( tlen / dt );			// number of interpolation steps between first two empirical data
	 const double GAMMA = windD1;				// optimal stochastic parameter for describing wind velocities with langevin
	 	 	 	 	 	 	 	 	 	 	    // (see Marvin Brune (Bachelor thesis) and Pedro Lind)
	 std::vector<double> velofwd;			    // forward velocity series,
	 velofwd.push_back(velemp.at(0));			// guarantees first value equal to empirical one
	 std::vector<double> velobwd;				// backward velocity series
	 velobwd.push_back(
			 velemp.at(velemp.size()-1));		// guarantees last value equal to empirical one

	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	// get linear interpolation between the two given data points
	 mm1 = ( velemp.at(1)- velemp.at(0))/(2*tbegend.at(0)-tbegend.at(1));			// determine slope for forward process
	 bb1 = (  ( velemp.at(1) + velemp.at(0) )*tbegend.at(0)							// get y-intercept for forward process
			 - velemp.at(1)*tbegend.at(1)  )
					 /(2*tbegend.at(0)-tbegend.at(1));
	 mm2 = ( velemp.at(0) - velemp.at(1) )/(2*tbegend.at(0)-tbegend.at(1));			// determine slope for backward process
	 bb2 = (  ( velemp.at(0) + velemp.at(1) )*tbegend.at(0)							// get y-intercept for backward process
			 - velemp.at(0)*tbegend.at(1) )
					 /(2*tbegend.at(0)-tbegend.at(1));

	 double ct = tbegend.at(0);					// current time step (beginning at t_beg)
	 for(int i = 0; i < tsteps; i++){ 			// for all requested time stepts
		 ct += dt; 								// increase time step
		 velofwd.push_back( 														// generate forward process
				 velofwd.at(i) - GAMMA*( velofwd.at(i) - mm1*ct - bb1) * dt			// add up the drift term
				 + std::sqrt(GAMMA*dt)* mt.randNorm(0,2));							// and the stochastic noise term of current step
		 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	 	// to generate next step
		 velobwd.push_back(
				 velobwd.at(i) - GAMMA*( velobwd.at(i) -mm2*ct - bb2 )*dt
				 + std::sqrt(GAMMA*dt)* mt2.randNorm(0,2));
	 }

	 double lambda;								// linear interpolate parameter (1-l) P1 + l P2
	 for(int i = 0; i < tsteps; i++  ){
//		 ct += dt;	// not needed... time is stored in RungeKutta step width
		 lambda = double(i)/double(tsteps);
		 veloint.push_back(													 // mix forward and backward time series
				 velofwd.at(i)*(1-lambda) + velobwd.at(tsteps-i-1)*lambda ); //
	 }

//	 tlen = tbegend.at(2)-tbegend.at(v+1);	// length of new time interval
//	 tsteps = floor( tlen / dt );			// number of interpolation steps between the two current empirical data


	 return veloint;
}


// -------------------------------------------------- counting methods ------------------------------------------------ //

/**
 *
 * parameter:
 * 		* par   : Param struct
 * 		* t     : current iteration time
 * 		* freq1 : frequency at former time step
 * 		* freq2 : frequency at current time step
 * 		* phas1 : phase at former time step
 * 		* phas2 : phase at current time step
 */
void StochasticInput::checkFlanks(Param& par, double t, double* freq1, double* freq2, double* phas1, double* phas2){

	bool o1, o2;
	for(int i = 0; i < par.len; i++){

		o1 = std::fabs(freq1[i]) > exceedthresh;
		o2 = std::fabs(freq2[i]) > exceedthresh;

		if( !o1 && o2 ){							// former freq below and current above is up flank
			par.frupflanks.at(i).push_back(t);		// push current time as freq up flank
		}
		if( o1 && !o2 ){							// current freq below and former above is down flank
			par.frdownflanks.at(i).push_back(t);	// push current time as freq down flank
		}
	}

}

void StochasticInput::calcTLagSteps(const std::vector<double>& timeLags){
	tLagSteps.clear(); 								// clear current time lag RK steps
	for(uint l = 0; l < timeLags.size(); l++){ 		// for every lag
		tLagSteps.push_back(						// calculate the Runge Kutta steps necessary to match the time lag
				std::round(timeLags.at(l)/par->rkStepWidth));
	}
}

void StochasticInput::resetSurv() {
	surviv.clear();
	surviv.reserve(pgrid->getNumNodes());		// does not fill the vector
	for( int i = 0; i < pgrid->getNumNodes(); i++){
		surviv.push_back(0);
	}
}


void StochasticInput::resetSurvAll(){
	survall = 0;
}

void StochasticInput::resetHist(){
	for(uint i = 0; i < freqHist.size(); i++){
		freqHist.at(i).clear();
	}
	freqHist.clear();

	std::vector<int> hist1;
	for(int i = 0; i < freqHistBinNum; i++){		// initialize a histogram vector
		hist1.push_back(0); 						// every entry is zero, no counts yet
	}
	for(int i = 0; i < pgrid->getNumNodes(); i++){ 	// for every node
		freqHist.push_back(hist1);					// initialize histogram
	}

}

void StochasticInput::resetIncrHist(){
	for(uint l = 0; l < freqIncHist.size(); l++){ 	// for all lags l
		for(uint n = 0;
				n < freqIncHist.at(l).size(); n++){ // for all nodes n
			freqIncHist.at(l).at(n).clear(); 		// clear frequency increment histogram
		}
		freqIncHist.at(l).clear();
	}
	freqIncHist.clear();							// clear histogram itself


	std::vector<int> histBin1;
	for(int i = 0; i < freqHistBinNum; i++){		// initialize a histogram vector
		histBin1.push_back(0); 						// every entry is zero, no counts yet
	}
	std::vector<std::vector<int> > hist2; 			// initialize a histogram matrix
	for(int n = 0; n < pgrid->getNumNodes(); n++){ 	// for every node and every bin
		hist2.push_back(histBin1);
	}

	for(uint l = 0; l < tLagSteps.size(); l++){ 	// for all lags l
		freqIncHist.push_back(hist2);				// initialize entry with a copy of a histogram matrix for all nodes and bins
	}

	std::vector<double> histFreq1; 					// for the frequencies at corresponding time lags
	for(int n = 0; n < pgrid->getNumNodes(); n++){	// initialize them for all nodes
		histFreq1.push_back(0);						// as zero due to the steady state at begin
	}

	for(uint l = 0; l < tLagSteps.size(); l++){		// for all time lags
		FreqLag.push_back(histFreq1);				// copy the steady state frequency vector
	}

}

void StochasticInput::countSurv(double* freq) {
	for( uint i = 0; i < surviv.size(); i++){
		surviv.at(i) +=	(fabs(freq[i]) < exceedthresh);
	}
}

void StochasticInput::countSurvAll(double* freq){
	bool allbelow = 1;
	for(int i = 0; i < pgrid->getNumNodes(); i ++){
		allbelow *= std::fabs(freq[i]) < exceedthresh;
	}
	survall += allbelow;

}

/**
 * creates a histogram by mapping values to integers corresponding to the bins todo comment
 */
void StochasticInput::countHist(double* freq){
	int hbin = 0;
	for(int i = 0; i < pgrid->getNumNodes(); i++){ // for node i
		hbin = std::min(std::max(int(std::floor(freq[i]/freqHistBinWidth)),-1*freqHistBinNumHalf),
				freqHistBinNumHalf-1) + freqHistBinNumHalf;	 										// todo comment!!
		freqHist.at(i).at(hbin)++;
	}
	// make frequency a multiple of 5mHz(angular frequency)
	// and sort it to corresponding bin
}

void StochasticInput::countIncrHist(double* freq, int step){
	for(uint l = 0; l < tLagSteps.size(); l++){		// for every time lag
		int hbin = 0;
		double freqInc = 0;
		if( step%tLagSteps.at(l) == 0 ){ 			// todo comment: if time has come for corsp lag
			for(int n = 0; n < pgrid->getNumNodes(); n++){ // for node n
				freqInc = freq[n] - FreqLag.at(l).at(n);	// to do comment: calculate frequency increment
				hbin = std::min(std::max(int(std::floor(freqInc/freqIncrHistBinWidth)),-1*freqHistBinNumHalf),
						freqHistBinNumHalf-1) + freqHistBinNumHalf;	 										// todo comment!!
				freqIncHist.at(l).at(n).at(hbin)++;
			}
		}
	}
	// make frequency increment a multiple of 5mHz(angular frequency)
	// and sort it to corresponding bin
}

// ------------------------------------------------- evaluation methods ----------------------------------------------- //
/**
 * Iterating a stochastic input in the given turbine nodes,
 * the power flow in the transmission lines is recorded. If it exceeds a threshold the
 * line is switched offline. This can lead to a cascading failure.
 * The exceedance probability is also recorded.
 *
 * @param Param& par: parameter object for filenames, flanks and other recordings
 * @param RungeKutta rk: solver object to iterate swing equation
 * @param vector<int> turbineNodes: vector storing all nodes that exhibit stochastic input
 */
void StochasticInput::stochCasc(Param& par, RungeKutta& rk, vector<int> turbineNodes){

//	// ================ traj ================ //
//	double** ftraj = new double*[par.RKsteps];
//	double** ptraj = new double*[par.RKsteps];
//	for(int i = 0; i < par.RKsteps; i++){
//		ftraj[i] = new double[par.len];
//		ptraj[i] = new double[par.len];
//	}
//	// ================ traj ================ //

	std::vector<double> survsave; 							// saves exceedance probability  values for every time interval
	std::vector<int> *kout1 = new std::vector<int>;			// saves all from nodes of failing transmission lines
	std::vector<int> *kout2 = new std::vector<int>;			// saves all to nodes of failing transmission lines
	std::vector<double> *tout = new std::vector<double>;	// saves the corresponding times at failure
	resetSurvAll(); 										// reset the exceedance probability
	par.resetFlanks();
	par.maxload.clear();
	par.maxlij.clear();

	double* thetaIn1; 										// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t
	double* omegaIn2 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t+dt
	double* tempPoint; 										// temporary array for switching in1 and in2

	for(int i = 0; i < pgrid->getNumNodes() ; i++){			// and fill every entry
		thetaIn2[i] = 0;									// of the first output array (is later switched with input)
		omegaIn2[i] = 0;									//
		omegaIn1[i] = 0;									// node's initial frequency
	}
	thetaIn1 = pgrid->getVoltAngSS()->data();				// node's initial phase

	pgrid->resetPowerMech();								// start with steady state power

	for(int s = 0; s < survInitSteps; s++){									// for all requested runge kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);
		pgrid->checkLines(thetaIn2,kout1,kout2,tout,s*par.rkStepWidth, par);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for(int s = survInitSteps; s < par.RKsteps; s++){						// for all requested runge kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}


		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		countSurvAll(omegaIn2);
		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);
		pgrid->checkLines(thetaIn2,kout1,kout2,tout,s*par.rkStepWidth, par);

//		std::copy(&omegaIn2[0],&omegaIn2[0]+par.len,						// store current frequencies
//				&ftraj[s][0]);
//		std::copy(&thetaIn2[0],&thetaIn2[0]+par.len,						// store current phases
//				&ptraj[s][0]);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

		if( (s-survInitSteps)%survInterval == 0){						// if current time intervals ends
			survsave.push_back(survall); 								// store the exceedance probability  of this interval
			resetSurvAll();												// and reset exceedance probability  counter for next interval
		}

	}


	survsave.push_back(std::numeric_limits<double>::quiet_NaN());			// if this method is called for different stochastic nodes
																			// the survivabilities for all intervals for each node are
																			// in blocks separated by NaNs
	CodeUtils::vecToBinFileFast(survsave, par.getOfile("path")+"surv/surv"							// write exceedance probability  results to binary file
			+par.getOfile("file"));

	CodeUtils::vecIToBinFileFast(*kout1, par.getOfile("path")+"stochcas/stochcasI"					// write from nodes of failing transmission lines
				+par.getOfile("file"));																// to binary file
	CodeUtils::vecIToBinFileFast(*kout2, par.getOfile("path")+"stochcas/stochcasII"					// write to nodes of failing transmission lines
				+par.getOfile("file"));																// to binary file
	CodeUtils::vecToBinFileFast(*tout, par.getOfile("path")+"stochcas/stochcasT"					// write corresponding times to binary file
				+par.getOfile("file"));
	CodeUtils::vecToBinFileFast(par.maxload, par.getOfile("path")+"stochcas/maxL"					// write maximum load over all lines to binary file
				+par.getOfile("file"));
	CodeUtils::vecToBinFileFast(par.maxloadp, par.getOfile("path")+"stochcas/maxLp"					// write maximum percentage of load for all lines
				+par.getOfile("file"));																// to binary file
	CodeUtils::vecIToBinFileFast(par.maxlij, par.getOfile("path")+"stochcas/maxlij"					// write the from and to nodes of maximum loaded
				+par.getOfile("file"));																// line (percentage) to binary file

	for(uint i = 0; i < par.frupflanks.size(); i++){
		CodeUtils::vecToBinFileFast(par.frupflanks.at(i), par.getOfile("path")+"flanks/flanks"		// write upflanks to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                    // at end of that binary file
				par.getOfile("path")+"flanks/flanks"												// write a nan to signal next node
				+par.getOfile("file"));
		CodeUtils::vecToBinFileFast(par.frdownflanks.at(i), par.getOfile("path")+"flanks/flanks"	// write downflanks phase to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                    // at end of that binary file
				par.getOfile("path")+"flanks/flanks"												// write a nan to signal next node
				+par.getOfile("file"));

	}

	kout1->clear();
	kout2->clear();
	tout->clear();

//	// ================ traj ================ //
//	for( int s = 0; s < par.RKsteps; s++){
//		CodeUtils::arrayToBinFile(ftraj[s],par.len,							// write stored frequency to binary file
//				par.getOfile("path")+"traj/freqTraj"
//					+par.getOfile("file"));
//		CodeUtils::arrayToBinFile(ptraj[s],par.len,
//				par.getOfile("path")+"traj/phasTraj"						// write stored phase to binary file
//					+par.getOfile("file"));
//	}
//
//	for(int i = 0; i < par.RKsteps; i++){
//		delete[] ftraj[i];
//		delete[] ptraj[i];
//	}
//	delete[] ftraj;
//	delete[] ptraj;
//	// ================ traj ================ //

	delete[] thetaIn1;														// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
}

/**
 * Calculates the power flow in all lines and measures the mean and maximum power flow for
 * a stochastic power input in one or more nodes
 *
 * @param Param par: Parameter object for solver parameters (RKsteps) and output filenames
 * @param RungeKutta rk: Runge-Kutta solver object to return the iteration of the swing equation
 * @param vector<int> turbineNodes: a list of all nodes exhibiting stochastic power generation
 */
void StochasticInput::stochFlows(Param& par, RungeKutta& rk, std::vector<int> turbineNodes){

	// store flows
	std::vector<std::vector<double> > flowTraj;
	// end store flows

	std::vector<double> fij;	// flow in line i j, stores results
	pgrid->resetLineFlows();

	double avSteps = par.RKsteps-survInitSteps;

	double *thetaIn1, *omegaIn1,	// arrays storing the phase angles and frequencies at time t    (input)
		*thetaIn2, *omegaIn2,		// and at time t+dt (output)
		*tempPoint; 									// temporary array for switching in1 and in2
	prepareInitialState(thetaIn1, omegaIn1, thetaIn2, omegaIn2, turbineNodes); //

	srand(par.getRandSeed());					// set random seed


	for(int s = 0; s < survInitSteps; s++){		// for all requested runge kutta steps	// todo comment

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

//		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for(int s = survInitSteps; s < par.RKsteps; s++){						// for all requested runge kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		pgrid->measureFlow(thetaIn2);

//		flowTraj.push_back(pgrid->saveFlow(thetaIn2));	// todo own mode


//		countHist(omegaIn2);
//		countIncrHist(omegaIn2,s);

//		countSurv(omegaIn2); 												// count exceedance probability
//		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);


		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

//		if( (s-survInitSteps)%survInterval == 0){
//			survsave.push_back(surviv);
//			resetSurv();
//		}

	}

	pgrid->getFIJ(&fij, avSteps);
	CodeUtils::vecToBinFileFast(fij,par.getOfile("path")+ "flow/fij" + par.getOfile("file"));

//	for(uint i = 0; i < flowTraj.size(); i++){	// todo 2: own mode, faster by creating one (dim) array...
//		CodeUtils::vecToBinFileFast(flowTraj.at(i),par.getOfile("path")+"flow/flowtraj"+par.getOfile("file"));
//	}

	delete[] thetaIn1;	// clean up memory
	delete[] thetaIn2;	//
	delete[] omegaIn1;
	delete[] omegaIn2;
}


/**
 * Calculates the exceedance probability for the grid. If the voltage frequency of all nodes stays below the given
 * threshold the exceedance probability is counted up.
 *
 * @param Param par: Parameter object for solver parameters (RKsteps) and output filenames
 * @param PowerGrid pg: Power grid object to get steady state values
 * @param RungeKutta rk: Runge-Kutta solver object to return the iteration of the swing equation
 * @param vector<int> turbineNodes: a list of all nodes exhibiting stochastic power generation
 */
void StochasticInput::ExceedProbAll(Param& par, PowerGrid& pg, RungeKutta& rk, std::vector<int> turbineNodes){

	std::vector<double> survsave; 				// vector to store the exceedance probability for every interval in
	resetSurvAll(); 							// reset exceedance probability measure variables
	par.resetFlanks(); 							// reset vectors to store times when frequency crosses the threshold

	double* thetaIn1 = new double[par.len];		// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint; 							// temporary array for switching in1 and in2

	for(int i = 0; i < par.len ; i++){			// for well defined state, fill every entry
		thetaIn2[i] = 0;						// of the voltage angle output array
		omegaIn2[i] = 0;						// and the voltage frequency output array
	}

	srand(par.getRandSeed());					// set random seed

	thetaIn1 = pg.getVoltAngSS()->data();		// node initial phase
	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = 0;						// node initial frequency
	}
	pg.resetPowerMech(); 						// node initial power

	for(int s = 0; s < survInitSteps; s++){					// for the requested Runge-Kutta steps without measuring the exceedance probability
															// bringing the stochastic process into a stationary state

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 	// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn),
					Pseries.at(tn).at(s));					// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth,
				omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		checkFlanks(par,s*par.rkStepWidth,
				omegaIn1,omegaIn2,thetaIn1,thetaIn2);		// check and store whether the frequency exceeds the given threshold

		tempPoint = omegaIn1;								// switch in and out arrays
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for(int s = survInitSteps; s < par.RKsteps; s++){						// for all requested Runge-Kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		countSurvAll(omegaIn2); 											// count exceedance probability
		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

		if( (s-survInitSteps)%survInterval == 0){ 	// whenever a exceedance probability interval ends
			survsave.push_back(survall); 			// store the exceedance probability  for that interval
			resetSurvAll();							// and reset it for the next one
		}

	}

	survsave.push_back(std::numeric_limits<double>::quiet_NaN());
	CodeUtils::vecToBinFileFast(survsave, par.getOfile("path")+"surv/surv"						// write stored phase to binary file
			+par.getOfile("file"));


	for(uint i = 0; i < par.frupflanks.size(); i++){
		CodeUtils::vecToBinFileFast(par.frupflanks.at(i), par.getOfile("path")+"flanks/flanks"	// write upflanks to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                // at end of that binary file
				par.getOfile("path")+"flanks/flanks"											// write a nan to signal next node
				+par.getOfile("file"));
		CodeUtils::vecToBinFileFast(par.frdownflanks.at(i), par.getOfile("path")+"flanks/flanks" // write downflanks phase to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                // at end of that binary file
				par.getOfile("path")+"flanks/flanks"											// write a nan to signal next node
				+par.getOfile("file"));

	}

	delete[] thetaIn1;								// clean up memory
	delete[] thetaIn2;
	delete[] omegaIn1;
	delete[] omegaIn2;
}


/**
 * Calculates the exceedance probability for every node in the grid individually. If the voltage frequency
 * of the corresponding node stays below the given threshold the exceedance probability is counted up for
 * that node.
 *
 * @param Param par: Parameter object for solver parameters (RKsteps) and output filenames
 * @param RungeKutta rk: Runge-Kutta solver object to return the iteration of the swing equation
 * @param vector<int> turbineNodes: a list of all nodes exhibiting stochastic power generation
 */
void StochasticInput::ExceedProb(Param& par, RungeKutta& rk, std::vector<int> turbineNodes){

	std::vector<std::vector<double> > survsave;
	resetSurv();
	resetHist();
	resetIncrHist();
	par.resetFlanks();

	double* thetaIn1;							// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[par.len];		// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[par.len];		// array storing the frequency at time t
	double* omegaIn2 = new double[par.len];		// array storing the frequency at time t+dt
	double* tempPoint; 							// temporary array for switching in1 and in2

	for(int i = 0; i < par.len ; i++){			// and fill every entry
		thetaIn2[i] = 0;						// of the first output array (is later switched with input)
		omegaIn2[i] = 0;						//
	}

	srand(par.getRandSeed());					// set random seed

	for(int i = 0; i < par.len; i++){
		omegaIn1[i] = 0;						// node's initial frequency
	}
	thetaIn1 = pgrid->getVoltAngSS()->data();	// node's initial phase

	setPowerMechStationary(turbineNodes);	// todo #2 also for others!!

	for(int s = 0; s < survInitSteps; s++){		// for all requested runge kutta steps	// todo comment

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;


	}

	for(int s = survInitSteps; s < par.RKsteps; s++){						// for all requested runge kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		countHist(omegaIn2);
		countIncrHist(omegaIn2,s);

		countSurv(omegaIn2); 												// count exceedance probability
		checkFlanks(par,s*par.rkStepWidth,omegaIn1,omegaIn2,thetaIn1,thetaIn2);

		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

		if( (s-survInitSteps)%survInterval == 0){
			survsave.push_back(surviv);
			resetSurv();
		}

	}

	// todo name extension for time series
	// todo method for saving
	std::vector<double> nanvec; 									// create a new double vector
	for( int i = 0; i < pgrid->getNumNodes(); i++){					// filled with
		nanvec.push_back(std::numeric_limits<double>::quiet_NaN());	// (quiet) NaNs.
	}																// it will indicate the begin/end of the result vector
	survsave.push_back(nanvec);										// for every simulation for a single perturbed node
	for( uint i = 0; i < survsave.size(); i++){
	CodeUtils::vecToBinFileFast(survsave.at(i), 					// then write the exceedance vector to binary file
			par.getOfile("path")+"surv/surv"
			+par.getOfile("file"));
	}

	// todo nur bei bestimmtem mode rausschreiben
	for(int i = 0; i < pgrid->getNumNodes(); i++){
		CodeUtils::vecIToBinFileFast(freqHist.at(i), par.getOfile("path")+"hist/histFreq"
				+ par.getOfile("file"));
	}

	for(uint l = 0; l < tLagSteps.size(); l++){
		for(int n = 0; n < pgrid->getNumNodes(); n++){
				CodeUtils::vecIToBinFileFast(freqIncHist.at(l).at(n), par.getOfile("path")+"hist/histIncrFreq"
						+ par.getOfile("file") + "-" + CodeUtils::d2Str(timeLags.at(l)) );
			}
	}


	for(uint i = 0; i < par.frupflanks.size(); i++){
		CodeUtils::vecToBinFileFast(par.frupflanks.at(i), par.getOfile("path")+"flanks/flanks"	// write upflanks to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                // at end of that binary file
				par.getOfile("path")+"flanks/flanks"											// write a nan to signal next node
				+par.getOfile("file"));
		CodeUtils::vecToBinFileFast(par.frdownflanks.at(i), par.getOfile("path")+"flanks/flanks" // write downflanks phase to binary file
				+par.getOfile("file"));
		CodeUtils::doubleToBinFileFast(std::numeric_limits<double>::quiet_NaN(),                // at end of that binary file
				par.getOfile("path")+"flanks/flanks"											// write a nan to signal next node
				+par.getOfile("file"));

	}


	delete[] thetaIn1;	// clean up memory
	delete[] thetaIn2;	//
	delete[] omegaIn1;
	delete[] omegaIn2;
}

void StochasticInput::traj(Param& par, RungeKutta& rk, std::vector<int> turbineNodes){


	double** freqTraj = new double*[par.RKsteps];			// storage for the frequency trajectory
	double** phasTraj = new double*[par.RKsteps];			// storage for the phase trajectory

	for(int i = 0; i < par.RKsteps; i++){					// initialize storage arrays
		freqTraj[i] = new double[pgrid->getNumNodes()];		// for frequency
		phasTraj[i] = new double[pgrid->getNumNodes()];		// and for phase
	}

	double* thetaIn1;										// array storing the phase angle at time t    (input)
	double* thetaIn2 = new double[pgrid->getNumNodes()];	// array storing the phase angle at time t+dt (output)
	double* omegaIn1 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t
	double* omegaIn2 = new double[pgrid->getNumNodes()];	// array storing the frequency at time t+dt
	double* tempPoint; 										// temporary array for switching in1 and in2

	for(int i = 0; i < pgrid->getNumNodes() ; i++){			// and fill every entry
		thetaIn2[i] = 0;						// of the first output array (is later switched with input)
		omegaIn2[i] = 0;						//
	}

	srand(par.getRandSeed());					// set random seed

	for(int i = 0; i < pgrid->getNumNodes(); i++){
		omegaIn1[i] = 0;						// node's initial frequency
	}
	thetaIn1 = pgrid->getVoltAngSS()->data();	// node's initial phase

	pgrid->resetPowerMech();	// TODO genau das fuer stufen function stoerung (auch fuer andere modes integr.) // hier wiederum balance factor integr.

	for(int s = 0; s < par.RKsteps; s++){									// for all requested runge kutta steps

		for( uint tn = 0; tn < turbineNodes.size(); tn++){ 					// for all nodes requested
			pgrid->setPowerMech(turbineNodes.at(tn), Pseries.at(tn).at(s));	// iterate the power corresponding to stochastic time series
		}

		rk.rk4(par.rkStepWidth, omegaIn1, thetaIn1, omegaIn2, thetaIn2);	// calculate the swing equation

		std::copy(&omegaIn2[0],&omegaIn2[0]+pgrid->getNumNodes(),			// store current frequencies
				&freqTraj[s][0]);
		std::copy(&thetaIn2[0],&thetaIn2[0]+pgrid->getNumNodes(),			// store current phases
				&phasTraj[s][0]);


		tempPoint = omegaIn1;												// switch in and out
		omegaIn1 = omegaIn2;
		omegaIn2 = tempPoint;
		tempPoint = thetaIn1;
		thetaIn1 = thetaIn2;
		thetaIn2 = tempPoint;

	}

	for( int s = 0; s < par.RKsteps-1; s++){
		CodeUtils::arrayToBinFile(freqTraj[s],pgrid->getNumNodes(),			// write stored frequency to binary file
				par.getOfile("path")+"frequency/freqTraj"
					+par.getOfile("file"));
		CodeUtils::arrayToBinFile(phasTraj[s],pgrid->getNumNodes(),
				par.getOfile("path")+"phase/phasTraj"						// write stored phase to binary file
					+par.getOfile("file"));
		delete[] freqTraj[s];												// clear frequency storage
		delete[] phasTraj[s];												// clear phase storage
	}

	delete[] thetaIn2;	// clean up memory
	delete[] omegaIn1;
	delete[] omegaIn2;

}



// ------------------------------------------------------- get&set ---------------------------------------------------- //
bool StochasticInput::isPratedGiven() const {
	return pratedGiven;
}

int StochasticInput::getFreqHistBinNum() const {
	return freqHistBinNum;
}

void StochasticInput::setFreqHistBinNum(int freqHistBinNum) {
	this->freqHistBinNum = (freqHistBinNum/2)*2; 		// only even numbers are allowed
	this->freqHistBinNumHalf = freqHistBinNum/2; 		// half in positive and negative direction. Important for later implementation
}

double StochasticInput::getFreqHistBinWidth() const {
	return freqHistBinWidth;
}

void StochasticInput::setFreqHistBinWidth(double freqHistBinWidth) {
	this->freqHistBinWidth = freqHistBinWidth;
}

void StochasticInput::setPratedGiven(bool pmaxGiven) {
	this->pratedGiven = pmaxGiven;
}

const std::vector<double>& StochasticInput::getTimeLags() const {
	return timeLags;
}

bool StochasticInput::isMultipleWindSer() const {
	return multipleWindSer;
}

bool StochasticInput::isFlucShare() const {
	return fluc_share;
}

void StochasticInput::setFlucShare(bool flucShare) {
	fluc_share = flucShare;
}

void StochasticInput::setMultipleWindSer(bool multipleWindSer) {
	this->multipleWindSer = multipleWindSer;
}

void StochasticInput::setTimeLags(const std::vector<double>& timeLags) {
	this->timeLags = timeLags;
}

// ------------------------------------------------------- to file ---------------------------------------------------- //
bool StochasticInput::toFile(Param* par){

	if( !par->param){
		return false;
	}

	string fname = "Param" + par->filenum;

	std::ofstream outfile( fname, ios::out|ios::app);

	outfile << toString();

	outfile.flush();
	outfile.close();

	return true;
}

std::string StochasticInput::toString(){
	std::stringstream outfile;
	outfile << "=================== Stochastic Input ====================" 	<< std::endl;
	outfile << "----------------- Algorithm Parameters ------------------" 	<< endl;
	outfile << "TurbineNodeConfigs: " 						<< endl;
	for(uint i = 0; i < turNodeMat.size(); i++){
		outfile << CodeUtils::ivec2str(turNodeMat.at(i));
	}
	outfile 												<< endl;
	outfile << "------------------ Turbine Parameters -------------------" 	<< endl;
	outfile << "Rated speed: " 			<< vrated 			<< endl;
	outfile << "Rated power: "			 		 			<< endl;
	outfile << CodeUtils::dvec2str(Prated);
	outfile << "Average speed: " 		<< vmean			<< endl;
	outfile 												<< endl;
	outfile << "---------------------- Exceedance -----------------------" 	<< endl;
	outfile << "Threshold: " 			<< exceedthresh		<< endl;
	outfile << "InitializationSteps: "	<< survInitSteps	<< endl;
	outfile << "Interval: "				<< survInterval		<< endl;
	outfile 												<< endl;
	outfile << "----------------- Frequency Histogram -------------------" 	<< endl;
	outfile << "Bin width: " 			<< freqHistBinWidth	<< endl;
	outfile << "Number of bins: "		<< freqHistBinNum	<< endl;
	outfile << "Minimum Frequency: "	<<
			-1*freqHistBinNumHalf*freqHistBinWidth			<< endl;
	outfile << "Maximum Frequency: "	<<
			freqHistBinNumHalf*freqHistBinWidth				<< endl;
	outfile 												<< endl;

	outfile << endl;
	return outfile.str();
}

// ==================================================== variables ===================================================== //
// =================================================== constructors =================================================== //
// ====================================================== methods ===================================================== //
// --------------------------------------------------- array methods -------------------------------------------------- //
// ------------------------------------------------- directory methods ------------------------------------------------ //
// ------------------------------------------------ mathematical methods ---------------------------------------------- //
// --------------------------------------------------- string methods ------------------------------------------------- //
// ------------------------------------------------------- get&set ---------------------------------------------------- //
// ------------------------------------------------------ read methods ------------------------------------------------ //
