/*
 * StochasticInput.h
 *
 *  Created on: Feb 27, 2019
 *      Author: mawolff
 */

#ifndef STOCHASTICINPUT_H_
#define STOCHASTICINPUT_H_


//#include "Param.h"
#include "PowerGrid.h"
#include "RungeKutta.h"

class StochasticInput {

// ==================================================== variables ===================================================== //
private:
	PowerGrid* pgrid;				// pointer to the grid being under investigation
	Param* par;						// pointer to parameter class
//	Function* func;					// the function that determines the nodes behaviour

	bool balance_mean;		// todo mode or similar
	bool fluc_share;		// to do: probably own mode or similar
	bool multipleWindSer;	// if the given wind series is separated into parts

	bool MODE_MULTI_TURB;			// if multiple turbines have stochastic input at the same time
	bool pratedGiven;				// when Pref is explicitly given by user do not calculate it
	bool writeStoch;				// if the stochastic power time series should be written to file
	std::string stochFileName;		// the power time series is written into a file with this name if requested

	double vmean;					// average wind speed
	double vrated;					// rated wind speed (where power curve saturates)
	std::vector<std::vector<int> >	// stores all turbine nodes configurations (vertical direction)
		turNodeMat;					// i.e. node indices having stochastic power input (horizontal direction)
	std::vector<double> Prated;		// rated power. The turbine reaches this power at rated speed
	double balanceFactor;			// the power of all conventional generators is multiplied by this value
									// to ensure power balance in steady state for low or high wind series averages
	std::vector<double>
		powerRenewShareInit;		// share of renewable power at the nodes as given by input file
	std::vector<double>
		powerRenewShare;			// share of renewable power at the nodes for current configuration

	double exceedthresh;			// threshold for exceedance measure
	double survall;					// survivability measured for all nodes below threshold (not individually)
	std::vector<double> surviv;		// survivability measure (number of time steps below freq. threshold, for every node)
	int survInitSteps;				// steps to simulate the stochastic process before measuring survivability
	int survInterval;				// the number of RK steps measuring the survivability
	std::vector<std::vector<int> >
						freqHist;	// histogram of frequencies measured in discrete bins of x mHz (angular frequency)
	int freqHistBinNum;				// resolution of frequency histogram
	int freqHistBinNumHalf;			// set number of bins and corresponding maximum and minimum frequency for histogram
	double freqHistBinWidth;		// bin width of frequency histogram in Hz (angular frequency)

	// increment histogram
	std::vector<double> timeLags;	// time lags for increments in seconds, initialized by reading from file
	std::vector<int> tLagSteps;		// the number of RK steps corresponding the time lags
	std::vector<std::vector<double> >
						FreqLag;	// frequencies at corresponding lags
	double freqIncrHistBinWidth;	// bin width of frequency increment histogram in Hz (angular frequency)
	std::vector<std::vector<std::vector<int> > >
					freqIncHist;	// 3 dim array x for lag, y for node, z for increment bins

//	int turIndepWS;					// number of turbine nodes with independent wind series
	std::vector<std::vector<double> > Pseries;	// stores the power input for all nodes and all times
	std::vector<std::vector<double> > windSpeedMeasured;		// the measured wind data (FINO1) as interpolation points
	std::vector<std::vector<double> > windSpeedCur;			// current wind speed series
	// todo idea vector<double> PseriesMean (is often needed)
	double windD1;					// drift coefficient for wind series interpolation (and diff. is sqrt(windD1))

// =================================================== constructors =================================================== //
public:
	StochasticInput(PowerGrid* pg, Param* pars);
	virtual ~StochasticInput();

// ====================================================== methods ===================================================== //
public:
	bool readFromFile(std::string pfile);

	void stochSwing(Param& par, RungeKutta& rk);

private:
	void prepareTurbines(Param& par, RungeKutta& rk,
			std::vector<int> turNodes,  int w);
	void stochEvaluation(Param& par, RungeKutta& rk,
			std::vector<int> turNodes);

	void stochCasc(Param& par, RungeKutta& rk, std::vector<int> turbineNode);
	void stochFlows(Param& par, RungeKutta& rk, std::vector<int> turbineNode);
//	void ExceedProbAll(Param& par, PowerGrid& pg, RungeKutta& rk, std::vector<int> turbineNodes);
//	void ExceedProb(Param& par, PowerGrid& pg, RungeKutta& rk, std::vector<int> turbineNodes);
	void ExceedProbAll(Param& par, PowerGrid& pg, RungeKutta& rk, std::vector<int> turbineNodes);
	void ExceedProb(Param& par, RungeKutta& rk, std::vector<int> turbineNodes);
	void traj(Param& par, RungeKutta& rk, std::vector<int> turbineNodes);

private:
	void prepareInitialState(double* &voltPhase, double* &voltFreq,
						double* &voltPhase2, double* &voltFreq2,
						std::vector<int> turbineNodes);
	void clearPSeries();
	void stochasticPseries(int turbNode, std::vector<double> *windSeries,
			std::vector<double> *powerSeries, Param& par);	// todo without par ?
	std::vector<double> stochInterp(Param& par, std::vector<double> tbegend,
			std::vector<double> velemp, double dt);
	double powerCurvePfac(int n);		// calculates prefactor of the power curve for node n
	double powerCurveFunc(double v, double pfac);	// functional form of power curve
	void speedToPower( int turbineNode,
			std::vector<double> *windspeed, std::vector<double> *power);
	void calcBalanceFactor(std::vector<int> tnodes);
	void getPartsAndLen(int& parts, int& partlen, std::vector<double> *wind);
	void getCurrentWindPart(int part, int partlen,
			std::vector<std::vector<double> > *windSeries,
			std::vector<std::vector<double> > *windPart);

	void checkFlanks(Param& par, double t, double* freq1, double* freq2,
			double* phas1, double* phas2);

	void setPowerMechStationary(std::vector<int> tnodes);
	void calcTLagSteps(const std::vector<double>& timeLags);
	void resetSurv();
	void resetSurvAll();
	void resetHist();
	void resetIncrHist();
	void countSurv(double* freq);
	void countSurvAll(double* freq);
	void countHist(double* freq);
	void countIncrHist(double* freq, int step);

public:
	bool toFile(Param* par);
	std::string toString();

	bool isPratedGiven() const;
	void setPratedGiven(bool pmaxGiven);
	int getFreqHistBinNum() const;
	void setFreqHistBinNum(int freqHistBinNum);
	double getFreqHistBinWidth() const;
	void setFreqHistBinWidth(double freqHistBinWidth);
	const std::vector<double>& getTimeLags() const;
	void setTimeLags(const std::vector<double>& timeLags);
//	const std::vector<int>& getTLagSteps() const;
//	void setTLagSteps(const std::vector<double>& timeLags);
	bool isMultipleWindSer() const;
	void setMultipleWindSer(bool multipleWindSer);
	bool isFlucShare() const;
	void setFlucShare(bool flucShare);
};

#endif /* STOCHASTICINPUT_H_ */

// ==================================================== variables ===================================================== //
// =================================================== constructors =================================================== //
// ====================================================== methods ===================================================== //
// --------------------------------------------------- array methods -------------------------------------------------- //
// ------------------------------------------------- directory methods ------------------------------------------------ //
// ------------------------------------------------ mathematical methods ---------------------------------------------- //
// --------------------------------------------------- string methods ------------------------------------------------- //
// ------------------------------------------------------- get&set ---------------------------------------------------- //
// ------------------------------------------------------ read methods ------------------------------------------------ //
