
%% example tjk surface
% plots a single surface obtained from the return time approach in 2d and
% 3d. Also a part of the surface is shown in higher resolution to
% illustrate the chaotic behaviour close to the limit cycle regime. The
% nodes are j = 9 and k = 3;
%
%
% V1.0 mawolff@uos.de 13-Jul-2020


global NO_CLEAR
if NO_CLEAR     % for testing scripts
else            % if NO_CLEAR is empty or false
    clear;      % do clear
end
addpath('methods');

% ============================== settings  ============================== %
save_fig = false;
dpth= 'data/';
spth = '';

FORMAT = 'fr';
fs = dissFS(FORMAT);
% font = 'Times New Roman';
font = 'LM Roman 10';
tblfont = 'LM Roman 10';

X_LABEL = '$\Delta\varphi$';
Y_LABEL = '$\Delta\omega$';

LC_VALUE = 0;
CLIM = 6;

% ----------------------------- coordinates ----------------------------- %
XOFFC = 0.035;   % offset for figure (c)
XOFF = 0.075;
YOFF = 0.15;
X_SPACE = XOFF;
X_SPACE_CB = 0.005; % space between axis and colorbar
SIZE_X_AX = (1 - 3*X_SPACE - XOFF)/3;
SIZE_Y_AX = (1-1.75*YOFF);
X_CB = 0.020;
Y_CB = SIZE_Y_AX;

%%
% ================================ load  ================================ %
try 
    file = [dpth 'fig3-4-tjk.mat'];
    load(file);
    matr = true;
catch ME 
    matr = false;
    switch ME.identifier 
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ... 
                file ' could not be read.']); 
        case 'MATLAB:FileIO:InvalidFid'  
            warning(['File ' file  ' could not be find.']);  
        otherwise 
            rethrow( ME ); 
    end 
    
end

%%

close all;
fig = figure;
% set( fig, 'position', [123 456 dissFigSize(FORMAT)]);
set(fig, 'position', [100 200 1200 450]);

sur.z( sur.z > 19) = LC_VALUE;
sur.z(1:3) = LC_VALUE;

% ============================ 3d surf plot  ============================ %
axa = axes;
set( axa, 'position', [XOFF YOFF SIZE_X_AX SIZE_Y_AX] );

su = surf(sur.x, sur.y, sur.z );
set( su, 'edgealpha', 0);
view([-10 50]);

set( axa, 'clim', [0 CLIM], ...
    'xlim', [min(min(sur.x)) max(max(sur.x))],...
    'ylim', [min(min(sur.y)) max(max(sur.y))],...
    'zlim', [0 12],...
    'fontsize', fs,...
    'ticklabelinterpreter','latex',...
    'xtick', [-3.1 -pi/2 0 pi/2 3.05],...
    'ytick', [-6.2 -pi 0 pi 6.15]);

xtl = {'$-\pi$', '$-\pi/2$', '$0$', '$\pi/2$', '$\pi$'};
xticklabels(xtl); 
ytl = {'$-2\pi$', '$-\pi$', '$0$', '$\pi$', '$2\pi$'};
yticklabels(ytl); 

xlha = xlabel(X_LABEL, 'interpreter', 'latex');
ylha = ylabel(Y_LABEL, 'interpreter', 'latex');
zlha = zlabel('$t_{jk}$', 'interpreter', 'latex');

xlim = get(axa,'xlim');
ylim = get(axa,'ylim');
zlim = get(axa,'zlim');

% apos = get(gca,'position');

set(zlha, 'Rotation',0);
set(zlha, 'Position',[xlim(1)-3*pi/8 ...
    ylim(2)+pi/4 (zlim(2)+zlim(1))/2]);

rotYlabel( ylha, axa );

% ylha = get(gca,'ylabel');
% set(ylha, 'Rotation',0);
set(ylha, 'Position',[-4.4437    0.2006   -0.0000]);
% xlha = get(gca,'xlabel');
set(xlha, 'Rotation',0);
set(xlha, 'Position',[-0.3800   -7.9008   -1.3804]);

% --------------------------- label text box  --------------------------- %
tblo.siz = dissFS(FORMAT);
tblo.fontname = tblfont;
tba = dissAlabel( fig, '(a)', tblo );
set( tba, 'position', get(tba, 'position')-[0 0.01 0.025 0] );

% ============================ 2d surf plot  ============================ %
axb = axes;
set( axb, 'position', [ (XOFF+X_SPACE+SIZE_X_AX)  YOFF ...
    (SIZE_X_AX-X_SPACE_CB-X_CB)  SIZE_Y_AX] );

su = surf(sur.x, sur.y, sur.z );
set( su, 'edgealpha', 0);
view([0 90]);

cm = colormap('parula');
cm = [[0 0 0]; cm];
colormap(cm);
cb = colorbar;

set( axb, 'clim', [0 CLIM], ...
    'xlim', [min(min(sur.x)) max(max(sur.x))],...
    'ylim', [min(min(sur.y)) max(max(sur.y))],...
    'fontsize', fs,...
    'ticklabelinterpreter','latex',...
    'xtick', [-3.1 -pi/2 0 pi/2 3.05],...
    'ytick', [-6.2 -pi 0 pi 6.15]);

xtl = {'$-\pi$', '$-\pi/2$', '$0$', '$\pi/2$', '$\pi$'};
xticklabels(xtl); 
ytl = {'$-2\pi$', '$-\pi$', '$0$', '$\pi$', '$2\pi$'};
yticklabels(ytl); 

xlabel(X_LABEL, 'interpreter', 'latex');
ylabel(Y_LABEL, 'interpreter', 'latex');
ylh = get(gca,'ylabel');
xlim = get(gca,'xlim');
ylim = get(gca,'ylim');

set(ylh, 'Rotation',0);
set(ylh, 'Position',[xlim(1)-0.45*abs(xlim(1)) ...
    -0.5 0]);

% ------------------------------ colorbar  ------------------------------ %
CB_LABELS = cell(7,1);
CB_LABELS{1} = 'LC';
for k = 1:6
    CB_LABELS{k+1} = num2str(k);
end

set( cb, 'ticklabels', CB_LABELS,...
    'ticks', 0:6, ...
    'position', [(XOFF+2*SIZE_X_AX+X_SPACE-X_CB) YOFF ...
        X_CB Y_CB]);
    
tbjk1 = tbox( [ (XOFF+2*SIZE_X_AX+X_SPACE-X_CB) ...
    (YOFF+SIZE_Y_AX+0.001) ...
    X_CB 0.075], ...
    '$t_{jk}$',dissFS(FORMAT) );
set( tbjk1, 'interpreter','latex');

% --------------------------- label text box  --------------------------- %
tbb = dissAlabel( fig, '(b)', tblo );
set( tbb, 'position', get(tba, 'position')...
    +[XOFF+SIZE_X_AX+0.25*X_SPACE 0 0 0] );


% ========================== chaos surf plot  =========================== %
axc = axes;
set( axc, 'position', [ (XOFF+2*X_SPACE+2*SIZE_X_AX+XOFFC)  YOFF ...
    (SIZE_X_AX-X_SPACE_CB-X_CB)  SIZE_Y_AX] );

surdet.z( surdet.z > 19) = LC_VALUE;
% surdet.z(1:3) = LC_VALUE;
su = surf(surdet.x, surdet.y, surdet.z );
set( su, 'edgealpha', 0);
view([0 90]);

detareay = 27:36;
detareax = 10:100;

xlims = [min(min(surdet.x(detareax,detareay) ))...
    max(max(surdet.x(detareax,detareay) ))];
ylims = [min(min(surdet.y(detareax,detareay) ))...
    max(max(surdet.y(detareax,detareay) ))];
zlims = [min(min(surdet.z(detareax,detareay) ))...
    max(max(surdet.z(detareax,detareay) ))];

xtick = xlims(1):(xlims(2)-xlims(1))/2:xlims(2);
ytick = ylims(1):(ylims(2)-ylims(1))/5:ylims(2);

set( gca, 'clim', zlims, ...
    'xlim', xlims,...
    'ylim', ylims,...
    'zlim', zlims,...
    'fontsize', fs,...
    'ticklabelinterpreter','latex',...
    'xtick', xtick,...
    'ytick', ytick);

xlabel(X_LABEL, 'interpreter', 'latex');
ylabel(Y_LABEL, 'interpreter', 'latex');
zlha = zlabel('$t_{jk}$');
set( zlha, 'interpreter', 'latex');
ylhc = get(gca,'ylabel');
apos = get(gca,'position');
set(ylhc, 'Rotation',0);
set(ylhc, 'Position',[2.9233    2.0516   -0.0000]);


% ------------------------------ colorbar  ------------------------------ %
cb2 = colorbar;
CB_LABELS = {'LC','5','10','15'};

set( cb2, 'ticklabels', CB_LABELS,...
    ...'ticks', 0:6, ...
    'position', [(XOFF+3*SIZE_X_AX+2*X_SPACE-X_CB+XOFFC) YOFF ...
        X_CB Y_CB]);
    
tbjk2 = tbox( [(XOFF+3*SIZE_X_AX+2*X_SPACE-X_CB+XOFFC) ...
    (YOFF+SIZE_Y_AX+0.001) ...
    X_CB 0.075], ...
    '$t_{jk}$',dissFS(FORMAT) );
set( tbjk2, 'interpreter','latex');

% --------------------------- label text box  --------------------------- %
tbc = dissAlabel( fig, '(c)', tblo );
set( tbc, 'position', get(tba, 'position') ...
    +[XOFF+2*SIZE_X_AX+1.25*X_SPACE+XOFFC/2 0 0 0]);%, ...
%     'fontname', font);

% ============================== save fig  ============================== %
if( ~~save_fig )
    saveas(fig, [spth 'RTexampleSurf.eps'], ...
        'epsc');
end

