function [ mpc ] = gatherGen( mpc, opt )
%% newmpc = gatherGen( mpc, opt )
%
% Gathers the generation of active and reactive power of all generators at
% each bus such that one aggregated generator remains per bus. See below
% return what is exactly done with each parameter or column
%
% * Parameters
%   * mpc : mpc file (see caseformat.m) with several gens per bus
%   * opt : options struct
%       * opt.allOp = true; take the generation of all generators as if all
%                   generators are operational (column 8 = 1 for all gens)
%                   false: use mpc.gen(:,8)
%
% * Returns
%   * newmpc : the mpc with gathered generators in the mpc.gen table
%
% * Treatment of columns:
%   -01 bus id : using first per bus
%   -02 P gen  : sum
%   -03 Q gen  : sum
%   -04 Q max  : sum
%   -05 Q min  : sum
%   -06 V_SP   : first per bus
%   -07 mBase  : sum
%   -08 status : all 1 (online), confer opt.allOp
%   -09 P max  : sum
%   -10 P min  : sum
%
% WARNING: further cols are for optimal power-flow that I have not
% considered yet
%
% V1.0 mawolff@uos.de 06-Jul-2020
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt = struct(); 
end
 
 opt = readOptions( opt );
 
%% code
ncol = size( mpc.gen, 2);
ugb = unique(mpc.gen(:,1)); % unique generators buses
nng = size( ugb, 1 );

ngen = zeros( nng, ncol );

for i = 1:numel(ugb)
    gbi = ugb(i);                   % i-th unique generator bus number
    gbiidx = mpc.gen(:,1) == gbi;   % label of all gen columns corresp. to gbi
    
    if( ~opt.allOp )                                    % if online status of gens should be considered
        gbiidx = and( gbiidx, logical(mpc.gen(:,8)) );  % use only online generators
    end
    
%     mbest = polyfit( mpc2.gen(:,2), mpc2.gen(:,7), 1) ; % mBase estimation

    ids = mpc.gen(gbiidx,1);
    ngen(i,1) = ids(1);
                                            % add up all ...
    ngen(i,2) = sum( mpc.gen(gbiidx,2) );   % generated active power at this bus
    ngen(i,3) = sum( mpc.gen(gbiidx,3) );   % generated reactive power at this bus
    ngen(i,4) = sum( mpc.gen(gbiidx,4) );   % max possible reactive power output at this bus
    ngen(i,5) = sum( mpc.gen(gbiidx,5) );   % min possible reactive power output at this bus
    
    vsp = mpc.gen(gbiidx,6);                % the voltage set-point should be equal for all gens at same bus
    ngen(i,6) = vsp(1);                     % hence use first one
    
    ngen(i,7) = sum( mpc.gen(gbiidx, 7) );  % sum up mBases
    ngen(i,8) = 1;                          % either all generators are online or only online gens were considered
    
    ngen(i,9) = sum( mpc.gen(gbiidx,9) );   % max possible active power output at this bus
    ngen(i,10) = sum( mpc.gen(gbiidx,10) ); % min possible active power output at this bus
    % TODO further cols are for optimal power-flow that I have not
    % considered yet
    
end
 
% ngen(:,1) = ugb; % only if they are sorted in original struct
mpc.gen = ngen;
return;
 
end
 
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'allOp') )
    opt.allOp = true;
 end
end
