function [ ret ] = highlightPGNodes( p, mpc, opt )
%% highlightPGNodes( p, mpc, opt )
%
% some features are not compatible with every model, a comprehensive
% documentation will hopefully given later and expanded from time to time
%
% * Parameters
%   * p : plot handle
%   * mpc
%   * opt
%       * opt.genmark = ''; marker for the generator loads (circle if
%                           empty)
%       * opt.loadmark = '';
%       * opt.gencolor = [0.6902  0.0627  0.2078]; Color for generator
%                   nodes
%       * opt.loadcolor = [1.0000  0.7020  0.0824]; Color for load nodes
%       * opt.loadscale = []; if empty (default) the nodes are scaled
%                           according to their power (loadscalef)
%       * opt.genscale = []; if empty (default) the nodes are scaled
%                           according to their power (genscalef)
%       * opt.loadscalef = @(x) (opt.pmin+ abs(x)/opt.pmax*20);
%                   function to scale the load node markers
%       * opt.genscalef = @(x) (opt.pmin+ abs(x)/opt.pmax*20);
%                   function to scale the generator node markers
%       * opt.numbering = 'seq'; how to number the nodes
%                       * seq  : sequentiell
%                       * gl   : generators first then loads
%                       * ext  : like they are named in the mpc.bus table
%                       * given: as given in opt.nodelabels (char cell mat)
%                       * none : otherwise no labels
%       * opt.waitbar = false; If a waitbar should show the progress
%       * opt.ax = p.Parent; The axis to plot the nodes on
%       * opt.tboxlabel = false; If the node labels should be separated
%               text boxes (true) or intrinsic graph labels (false)
%       * opt.model = 'pf'; or 'fsm' TODO comment
%
%  * Returns
%   * ret : struct containing fields
%       * ret.ntb : array with textbox handles of node numbers (if gl
%               numbering)
%       * ret.bc : array with handles of small black circle plots
%     if dualbus = 'both':
%       * ret.latg : the dots representing the load at generator buses
%                   (plot handles)
%       * ret.gatl : the dots representing the generation at effective
%                   load buses (plot handles)
%
%
% V1.3 mawolff@uos.de 30-Jun-2020


% TODO: set 'power' scale automatically if dualbus is 'both'
% TODO: better programming style for labelling
% TODO: comments

% * new structure of whole function
% - backup of V1.2 at Dissertation Chapter 2

% V1.2 mawolff@uos.de 25-Jun-2020
% * added the dualbus option

% V1.11 mawolff@uos.de 28-Aug-2018
% * added ext numbering
% * added text box label option

% V1.1 mawolff@uos.de 14-Aug-2018
% * added waitbar
% * added the axes option

% V1.0 mawolff@uos.de 19-Mar-2018

%% pseudocode


%% input handling

if( nargin < 3 )
    opt = struct();
end

nodetab = struct();
nodetab.xpos = get(p, 'xdata');
nodetab.ypos = get(p, 'ydata');

if( isfield( opt, 'ax') )
    ax = opt.ax;
else
    ax = p.Parent;
end
ax.NextPlot = 'add';      % needed to plot different markers

if( ~isfield( opt, 'pmin') ) % if options are given but not for pmin
    opt.pmin = 5;   % todo explain this in header comment
end

Pload = mpc.bus(:,3);                            % power consumed at load nodes
Pgen = mpc.gen(:,2);                % effective generated power at generator nodes
opt.pmax = max( max(abs(Pload)), max(abs(Pgen)) );  % maximum power produced or consumed at one node

opt = readOptions( opt );

switch( lower(opt.model) )
    case {'pf', 'gsm'}
        mpc = gatherGen( mpc );
    otherwise
        
end

%% code

extnum = mpc.bus(:,1);
mpc = ext2int( mpc );       % make sure that mpc is in internal order

[gbus, lbus] = getglbus( mpc );     % get generator and load buses
Ngb = numel( gbus );                % number of generator buses
Nlb = numel( lbus );                % number of load buses
Nb  = Ngb + Nlb;                    % number of buses
Ng  = size(mpc.gen,1);              % number of generators
Nge = sum( mpc.bus(:,3) < 0 );      % number of effective generators
% Nl  = sum( mpc.bus(:,3) > 0 );      % number of loads
Nle = sum( mpc.gen(:,2) < 0 );      % number of effective loads
idxLoad = mpc.bus(:,3) ~= 0;        % buses with active load
Nli = sum( idxLoad );               % number of buses with active load

% ============================= only buses  ============================= %
if( strcmpi(opt.model, 'pf' ) )         % for the 'power-flow' view
    N = Nb;
%     Ng = numel( gbus );
    nodetab.id      = zeros(N,1);
    nodetab.Pm      = zeros(N,1);
    nodetab.Peff    = zeros(N,1);
    nodetab.Pg      = zeros(N,1);
    nodetab.Pl      = zeros(N,1);
    nodetab.S       = zeros(N,1);
    nodetab.scale   = zeros(N,1);
    nodetab.color   = zeros(N,3);
    nodetab.marker  = zeros(N,1);
    nodetab.name    = cell(N,1);
    
    Pg = zeros(N,1);
    Peg = zeros(N,1);
    Pl = zeros(N,1);
    Pel = zeros(N,1);
    posLbus = 1:Nb;
    posGbus = 1:Nb;
    negLbus = find( mpc.bus(:,3) < 0 ); % buses with negative load
    negGbusi = ( mpc.gen(:,2) < 0 );    % gen buses with negative gen
    negGbus = mpc.gen( negGbusi, 1 );   % convert genbus indices to bus indices
    posLbus( negLbus ) = [];
    posGbus( negGbus ) = [];
    Peg( negLbus ) = -mpc.bus( negLbus, 3 ); % effective generation
    Pel( negGbus ) = -mpc.gen( negGbusi, 2 );
    Pl( posLbus ) = mpc.bus(posLbus,3);
    Pgg = zeros(Ng,1);
    Pgg( ~negGbusi ) = mpc.gen( ~negGbusi,2);
    Pg( gbus ) = Pgg;
    Pl = Pl + Pel;
    Pg = Pg + Peg;
    
% --------------------------- terminal buses  --------------------------- %
    nodetab.id(1:Nb)       = 1:Nb;                  % buses are labelled from 1 to Nb (seq)
    nodetab.Pm(1:Nb)       = max(abs(Pg),abs(Pl));  % largest gen or load
    nodetab.Peff(1:Nb)     = Pg-Pl;                 % effective gen or load
    nodetab.Pg(1:Nb)       = Pg;                    % generated power at each bus
    nodetab.Pl(1:Nb)       = Pl;                    % consumed power at each bus
    nodetab.S(1:Nb)        = zeros(Nb,1);           % maybe later
    
    if( strcmpi( opt.dualbus, 'effective' ) ...
            || strcmpi( opt.dualbus, 'genonly' ) )       % if 'effective' is chosen
        if( size( opt.gencolor, 1 ) == 1 )          % if common color for gens ...
            exo.dim = Ngb;                          % is given
            opt.gencolor = expand( opt.gencolor, exo ); % use it for all of them
        end
        if( size( opt.loadcolor, 1 ) == 1 )         % same for load colors
            exo.dim = Nb-Ngb;
            opt.loadcolor = expand( opt.loadcolor, exo );
        end
        
        nodetab.color(gbus,:)  = opt.gencolor;          % 
        nodetab.color(lbus,:)  = opt.loadcolor;         %
        if( isempty( opt.genscale ) )
            if( strcmpi( opt.dualbus, 'effective' ) )
                genscale = opt.genscalef( nodetab.Peff(gbus) );
            elseif( strcmpi( opt.dualbus, 'genonly' ) ) % if only gen power should be shown
                genscale = opt.genscalef( nodetab.Pg(gbus) ); % show only gen power
            end
        else
            genscale = opt.genscale;
        end
        if( isempty( opt.loadscale ) )
            loadscale = opt.loadscalef( nodetab.Peff(lbus) );
        else
            loadscale = opt.loadscale;
        end
        nodetab.scale(lbus) = loadscale;
        nodetab.scale(gbus) = genscale;
        
        nodetab.marker(lbus)   = opt.loadmark;  %
        nodetab.marker(gbus)   = opt.genmark;   %
        
        noPbus = nodetab.Peff == 0;
        nodetab.bc(1:Nb)        = false(Nb,1);           % black circles are plotted
        nodetab.bc(noPbus)      = true;                  % on busses without generation or load
        exo.dim = sum( noPbus );
        nodetab.color(noPbus,:) = expand([ 1 1 1 ],exo );
        nodetab.gc(1:Nb)        = false(Nb,1);           % effective busses dont indicate
        nodetab.lc(1:Nb)        = false(Nb,1);           % effective load or gen
        
        
    elseif( strcmpi( opt.dualbus, 'both' ) )
        effLbus = find( nodetab.Peff <= 0 );
        effGbus = find( nodetab.Peff > 0 );
        
        nodetab.scale( effLbus ) = opt.loadscalef( nodetab.Pl( effLbus ) );
        nodetab.scale( effGbus ) = opt.genscalef( nodetab.Pg( effGbus ) );
        
        exo.dim = numel( effGbus );
        opt.gencolor = expand( opt.gencolor, exo );
        nodetab.color( effGbus, : ) = opt.gencolor;
        
        exo.dim = numel( effLbus );
        opt.loadcolor = expand( opt.loadcolor, exo );
        nodetab.color( effLbus, : ) = opt.loadcolor;
        
        nodetab.marker(effLbus)   = opt.loadmark;  %
        nodetab.marker(effGbus)   = opt.genmark;   %
        
        noPbus = nodetab.Peff == 0;
        nodetab.bc(1:Nb)        = false(Nb,1);           % black circles are plotted
        nodetab.bc(noPbus)      = true;                  % on busses without generation or load
        exo.dim = sum( noPbus );
        nodetab.color(noPbus,:) = expand([ 1 1 1 ],exo );
        
        nodetab.gc        = false(Nb,1);           % effective busses dont indicate
        nodetab.lc        = false(Nb,1);           % effective load or gen
        gbusses           = false(Nb,1);
        gbusses(gbus)     = true;   % genbuss logical indexing
        nodetab.gc(1:Nb)        = and(gbusses, nodetab.Peff <= 0 );
        nodetab.lc(1:Nb)        = and(gbusses, nodetab.Peff >  0);
        nodetab.lc(1:Nb)        = and(nodetab.lc, nodetab.Pl >  0);
        
    else
        error(['The option ' ...
            opt.dualbus ...
            ' for ''dualbus'' is not known.']);
    end
    
    glnote = iseq2igl( (1:Nb)', mpc );
    
    if( strcmpi( opt.numbering, 'seq' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(i);
        end
    elseif( strcmpi( opt.numbering, 'gl' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(glnote(i));
        end
    elseif( strcmpi( opt.numbering, 'ext' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(extnum);
        end
    elseif( strcmpi( opt.numbering, 'given' ) )
        for i = 1:Nb
            nodetab.name{i} = opt.nodelabels{i};
        end
    end
    
% ========================== gathered SM model ========================== %
elseif( strcmpi(opt.model, 'gsm' ) )
    N = Nb * 2;
    nodetab.id      = zeros(N,1);
    nodetab.P       = zeros(N,1);
    nodetab.S       = zeros(N,1);
    nodetab.scale   = zeros(N,1);
    nodetab.color   = zeros(N,3);
    nodetab.marker  = zeros(N,1);
    nodetab.name    = cell(N,1);
    
% --------------------------- terminal buses  --------------------------- %
    nodetab.id(1:Nb)       = 1:Nb;          % buses are labelled from 1 to Nb (seq)
    nodetab.P(1:Nb)        = zeros(Nb,1);   % buses do not produce active
    nodetab.S(1:Nb)        = zeros(Nb,1);   % or apparent power
    nodetab.color(1:Nb,:)    = ones(Nb,3);    % all buses are white ...
    nodetab.marker(1:Nb)   = 'o';           % circles and
    nodetab.bc(1:Nb)       = true(Nb,1);    % black circles are plotted on them
    nodetab.gc(1:Nb)       = false(Nb,1);   % they have neither effective generation
    nodetab.lc(1:Nb)       = false(Nb,1);   % nor effective consumption
%     nodetab.scale(1:Nb)    = opt.pmin * ones(Nb,1); % buses have minimal scale
    nodetab.scale(1:Nb)    = opt.bcScale * ones(Nb,1); % buses have scale to 
    % fit number inside or that given by the user
    
    glnote = iseq2igl( (1:Nb)', mpc );
    
    if( strcmpi( opt.numbering, 'seq' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(i);
        end
    elseif( strcmpi( opt.numbering, 'gl' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(glnote(i));
        end
    elseif( strcmpi( opt.numbering, 'ext' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(mpc.bus(i,1));
        end
    elseif( strcmpi( opt.numbering, 'given' ) )
        for i = 1:Nb
            nodetab.name{i} = opt.nodelabels{i};
        end
    end
    
% ------------------------- internal nodes (IN) ------------------------- %
    Pin = [ mpc.gen(:,2); mpc.bus(lbus,3) ];
    Pin = [Pin [gbus;lbus]];
    Pin = sortrows(Pin,2);
    Pin = Pin(:,1);
%     Pin = [mpc.gen(:,2); mpc.bus( idxLoad, 3 )];
    Ni = Nb;
    
    if( isempty( opt.genscale ) )
        genscale = opt.genscalef( mpc.gen(:,2) );
    else
        genscale = opt.genscale;
    end
    if( isempty( opt.loadscale ) )
        loadscale = opt.loadscalef( mpc.bus( lbus, 3 ) );
    else
        loadscale = opt.loadscale;
    end
    
    if( size( opt.gencolor, 1 ) == 1 )
        exo.dim = Ng;
        opt.gencolor = expand( opt.gencolor, exo );
    end
    if( size( opt.loadcolor, 1 ) == 1 )
        exo.dim = Nlb;
        opt.loadcolor = expand( opt.loadcolor, exo );
    end
    
    raING = gbus+Nb;
    raINL = lbus+Nb;
    raIN = (Nb+1):(Nb+Ni);
    nodetab.id(raIN)       = raIN;          % IN are labelled from Nb+1 to Nb+Ni
    nodetab.P(raIN)        = Pin;           % active power at IN
    nodetab.S(raIN)        = zeros(Ni,1);   % apparent power at IN (maybe later)
    nodetab.marker(raING)  = opt.genmark;   % generator markers
    nodetab.marker(raINL)  = opt.loadmark;  % generator markers
    nodetab.bc(raIN)       = false(Ni,1);   % internal nodes have generation or load but
    nodetab.gc(raIN)       = false(Ni,1);   % they have neither effective generation
    nodetab.lc(raIN)       = false(Ni,1);   % nor effective consumption
    nodetab.scale(raING)   = genscale;      %
    nodetab.scale(raINL)   = loadscale;     %
    nodetab.color(raING,:) = opt.gencolor;  %
    nodetab.color(raINL,:) = opt.loadcolor; %
    
    if( strcmpi( opt.numbering, 'seq' ) || ...  % TODO maybe here different
            strcmpi( opt.numbering, 'gl' )  || ...  % IN in gl anyway
            strcmpi( opt.numbering, 'ext' ) )       % for compatibility: bus ext, IN gl
        for i = (Nb+1):(Nb+Ni)
            nodetab.name{i} = num2str(i);
        end
    elseif( strcmpi( opt.numbering, 'given' ) )
        for i = 1:(Nb+1):(Nb+Ni)
            nodetab.name{i} = opt.nodelabels{i};
        end
    end
    
    
% ============================ full SM model ============================ %
elseif( strcmpi(opt.model, 'fsm' ) )    % for the full SM model
    N = Nb + Ng + Nli;
    nodetab.id      = zeros(N,1);
    nodetab.P       = zeros(N,1);
    nodetab.S       = zeros(N,1);
    nodetab.scale   = zeros(N,1);
    nodetab.color   = zeros(N,3);
    nodetab.marker  = zeros(N,1);
    nodetab.name    = cell(N,1);
    
% --------------------------- terminal buses  --------------------------- %
    nodetab.id(1:Nb)       = 1:Nb;          % buses are labelled from 1 to Nb (seq)
    nodetab.P(1:Nb)        = zeros(Nb,1);   % buses do not produce active
    nodetab.S(1:Nb)        = zeros(Nb,1);   % or apparent power
    nodetab.color(1:Nb,:)    = ones(Nb,3);    % all buses are white ...
    nodetab.marker(1:Nb)   = 'o';           % circles and
    nodetab.bc(1:Nb)       = true(Nb,1);    % black circles are plotted on them
    nodetab.gc(1:Nb)       = false(Nb,1);   % they have neither effective generation
    nodetab.lc(1:Nb)       = false(Nb,1);   % nor effective consumption
%     nodetab.scale(1:Nb)    = opt.pmin * ones(Nb,1); % buses have minimal scale
    nodetab.scale(1:Nb)    = 15 * ones(Nb,1); % buses have minimal scale
    
    glnote = iseq2igl( (1:Nb)', mpc );
    
    if( strcmpi( opt.numbering, 'seq' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(i);
        end
    elseif( strcmpi( opt.numbering, 'gl' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(glnote(i));
        end
    elseif( strcmpi( opt.numbering, 'ext' ) )
        for i = 1:Nb
            nodetab.name{i} = num2str(mpc.bus(i,1));
        end
    elseif( strcmpi( opt.numbering, 'given' ) )
        for i = 1:Nb
            nodetab.name{i} = opt.nodelabels{i};
        end
    end
    
% ------------------------- internal nodes (IN) ------------------------- %
    %     [tnode, inode, ~] = getInternalNodesFSM( mpc );
    Pin = [mpc.gen(:,2); mpc.bus( idxLoad, 3 )];
    Ni = Ng + Nli;
    
    if( isempty( opt.genscale ) )
        genscale = opt.genscalef( mpc.gen(:,2) );
    else
        genscale = opt.genscale;
    end
    if( isempty( opt.loadscale ) )
        loadscale = opt.loadscalef( mpc.bus( idxLoad, 3 ) );
    else
        loadscale = opt.loadscale;
    end
    
    if( size( opt.gencolor, 1 ) == 1 )
        exo.dim = Ng;
        opt.gencolor = expand( opt.gencolor, exo );
    end
    if( size( opt.loadcolor, 1 ) == 1 )
        exo.dim = Nli;
        opt.loadcolor = expand( opt.loadcolor, exo );
    end
    
    raING = (Nb+1):(Nb+Ng);
    raINL = (Nb+Ng+1):(Nb+Ni);
    raIN = (Nb+1):(Nb+Ni);
    nodetab.id(raIN)       = raIN;          % IN are labelled from Nb+1 to Nb+Ni
    nodetab.P(raIN)        = Pin;           % active power at IN
    nodetab.S(raIN)        = zeros(Ni,1);   % apparent power at IN (maybe later)
    nodetab.marker(raING)  = opt.genmark;   % generator markers
    nodetab.marker(raINL)  = opt.loadmark;  % generator markers
    nodetab.bc(raIN)       = false(Ni,1);   % internal nodes have generation or load but
    nodetab.gc(raIN)       = false(Ni,1);   % they have neither effective generation
    nodetab.lc(raIN)       = false(Ni,1);   % nor effective consumption
    nodetab.scale(raING)   = genscale;      %
    nodetab.scale(raINL)   = loadscale;     %
    nodetab.color(raING,:) = opt.gencolor;  %
    nodetab.color(raINL,:) = opt.loadcolor; %
    
    if( strcmpi( opt.numbering, 'seq' ) || ...  % TODO maybe here different
            strcmpi( opt.numbering, 'gl' )  || ...  % IN in gl anyway
            strcmpi( opt.numbering, 'ext' ) )       % for compatibility: bus ext, IN gl
        for i = (Nb+1):(Nb+Ng+Nli)
            nodetab.name{i} = num2str(i);
        end
    elseif( strcmpi( opt.numbering, 'given' ) )
        for i = 1:(Nb+1):(Nb+Ng+Nli)
            nodetab.name{i} = opt.nodelabels{i};
        end
    end
    
else
    error(['Model ' opt.model ' is not known']);
end









% ============================= node color  ============================= %
p.NodeColor = nodetab.color;

% ============================= node scale  ============================= %
p.MarkerSize = nodetab.scale;

% ========================= plot black circles  ========================= %
cp = nodetab.bc;
ret.bc = plot( ax, nodetab.xpos( cp ), nodetab.ypos( cp ), ...
    'marker', char( nodetab.marker(find(cp,1)) ), ...
    'linestyle', 'none',...
    'color', 'k',...
    'markerfacecolor', 'none',...
    ... 'markersize', nodetab.scale( bcp ) ,... % only with for loop possible
    'markersize', nodetab.scale( find(cp,1) ) ,...
    'linewidth', 2);

% ====================== plot gen and load circles ====================== %
for i = find( shape('h',nodetab.gc) )
    ret.latg{i} = plot( ax, nodetab.xpos(i), nodetab.ypos(i), ...
        'marker', opt.genmark, ...
        'linestyle', 'none',...
        'markerfacecolor', opt.gencolor(1,:),...
        'markersize', opt.loadscalef( max(0.001,nodetab.Pg(i)) ), ...
        'linewidth', 1, ...
        'color', 'none' ...
        );
end

for i = find( shape('h',nodetab.lc) )
    ret.latg{i} = plot( ax, nodetab.xpos(i), nodetab.ypos(i), ...
        'marker', opt.loadmark, ...
        'linestyle', 'none',...
        'markerfacecolor', opt.loadcolor(1,:),...
        'markersize', opt.loadscalef( max(0.001,nodetab.Pl(i)) ) , ...
        'linewidth', 1, ...
        'color', 'none' ...
        );
end

% ============================= node labels ============================= %
if( opt.tboxlabel )
    p.NodeLabel = {};
    for i = 1:size( nodetab.name, 1 )
        pxy = val2FigCoords([p.XData(1,i) p.YData(1,i)], ax)... % die box etwas nach unten verschieben, um den
            -ones(1,2)*0.005;        % text zu zentrieren (vielleicht mit marker size arbeiten?)
        pxy = pxy + opt.tbshift;
        ret.ntb(i) = tbox( [pxy 0.01 0.01], nodetab.name{i}, opt.tbfs );
    end
    
else
    p.NodeLabel = nodetab.name;
    
end

return;

end


function [ opt ] = readOptions( opt )
if( ~isfield( opt, 'f') )
    opt.f = 0;
end
if( ~isfield( opt, 'genmark') )
    opt.genmark = 'o';
end
if( ~isfield( opt, 'loadmark') )
    opt.loadmark = 'o';
end
if( ~isfield( opt, 'gencolor') )
    opt.gencolor = [0.6902    0.0627    0.2078];
end
if( ~isfield( opt, 'loadcolor') )
    opt.loadcolor = [1.0000    0.7020    0.0824];
end
if( ~isfield( opt, 'loadscale') )
    opt.loadscale = [];
end
if( ~isfield( opt, 'loadscalef') )
    opt.loadscalef = @(x) (opt.pmin+ abs(x)/opt.pmax*20);
end
if( ~isfield( opt, 'genscale') )
    opt.genscale = [];
end
if( ~isfield( opt, 'genscalef') )
    opt.genscalef = @(x) (opt.pmin+ abs(x)/opt.pmax*20);
end
if( ~isfield( opt, 'numbering') )
    opt.numbering = 'seq';
end
if( ~isfield( opt, 'nodecolor') )
    opt.nodecolor = [];
end
if( ~isfield( opt, 'tbfs') )
    opt.tbfs = 12;
end
if( ~isfield( opt, 'tboxlabel') )
    opt.tboxlabel = false;
end
if( ~isfield( opt, 'waitbar') )
    opt.waitbar = false;
end
if( ~isfield( opt, 'tbshift') )
    opt.tbshift = [0 0];
end
if( ~isfield( opt, 'model') )
    opt.model = 'pf';
end
if( ~isfield( opt, 'dualbus') )
    opt.dualbus = 'effective';
end
if( ~isfield( opt, 'bcScale') )
    opt.bcScale = 15;
end

end

