function [ mpch, Yh ] = homMPC( mpc, opt )
%% homMPC( mpc, opt )
%   creates a homogeneous grid stored in the mpc format out of a grid
%   stored in the mpc format
%
% * Parameters:
%   * mpc: a (heterogeneous) grid stored in the mpc format
%   * opt: option struct
%       * opt.prem: print out which lines are removed because they were
%                   doubled
%       * opt.yf: admittance factor
%                   all resistances and reactances are multiplied by this
%                   factor and the susceptances is divided through yfak
%                   HINT: does nothing in combination with rxb
%       * opt.lf: load factor
%                   all loads are multiplied by this number
%       * opt.R = nan; set R manually (if empty or nan its not used)
%       * opt.X = nan; set X manually (if empty or nan its not used)
%       * opt.B = nan; set B manually (if empty or nan its not used)
%       * opt.method = 'y'; arithmetic mean of admittances (default)
%                       other methods
%                       'rxb': the mean of R, X and B separately
% * Returns:
%   * mpch: the homogeneous grid stored in the mpc format
%   * Yh : the homogeneous admittance matrix  
%
% V1.3 mawolff@uos.de 06-Apr-2018 (y-method)

% V1.2 mawolff@uos.de 06-Apr-2018 (other option struct)

% V1.1 mawolff@uos.de 07-Mar-2018 (homogenMPC)

% V1.0 mawolff@uos.de 11-Jan-2017

%% pseudocode
% * The mean value for all resistances, reactances and susceptances is
%       determined and assigned to all lines
% * All shunt elements are removed
% * The load is substracted from the generation at all generator busses 
% * All powers of the loads and generators are averaged respectively
%       this is also done for the reactive power
% * Line occuring twice are removed 
% * Transformers are "removed" (nominal values are assumed)
%
%

%% todo
%

%% code
%% handle input
if( nargin < 2)
    opt.f = 0;
end

opt = readOptions( opt );

%% mpc creation

mpch = mpc;
if( strcmp( opt.method, 'rxb') )
    if( isnan( opt.R) || isempty( opt.R) )
        resmean = mean(mpc.branch(:,3));    % mean    of resistance
        homR = resmean*opt.yf;
    else
        homR = opt.R;
    end
    if( isnan( opt.X ) || isempty( opt.X ) )
        reacmean = mean(mpc.branch(:,4));   % mean    of reactance
        homX = reacmean*opt.yf;
    else
        homX = opt.X;
    end
    if( isnan( opt.B ) || isempty( opt.B ) )
        bmean = mean(mpc.branch(:,5));      % mean    of lc susceptance
        homB = bmean/opt.yf;
    else
        homB = opt.B;
    end
    
    % Setting admittance here
    mpch.branch(:,3) = homR;                % setting the resistance
    mpch.branch(:,4) = homX;                % setting the reactance
    mpch.branch(:,5) = homB;                % setting lc
end
if( strcmp( opt.method, 'y' ) )
    if( ~(isnan( opt.R) || isempty( opt.R)) )
        mpch.branch(:,3) = opt.R;
    end
    if( ~(isnan( opt.X ) || isempty( opt.X )) )
        mpch.branch(:,4) = opt.X;
    end
    if( ~(isnan( opt.B ) || isempty( opt.B )) )
        mpch.branch(:,5) = opt.B;
    end
    
    
end



mpch.bus(:,5) = 0;                      % set shunt conductance
mpch.bus(:,6) = 0;                      % set shunt susceptance

[genbusses, loadbusses] = ...
    getglbus(mpc, '-idx');              % get indices of load and gen busses
mpch.gen(:,2) = mpch.gen(:,2) ...
    - mpch.bus(genbusses,3);            % subtract the P demand at gen busses
mpch.gen(:,3) = mpch.gen(:,3) ...
    - mpch.bus(genbusses,4);            % subtract the Q demand at gen busses
mpch.bus(genbusses,3) = 0;              % and set demand at gen busses to zero
mpch.bus(genbusses,4) = 0;              % and set demand at gen busses to zero

mpch.gen(:,2) = mean(mpch.gen(:,2));    % take the average P at genbusses 
mpch.gen(:,3) = mean(mpch.gen(:,3));    % take the average Q at genbusses

mpch.bus(:,8) = 1;                      % all voltages the same

mpch.bus(loadbusses,3) = ...                                 % set the load in 
    opt.lf*numel(genbusses)/numel(loadbusses)*mpch.gen(1,2);   % such a way, that 
mpch.bus(loadbusses,4) = ...                                 % the power is 
    opt.lf*numel(genbusses)/numel(loadbusses)*mpch.gen(1,3);   % balanced (without taking losses into account)

% ------------------------ removes doubled lines ------------------------ %
for i = size(mpch.branch,1):-1:1
    fc = mpch.branch(i,1);
    tc = mpch.branch(i,2);
    for j = size(mpch.branch,1):-1:(i+1)
        if( mpch.branch(j,1) == fc && mpch.branch(j,2) == tc )
            if( opt.prem )
                disp('remove line');
                fprintf( '%1d %1d\n',mpch.branch(j,1), mpch.branch(j,2));
            end
            mpch.branch(j,:) = [];
        end
    end
end
% ----------------------------------   ---------------------------------- %

mpch.branch(:,9) = 0;       % no tap transformer
mpch.branch(:,10) = 0;      % no tap transformer

end


function [ opt ] = readOptions( opt )
%       * opt.prem = false: print out which lines are removed because they were
%                   doubled
%       * opt.yf = 1: admittance factor
%                   all resistances and reactances are multiplied by this
%                   factor and the susceptances is divided through yfak
%                   HINT: does nothing in combination with rxb
%       * opt.lf = 1: load factor
%                   all loads are multiplied by this number
%       * opt.R = nan; set R manually (if empty or nan its not used)
%       * opt.X = nan; set X manually (if empty or nan its not used)
%       * opt.B = nan; set B manually (if empty or nan its not used)
 
 if( ~isfield( opt, 'prem') )
    opt.prem = false;
 end
 if( ~isfield( opt, 'yf') )
    opt.yf = 1;
 end
 if( ~isfield( opt, 'lf') )
    opt.lf = 1;
 end
 if( ~isfield( opt, 'R') )
    opt.R = nan;
 end
 if( ~isfield( opt, 'X') )
    opt.X = nan;
 end
 if( ~isfield( opt, 'B') )
    opt.B = nan;
 end
 if( ~isfield( opt, 'method') )
    opt.method = 'y';
 end
end


