function [ grstruct, hdl ] = loadGraph( file, opt )
%% loadGraph( file, opt )
% loadGraph( file )
% loadGraph( 'ui' )
% loadGraph( opt )
%
% To display all graphs already stored in the default path type
% loadGraph( 'show' )
% loadGraph( 'all' )
%
% To get the default path for saving and loading graphs type
% path = loadGraph( 'defpath' ); 
%
% * Parameters
%   * file : the file to read the graph from
%       * if file == 'ui', a UI window will open to choose a file
%   * opt  : option struct
%       * opt.ui = false; if a file selection via UI is requested
%
%  * Returns
%   * grstruct : output struct with fields
%       * G  : Graph struct
%       * xg : Graph nodes coordinates x-values
%       * yg : Graph nodes coordinates y-values
%       * gplot : The plot handle
%       * ntb   : possible handles of the node numbers (if not intrinsic)
%       * bc : the blacked nodes having no power/minimal inertia
%   * hdl : struct with handles
%       * hdl.fig : figure with graph plot
%       * hdl.ax  : axes containing graph plot
%
%
% V1.0 mawolff@uos.de 22-Aug-2018
 
%% pseudocode
 
 
%% input handling
if( nargin == 1)
    if( ischar( file ) )
        if( strcmp( file, 'show') || strcmp( file, 'all') )
            pth = 'graphData/';
            list = getFilesByRegex(pth,'.*');
            for i = 1:numel(list)
                disp(list{i});
            end
            return;
        end
        if( strcmp( file, 'defpath') )              % in case path is requested, the
            grstruct = 'graphData/';    % return value is the path itself
            return;
        end
    end
end

if( nargin < 2 ) 
    if( isstruct( file ) )
        opt = file;
    else
        opt.ui = false;
        if( strcmp( file, 'ui') )
            opt.ui = true;
        end
    end
end
 
 opt = readOptions( opt );
 
%% code

if( opt.ui )
    [fil, pth ] = uigetfile( {'*.mat', '.mat'; ...
        '*.*', 'all files'} );
    file = [pth fil];
end

try
    S = load( file );
    
    fig = figure;
    axes1 = axes;
    set( axes1, 'xlim', [0 100], 'ylim', [0 100]);
    axes1.Units = 'normalized';
    axpos = get( axes1, 'position');
    
    grstruct.G = S.G;
    grstruct.xg = S.xg;
    grstruct.yg = S.yg;
    gplot = S.gplot;
    
    grstruct.gplot = copyobj( gplot, axes1 );
    
    if( isfield( S, 'ntb' ) )
        if( isstruct( S.ntb ) )
            tntb = S.ntb;
            N = size(S.xg,1);
            grstruct.ntb = zeros( N, 1);
            for i = 1:size(tntb.pos,1)
                rtbpos = tntb.pos(i,:);
                rtbpos(1) = rtbpos(1)*axpos(3);
                rtbpos(2) = rtbpos(2)*axpos(4);
                rtbpos = rtbpos +[axpos(1:2) 0 0];
                grstruct.ntb(i) = tbox( rtbpos, tntb.String{i}, 12);
            end
        end
    end
    
if( isfield( S, 'bc' ) )     
    Nb = numel(S.bc);
    grstruct.bc = cell( Nb, 1);
    for i = 1:Nb
        grstruct.bc{i} = copyobj( S.bc{i}, axes1 );
    end
end

    
catch ME
    switch ME.identifier
        case 'MATLAB:load:couldNotReadFile'
            warning(['File ' ...
                file ' could not be read.']);
            return;
        case 'MATLAB:FileIO:InvalidFid'
            warning(['File ' file  ' could not be found.']);
            return;
        otherwise
            rethrow( ME );
    end
    
end

hdl.fig = fig;
hdl.ax = axes1;

return;

 
end
 
 
function [ opt ] = readOptions( opt )
 % AUTO-GENERATED option handling
 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
end
