function [ G ] = mpc2Graph( mpc, opt )
%% G = mpc2Graph( mpc, opt )
% G = mpc2Graph( mpc )
%
% make sure that the mpc is sorted according to 
% mpc.branch = sortrows( mpc.branch, [1 2]);
%
% * Parameters
%   * mpc : matpower case format
%   * opt : option struct
%       * opt.xg : nodes x-coordinates, if not fitting length they are not
%                       regarded
%       * opt.yg : nodes y-coordinates
%       * opt.lscale = @(x) x : weight scale function for lines
%       * opt.nodec : node color vector
%       * opt.dir = false; if the graph is created with directed links or
%               not
%       * opt.model = 'pf'; corresponding to which model the graph shall be
%               created :
%               * pf  : power-flow, the graph nodes correspond to the buses
% work in prog. ! gsm : (gathered) synchronous machine model, each buses is
%                       regarded as one terminal node and one internal node
%                       (SM) is added to each terminal node
%               * fsm : (full) synchronous machine model, each buses is
%                       regarded as one terminal node and each generator
%                       and load is added separately added to the terminal
%                       node
%       * opt.xd = 92.8 * P^(-1.3); the (direct) internal reactances are
%                   calculated according to Nishikawas formula when not
%                   given explicitly. The length of this vector must be
%                   equal to the number of internal nodes in the models gsm
%                   and fsm respectively. For gsm this is equal to the
%                   number of buses. For fsm it is equal to the number of
%                   generators plus the number of loads (buses with load).
%                   (per bus: first gens then load)
%       * opt.symlines = true;  if the line strength should be symmetrical
%                   or as given by the admittance matrix
%
%  * Returns
%   * G : matlab Graph struct
%
%
% V1.1 mawolff@uos.de 30-Jun-2020

% added opt.model
% V1.0 mawolff@uos.de 24-Jan-2018

%% Notes
% the labelling is [ buses:seq IN:gen IN:load ]

%% pseudocode
 
 
%% input handling

N = size( mpc.bus,1);           % get number of nodes
opt.N = N;                      % pass it to the readOptions method
opt = readOptions( opt );

%% code

mpc = ext2int( mpc );                   % get continous indices for busses and branches
y0 = makeYbus(mpc);                     % create admittance matrix
% if( opt.rdl )
mpc.branch = remDLines( mpc.branch );   % remove doubled lines (not supported by graph struct)
% end

if( opt.symlines )
    weights = zeros(size(y0,1),size(y0,2));     % predefine weights-matrix
    for i = 1:size(weights,1)                   % iterate through its upper
        for j = (i+1):size(weights,1)           % triangle to make it symmetric
            weights(i,j) = (abs(y0(i,j)) ...
                +abs(y0(j,i)))/2;               % define weight as mean of the
            weights(j,i) = weights(i,j);        % from-to and to-from admittance
        end
    end
else
    weights = abs(y0);
end


ws = zeros(numel(mpc.branch(:,1)) ,1);      % weight of single line is
for i = 1:numel(mpc.branch(:,1))            % for every line
    ws(i) = weights(mpc.branch(i,1),...     % the corresponding
        mpc.branch(i,2));                   % matrix entry
end

if( strcmpi( opt.model, 'pf' ) )
    gline = mpc.branch(:,1:2);              % graph lines (gline) are the transmission lines
    gnode = mpc.bus(:,1);                   % nodes (gnode) are the buses
elseif( strcmpi( opt.model, 'gsm' ) )
    inode = mpc.bus(:,1)+N;                 % each terminal node obtains one internal node
    gnode = [mpc.bus(:,1); inode];          % nodes (gnode) are the buses and the internal nodes
    gline = [mpc.branch(:,1:2);             % graph lines (gline) are the transmission 
        inode mpc.bus(:,1)];                % lines and the lines to the internal nodes
    % IMPORTANT: for the current programming the other direction
    % [mpc.bus -> inode] does not work
    xdf = @(P) 92.8 * abs(P).^(-1.3);
    gmpc = gatherGen( mpc );
    [gbus, lbus] = getglbus( gmpc );
    P = [ gmpc.gen(:,2); gmpc.bus(lbus,3) ];
    P = [P [gbus;lbus]];
    P = sortrows(P,2);
    P = P(:,1);
    wsi = max(0.1, 1./xdf(P));
    ws = [ ws; wsi ];
elseif( strcmpi( opt.model, 'fsm' ) )
    [tnode, inode, wsi] = ...               % get the internal nodes and the
        getInternalNodesFSM( mpc );         % terminal nodes corresponding to each internal node
    gline = [mpc.branch(:,1:2); [inode tnode]];    % graph lines (gline) are the transmission lines
    gnode = [mpc.bus(:,1); inode ];              % nodes (gnode) are the buses
    wsi = max(0.1, wsi); % TODO : opt.min weight
    ws = [ws; wsi];
else
    error(['The model ' opt.model 'is not known'] );
end

Nall = size(gnode,1);
if( ~isfield( opt, 'nodec') )
    opt.nodec = 0.7*ones(Nall,1);
end


nodeName = cell(Nall,1);                  % nodeName must be cell
for i = 1:Nall                            % iterate through all nodes
    nodeName{i} = num2str(gnode(i,1));   % give them the corresponding number as name
end

if( numel(opt.xg) < opt.N )
    nodes = table(nodeName, gnode, opt.nodec, ...
        'VariableNames', {'Name','NodeLabel','NodeColor'} );
else
    nodes = table(nodeName, opt.xg, opt.yg, gnode,opt.nodec, ...
        'VariableNames', {'Name','XCoord','YCoord','NodeLabel','NodeColor'} );
end
if( opt.multi )
    edges = table([gline(:,1) gline(:,2)], opt.lscale(ws),...
        linenum,...
        'VariableNames',{'EndNodes', 'Weight', 'Multi'});
else
    edges = table([gline(:,1) gline(:,2)], opt.lscale(ws),...
        'VariableNames',{'EndNodes', 'Weight'});
end
if( opt.dir )
    G = digraph( edges, nodes);
else
    G = graph( edges, nodes);
end
return;

end
 
function [ opt ] = readOptions( opt )

 if( ~isfield( opt, 'f') )
    opt.f = 0;
 end
 if( ~isfield( opt, 'xg') )
    opt.xg = 0;
 end
 if( ~isfield( opt, 'yg') )
    opt.yg = 0;
 end
 if( ~isfield( opt, 'lscale') )
    opt.lscale = @(x) x;
 end
 if( ~isfield( opt, 'dir') )
    opt.dir = false;
 end
 if( ~isfield( opt, 'multi') )
    opt.multi = false;
 end
 if( ~isfield( opt, 'model') )
    opt.model = 'pf';
 end
 if( ~isfield( opt, 'symlines') )
    opt.symlines = true;
 end
%  if( ~isfield( opt, 'rdl') )    % maybe in later version
%     opt.rdl = true;               % duplicate edges not supported by
%     given matlab graph struct
%  end
end
