function [ moms, mu, sig ] = pdfmom( x, pdf, opt )
%% pdfmom( pdf, mom)
%  Calculates the first moments of the given pdf(x).
% * Parameters:
%   * x: the x-values
%   * pdf: the corresponding y-values, the probability density (pdf).
%   * (opt): option struct
%   * opt.nm: how many moments should be calculated. If not given the mean
%       and standard deviation is calculated. If sig is requested this is
%       set to max(nm,2).
%   * opt.cen = false; if central moments should be calculated (the mean is
%                   subtracted from PDF)
%
% * returns:
%   * moms: the moments in a row vector
%   * (mu): the mean
%   * (sig): the standard deviation
%
% V1.0 mawolff@uos.de 09-Jun-2019

% V1.0 mawolff@uos.de 09-Jun-2019
% option struct added
% central moments option added
% V1.0 mawolff@uos.de 17-Feb-2017

%% input handling
% if( nargin == 2 ) 
%     nm = 2;      
% end
% if( nargout > 2)
%     nm = max(nm,2);
% end
if( nargin < 3 )
    opt.nm = 2;
end

opt = readOptions( opt );

%% code

N = sum(pdf);
moms = zeros(1,opt.nm);


moms(1) = 1/N*sum(x.*pdf);
mu = moms(1);
if( opt.cen )
    for i = 2:opt.nm
        moms(i) = 1/N*sum((x-mu).^(i).*pdf);
    end
    sig = sqrt(moms(2));
else
    for i = 2:opt.nm
        moms(i) = 1/N*sum(x.^(i).*pdf);
    end
    sig = sqrt( moms(2) - moms(1)^2 );
end


return;
end

function [ opt ] = readOptions( opt )
 
 if( ~isfield( opt, 'nm') ) % if number of mom not given
    opt.nm = 2;             % calculate only mean and standard dev
 else                       % otherwise
    opt.nm = max(opt.nm,2); % calculate at least mean and standard dev
 end
 
 if( ~isfield( opt, 'cen' ) )   % if not given if central moments needed
     opt.cen = false;           % calculate not central moments as in version 1.0
 end                            % (better for compatibility this way)
end

% example
% npdf = @(x,m,s) (2*pi*s^2).^(-0.5) *exp(-0.5* ((x-m)/s).^2);
% 
% x = -100:0.1:100;
% y1 = npdf(x,3,4);
% y2 = npdf(x,3,7);
% y3 = npdf(x,6,6);
% [a,b,c] = pdfmom(x,y1);
% disp([a b c]);
% [a,b,c] = pdfmom(x,y2);
% disp([a b c]);
% [a,b,c] = pdfmom(x,y3);
% disp([a b c]);