function [ dynmod, par ] = raw2swing( mpc, opt )
%% raw2swing( mpc, opt )
% Creates the parameters for the dynamical models (EN,SM,SP(not yet)) from
% the raw mpc struct.
% If desired a homogenization can be choosen.
% The internal nodes can be enabled or disabled.
%
%
% * Parameters
%   * mpc : grid parameters in a mpc table form
%   * opt : option struct   
%     opt.xdmax = 1;        maximum of internal reactance
%     opt.dynmodel = 'en';  choose dynamical model EN, SM, SP
%                           (not casesensitive)
%     opt.IN = true;        if internal nodes are used
%     opt.splitP = true;    distinguish between the real mechanical power 
%                           and the losses of the diagonal terms
%     opt.xdf = 1;          use the full internal nodes this can be changes
%                           to analyse impact
%     opt.hom = false;      choose homogenization or not
%     opt.hetpar = 1e-6;    if homogenization choosen how to interpolate 
%                           between complete homogenization hetpar approx
%                           0, almost heterogen (hetpar approx 1) 
%                           or something in between
%     opt.redpass = true;   reduce passive nodes (nodes that do not consume
%                           or produce active or reactive power. Hence they
%                           are not modelled as synchronous machines.
%     opt.losses = false;   when homogenizing, the losses are removed by
%                           default. Does not affect heterogeneous grid.
%     opt.ratedS = [];      when given, the parameters of the swing
%                           equation are calculated by these values rather
%                           than the mechanical power Pm. Leave empty if
%                           not requested. The value must be given in MVA
%                           (10^6 VA).
%                           For EN model: only generator ratedS
%                           For SM model: all ratedS (gen and motor)
%           
%                           There is the possibility to define explicitly
%     opt.xdg = nan(Ng,1);  the transient reactance for generators
%     opt.xdl = nan(Nl,1);  the transient reactance for loads
%     opt.Hg = nan(Ng,1);   the inertia for generators
%     opt.Hl = nan(Nl,1);   the inertia for loads
%
%  * Returns
%   * dynmod : struct containing all parameters for the dynamic model 
%               all parameters are in gl-notation (all generator nodes
%               first then all load nodes)
%       * dynmod.H
%       * dynmod.P
%       * dynmod.Pd
%       * dynmod.A
%       * dynmod.gamma
%       * dynmod.K
%       * dynmod.ssa (in rad)
%   * par
%       * par.Ybus   : final admittance matrix (in seq-notation)
%       * par.rmpc   : resulting mpc struct
%           WARNING: only accurate when passive nodes are not reduced
%       * par.remNodes : passive nodes removed from mpc
%
% HINT: doubled lines in the mpc file are corretely taken into account when
% constructing Ybus (the admittance is approx. twice as strong).
%
% V1.3 mawolff@uos.de 03-Feb-2021


% added the option to set rated power values for a correct treatment of
% synchronous condensers and the reference bus
% V1.2 mawolff@uos.de 24-Jun-2020
% updated newtonpf to version 7.0 and fixed Sbus issues (bugged before but,
% however, functional)
% V1.1 mawolff@uos.de 31-Jan-2019
% newtonpf_int added to ensure version stability
% V1.0 mawolff@uos.de 14-Sep-2018
 
%% pseudocode
 
 
%% input handling
if( nargin < 2 ) 
   opt.f = 0; 
end
 
 opt = readOptions( opt, mpc );
 
%% code

% ============================= initialize  ============================= %
hetpar = opt.hetpar;    % heterogeneity parameter hetpar = 0: homogen 
                        % hetpar = 1: (completely) heterogen
% lf = 1;                 % load factor to balance losses, see hombus
                        % function
% <<<<< todo make flag with and without losses

% define named indices into bus, gen, branch matrices
BUS_I = 1;          % bus indices
PD = 3;             % active power demand
QD = 4;             % reactive power demand
QT = 17;            % 
GEN_BUS = 1;        % bus numbers containing generators

mpopt = mpoption;   % use default matpower options


if size(mpc.branch,2) < QT              % add zero columns to branch for flows if needed
  mpc.branch = [ mpc.branch ...
      zeros(size(mpc.branch, 1), ...
      QT-size(mpc.branch,2)) ];
end

ampc = mpc;                             % copy for the mpc without passive nodes ( only active)
mpc = ext2int(mpc);                     % convert to internal indexing
[baseMVA, bus, gen, branch] = ...       % extract fields for easier usage and debuging
    deal(mpc.baseMVA, mpc.bus, ...
    mpc.gen, mpc.branch);

[V0, ref, pv, pq] = initCVolt( bus, gen );

[genbusses, loadbusses] = ...
    getglbus(mpc);          % get indices of load and gen busses

% ========================= treat passive nodes ========================= %
if( opt.redpass )                       % if passive nodes should be reduced
    passive = true( size(bus,1), 1 );   % get all nodes and remove those producing or consuming power
    passive( gen(:,GEN_BUS) ) = false;  % generators producing (complex) power are not passive
    passive(  and( ...                  % all buses 
        (abs(mpc.bus(:,PD)) ~= 0), ...      % consuming active 
        (abs(mpc.bus(:,QD)) ~= 0) )  ) ...  % or reactive power
        = false;                            % are not passive
    passext = ampc.bus(passive,BUS_I);      % bus names of passive nodes
    remNodes = NaN( size( mpc.bus, 1), 1);  % store removed nodes
    for i = 1:numel( passext )              % for all passive nodes
        [ampc, ifo] = remNodeMPC( passext(i), ampc ); % remove the node
        remNodes(i) = ifo.busrow + i - 1;             % store the number
    end
    ampc = ext2int(ampc);               % convert to internal indexing
    [abaseMVA, abus, agen, abranch] = deal(ampc.baseMVA, ampc.bus, ...
        ampc.gen, ampc.branch);
    [agenbusses, aloadbusses] = ...
        getglbus(ampc);         % get indices of load and gen busses
    [aV0, aref, apv, apq] ...
        = initCVolt( abus, agen );
else                                    % if not requested
    remNodes = [];                      % no nodes are removed
end

% ============================= homogenize  ============================= %
if( opt.hom )

    [bus, gen] = hombus( ...                        % homogenize all busses
        bus, gen, loadbusses, genbusses, hetpar );
    if( opt.redpass )                               % if passive reduction requested
        [abus, agen] = hombus( ...                  % also the reduced grid is homogenized
            abus, agen, aloadbusses, agenbusses, hetpar );  % both are needed later
    end

if( hetpar < 0.5 )          % no real reason for this <<<<<<<<< TODO
    branch(:,9) = 0;        % no tap transformer
    branch(:,10) = 0;       % no tap transformer
end
if( ~opt.losses )           % if no losses requested
    branch(:,3) = 0;        % delete all resistances
end

[Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);
[Ybus, Yf, Yt] = homY( Ybus, Yf, Yt, hetpar );      % homogenize Y matrix
% creates the admittance matrix from the table input values
% the value can be scaled from original values (heterogen) to all the same
% (homogenized)
% busses are here needed just for the shunt elements and to check if
% it is internal numbering

else        % if no homogenization requested simply build admittance matrix
    [Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);
end

% ======================== calculate power flow  ======================== %
if( opt.redpass )                           % without passive nodes
    Sbus = @(Vm) makeSbus(abaseMVA, abus, agen, mpopt, Vm );
    
    Yaa = Ybus( ~passive, ~passive );
    Yab = Ybus( ~passive,  passive );
    Yba = Ybus(  passive, ~passive );
    Ybb = Ybus(  passive,  passive );
    Ybus = Yaa - Yab*(Ybb\Yba);
    
    [V, success, iterations] = newtonpf_int( ...    % steps closer to solution via linearization
        Ybus, Sbus, aV0, aref, apv, apq, mpopt);    % to find power flow
    % Use _int (intern) NR method to ensure version stability
    
%     [abus, agen] = pfsoln( ...            % adjusts the references bus power such that
%         abaseMVA, abus, agen, V, Ybus, ...  % the power balance holds in this iteration step
%         aref, apv, apq);
    [abus, agen, abranch] = pfsoln( ...            % adjusts the references bus power such that
        baseMVA, abus, agen, abranch, Ybus, Yf, ...% the power balance holds in this iteration step
        Yt, V, aref, apv, apq);
    
else                                        % or with passive nodes
    Sbus = @(Vm) makeSbus(baseMVA, bus, gen, mpopt, Vm );    % compute complex bus power injections (generation - load)

    [V, success, iterations] = newtonpf_int( ...% steps closer to solution via linearization
        Ybus, Sbus, V0, ref, pv, pq, mpopt);    % to find power flow
    
    [bus, gen, branch] = pfsoln( ...            % adjusts the references bus power such that
        baseMVA, bus, gen, branch, Ybus, Yf, ...% the power balance holds in this iteration step
        Yt, V, ref, pv, pq);
end

if( ~success )
    error('Power flow could not be solved.');
end

if( nargout > 1)                            % create result mpc struct if requested
    [mpc.bus, mpc.gen, mpc.branch] = ...
        deal(bus, gen, branch);
    par.rmpc = int2ext(mpc);
end

% ===================== create dynamical parameters ===================== %
if( opt.redpass )
    pfsol.bus = abus;                       % collect the power flow results in a struct
    pfsol.gen = agen;                       % to pass it to the dynamic parameter method
    pfsol.Ybus = Ybus;
    pfsol.baseMVA = abaseMVA;
    % give the x_d here by setting
    Pmwg = agen(:,2);                       % power at generator busses in MW
    Pmwl = abus(aloadbusses,3);             % power at load busses in MW
else
    pfsol.bus = bus;                        % collect the power flow results in a struct
    pfsol.gen = gen;                        % to pass it to the dynamic parameter method
    pfsol.Ybus = Ybus;
    pfsol.baseMVA = baseMVA;
    % give the x_d here by setting
    Pmwg = gen(:,2);                        % power at generator busses in MW
    Pmwl = bus(loadbusses,3);               % power at load busses in MW
end

x_d_max = opt.xdmax;                        % maximum of internal reactance (default = 1.0)
xdp = @(P) min([92.8*abs(P).^(-1.3), ...    % Nishikawas formula
    x_d_max*ones(size(P))], [], 2);
H_min = 0.1;                                % minimum inertia
Hest = @(P) max([0.04*P, H_min*ones(size(P))], [], 2);  % Nishikawas formula
if( ~isempty( opt.ratedS ) )                % if rated powers are given
    switch upper( opt.dynmodel )            % depeding on model
        case 'SM'                                       % in synchronous machine model 
            xdg = xdp( opt.ratedS(genbusses) );         % calculate the transient reactances
            xdl = xdp( opt.ratedS(loadbusses) );        % for both gens and loads
            pfsol.Hg = Hest( opt.ratedS(genbusses) );   % and interias for 
            pfsol.Hl = Hest( opt.ratedS(loadbusses) );  % both gens and loads
        case 'EN'                                       % in effective network model
            xdg = xdp( opt.ratedS );                    % transient reactances for gens
            pfsol.Hg = Hest( opt.ratedS );              % inertias for gens
        otherwise                                       % for other models (SP)
            error(['rated power in connection' ...      % nothing implemented yet
                ' with this model' ...  
                ' not yet implemented'] );
    end
else                                        % if no rated powers are given
    xdg = xdp( Pmwg );                      % estimate the internal reactance for generator
    xdl = xdp( Pmwl );                      % and load nodes depending on 
                                            % current active power production and consumption
end
xdf = opt.xdf;                              % use factor 1 or the one given by user
xgnan = isnan(opt.xdg);                     % determine not given transient reactances (for gens)
opt.xdg(xgnan) = xdg(xgnan);                % fill them with calculated ones
pfsol.xdg  = xdf*opt.xdg;                   % scale xd with the factor xdf 
par.xdin = opt.xdg;
if( ~strcmpi( opt.dynmodel, 'en' ) )
    xlnan = isnan(opt.xdl);                     % determine not given transient reactances (for gens)
    opt.xdl(xlnan) = xdl(xlnan);                % fill them with calculated ones
    pfsol.xdl = xdf*opt.xdl;                        % to analyse impact
    par.xdin = [opt.xdg; opt.xdl];
end

switch upper( opt.dynmodel )
    case 'SM'
        [sm, det] = PF2SM_model( pfsol, opt );  % splitting en, sm and dynmod not necessary (atm)
%         sm.ssa = angle(det.E);
        dynmod = sm;
        dynmod.Vm = abs(det.E);
        Ydyn = det.Y_SM;
    case 'EN'
        [en, det] = PF2EN_model( pfsol, opt );  
%         en.ssa = angle(det.E);
        dynmod = en;
        dynmod.Vm = abs(det.E);
        Ydyn = det.Y_EN;
    otherwise
        error( 'not implemented yet or does not exist' );
end

if( opt.splitP )
    dynmod.P = dynmod.Pm;           
    dynmod.Pd = full(dynmod.Pd);    
else
    dynmod.P = dynmod.A;    
    dynmod.Pd = 0*dynmod.A; 
end
if( nargout > 1 )
    par.Ybus = Ybus;
    par.Y_dyn = Ydyn;
    par.Y0_dyn = det.Y0;
    par.xdout = det.x_d;
    par.remNodes = remNodes(~isnan(remNodes));
end
return;

end
 
 
function [ opt ] = readOptions( opt, mpc )

Nn = size(mpc.bus,1);               % number of nodes
Ng = size(mpc.gen,1);               % number of generators
Nl = Nn-Ng;                         % number of loads

 if( ~isfield( opt, 'dynmodel') )   % the EN model as dynamical model
    opt.dynmodel = 'en';            % (only generators have inertia dynamics)
 end
 if( ~isfield( opt, 'IN') )         % use internal nodes
    opt.IN = true;                  % 
 end
 if( ~isfield( opt, 'splitP') )     % distinguish between the real mechanical
    opt.splitP = true;              % power and the losses of the diagonal terms
 end
 if( ~isfield( opt, 'hom') )        % choose at default no homogenization
    opt.hom = false;                %
 end
 if( ~isfield( opt, 'hetpar') )     % if homogenization choosen 
    opt.hetpar = 1e-6;              % use approximately the complete 
 end                                % homogenization (het approx 0)
 if( ~isfield( opt, 'losses') )     % if homogenization choosen 
    opt.losses = false;             % as default dont use losses
 end                                % resulting in reference bus also being homogenized
 if( ~isfield( opt, 'redpass') )    % by default
    opt.redpass = true;             % set flag to reduce passive nodes (P^m = 0)
 end                                % 
 if( ~isfield( opt, 'ratedS') )     % if no rated (apparent) power is given
    opt.ratedS = [];                % create empty field
 end       
                                    % if nothing different requested use
 if( ~isfield( opt, 'xdmax') )      % maximum of internal reactance
    opt.xdmax = 1;                  % as suggested by Nishikawa (as 1)
 end
 if( ~isfield( opt, 'xdf') )        % use the full internal nodes
    opt.xdf = 1;                    % this can be changes to analyse impact
 end
 if( ~isfield( opt, 'xdg') )        % if no transient reactances are given explicitly
     opt.xdg = nan(Ng,1);           % create field filled by nan
 end
 if( ~isfield( opt, 'xdl') )        % if no transient reactances are given explicitly
     opt.xdl = nan(Nl,1);           % create field filled by nan
 end
 if( ~isfield( opt, 'Hg') )         % if no inertias are given explicitly
     opt.Hg = nan(Ng,1);            % create field filled by nan
 end
 if( ~isfield( opt, 'Hl') )         % if no transient reactances are given explicitly
     opt.Hl = nan(Nl,1);            % create field filled by nan
 end
end


function [ Ybus, Yf, Yt ] = homY( Ybus, Yf, Yt, hetpar )

Nn = size(Ybus,1);          % num of nodes
Nl = size(Yf,1);            % num of lines

Yhet = tri( Ybus );         % use every line only once
Yhet = nodiag( Yhet );      % dont use the self admittances

Yconmask = abs(Ybus) > 0;   % get matrix with all real connections to mask interpolation later
Yfconmask = abs(Yf) > 0;    % get matrix with all real "from" connections to mask interpolation later
Ytconmask = abs(Yt) > 0;    % get matrix with all real "to" connections to mask interpolation later
Ycon = abs(Yhet) > 0;       % get indices representing an actual connecting line
Ym = mean( Yhet(Ycon) ) ;   % get the mean admittance of all these lines

% scale between homogenized and heterogen

Yhdiag = - sumd( nodiag( ...
    Yconmask *Ym), 'h' );               % calculalte diagonal elements
Ydiag = diag( Ybus );                   % 
ydiagres = (1-hetpar)*Yhdiag + ...
    hetpar*Ydiag;

Yvec = reshape( Ybus, Nn*Nn, 1);        % calculate the homogenized admittance matrix
Yvec = (1-hetpar)*Ym + hetpar*Yvec;
yres = reshape( Yvec, Nn, Nn );
yres( logical(eye(Nn)) ) = ydiagres;
yres( ~Yconmask ) = 0;

Yfvec = reshape( Yf, Nn*Nl, 1);         % calc the from matrix
Yfvec = (1-hetpar)*Ym + hetpar*Yfvec;
yfres = reshape( Yfvec, Nl, Nn );
yfres( ~Yfconmask ) = 0;

Ytvec = reshape( Yt, Nn*Nl, 1);         % calc the to matrix
Ytvec = (1-hetpar)*Ym + hetpar*Ytvec;
ytres = reshape( Ytvec, Nl, Nn );
ytres( ~Ytconmask ) = 0;

% calc the diag elements of homogeneous matrix and make slide diff to het
% Yh( logical(eye( size(Yh,1) ) ) ) = ... % the self admittances are the sum
%     -sumd(Yh, 'h');                     % of off-diagonal elements (laplacian matrix)

Ybus = yres;
Yf = yfres;
Yt = ytres;

return;
end

function [ bus, gen ] = hombus( bus, gen, lbus, gbus, hetpar )

lf = 1;     % load factor to compensate losses in transmission lines
%   this is not relevant yet. For further studies this can be implemented
%   or better done by a factor for all generators since physically load is
%   fixed and generators can be adjusted

ngb = numel(gbus);
nlb = numel(lbus);
nb  = ngb + nlb;

define_constants

bus(:,GS) = 0;                      % set shunt conductance
bus(:,BS) = 0;                      % set shunt susceptance

gen(:,PG) = gen(:,PG) ...
    - bus(gbus,PD);                 % subtract the P demand at gen busses
% gen(:,QG) = gen(:,QG) ...
%     - bus(gbus,QD);                 % subtract the Q demand at gen busses
bus(gbus,PD) = 0;                   % and set demand at gen busses to zero

Pm = mean(gen(:,PG));               % calc average power generation
% Qm = mean(gen(:,QG));               % calc average reactive power 
Qlm = lf*mean(bus(:,QD));              % calc average reactive power 
Qm = nlb/ngb*Qlm;
gen(:,PG) = (1-hetpar)*Pm + ...     % interpolate hom P with het P
    hetpar*gen(:,PG);               
gen(:,QG) = (1-hetpar)*Qm + ...     % interpolate hom Q with het Q
    hetpar*gen(:,QG);               

bus(gbus,QD) = 0;                   % and set demand at gen busses to zero

bus(:,VM) = (1-hetpar)*1 + ...      % interpolate between hom 
    hetpar*bus(:,VM);           
                                    % homogenized voltage all at 1p.u. and
                                    % heterogen as given

Plm = lf*ngb/nlb*Pm;     % averaged load P demand
% Qlm = lf*ngb/nlb*Qm;     % averaged load Q demand
bus(lbus,PD) = (1-hetpar)*Plm + ...      % in such a way that power is 
    hetpar*bus(lbus,PD);                 % balanced without losses
bus(lbus,QD) = (1-hetpar)*nb/nlb*Qlm + ...
    hetpar*bus(lbus,QD);

return;
end


function [ V0, ref, pv, pq ] = initCVolt(bus, gen)

define_constants

% get bus index lists of each type of bus
[ref, pv, pq] = bustypes(bus, gen);

% generator info
on = find(gen(:, GEN_STATUS) > 0);      %% which generators are on?
gbus = gen(on, GEN_BUS);                %% what buses are they at?


% % initial state
% V0    = ones(size(bus, 1), 1);        %% flat start
V0  = bus(:, VM) .* exp(sqrt(-1) * pi/180 * bus(:, VA));
vcb = ones(size(V0));           %% create mask of voltage-controlled buses
vcb(pq) = 0;                    %% exclude PQ buses
k = find(vcb(gbus));            %% in-service gens at v-c buses
V0(gbus(k)) = gen(on(k), VG) ./ abs(V0(gbus(k))).* V0(gbus(k));
%   mawolff % normalize V0 for active generators?

return;
end

function [V, converged, i] = newtonpf_int(Ybus, Sbus, V0, ref, pv, pq, mpopt)
%NEWTONPF  Solves the power flow using a full Newton's method.
% newtonpf_int is the internal method (copied from MATPOWER) to ensure
% version stability
%   [V, CONVERGED, I] = NEWTONPF(YBUS, SBUS, V0, REF, PV, PQ, MPOPT)
%   solves for bus voltages given the full system admittance matrix (for
%   all buses), the complex bus power injection vector (for all buses),
%   the initial vector of complex bus voltages, and column vectors with
%   the lists of bus indices for the swing bus, PV buses, and PQ buses,
%   respectively. The bus voltage vector contains the set point for
%   generator (including ref bus) buses, and the reference angle of the
%   swing bus, as well as an initial guess for remaining magnitudes and
%   angles. MPOPT is a MATPOWER options struct which can be used to 
%   set the termination tolerance, maximum number of iterations, and 
%   output options (see MPOPTION for details). Uses default options if
%   this parameter is not given. Returns the final complex voltages, a
%   flag which indicates whether it converged or not, and the number of
%   iterations performed.
%
%   See also RUNPF.

%   MATPOWER
%   Copyright (c) 1996-2015 by Power System Engineering Research Center (PSERC)
%   by Ray Zimmerman, PSERC Cornell
%
%   newtonpf.m 2644 2015-03-11 19:34:22Z ray
%
%   This file is part of MATPOWER.
%   Covered by the 3-clause BSD License (see LICENSE file for details).
%   See http://www.pserc.cornell.edu/matpower/ for more info.

%% default arguments
if nargin < 7
    mpopt = mpoption;
end

%% options
tol     = mpopt.pf.tol;
max_it  = mpopt.pf.nr.max_it;

%% initialize
converged = 0;
i = 0;
V = V0;
Va = angle(V);
Vm = abs(V);

%% set up indexing for updating V
npv = length(pv);
npq = length(pq);
j1 = 1;         j2 = npv;           %% j1:j2 - V angle of pv buses
j3 = j2 + 1;    j4 = j2 + npq;      %% j3:j4 - V angle of pq buses
j5 = j4 + 1;    j6 = j4 + npq;      %% j5:j6 - V mag of pq buses

%% evaluate F(x0)
mis = V .* conj(Ybus * V) - Sbus(Vm);
F = [   real(mis([pv; pq]));
        imag(mis(pq))   ];

%% check tolerance
normF = norm(F, inf);
if mpopt.verbose > 1
    fprintf('\n it    max P & Q mismatch (p.u.)');
    fprintf('\n----  ---------------------------');
    fprintf('\n%3d        %10.3e', i, normF);
end
if normF < tol
    converged = 1;
    if mpopt.verbose > 1
        fprintf('\nConverged!\n');
    end
end

%% do Newton iterations
% dxsu = zeros( max_it+1, numel(F) );
% dxsu(1,:) = F;
while (~converged && i < max_it)
    %% update iteration counter
    i = i + 1;
    
    %% evaluate Jacobian
%     [dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V);
% version 7.0
    [dSbus_dVa, dSbus_dVm] = dSbus_dV(Ybus, V);
    [dummy, neg_dSd_dVm] = Sbus(Vm);
    dSbus_dVm = dSbus_dVm - neg_dSd_dVm;
    
    j11 = real(dSbus_dVa([pv; pq], [pv; pq]));
    j12 = real(dSbus_dVm([pv; pq], pq));
    j21 = imag(dSbus_dVa(pq, [pv; pq]));
    j22 = imag(dSbus_dVm(pq, pq));
    
    J = [   j11 j12;
            j21 j22;    ];

    %% compute update step
    dx = -(J \ F);

    %% update voltage
    if npv
        Va(pv) = Va(pv) + dx(j1:j2);
    end
    if npq
        Va(pq) = Va(pq) + dx(j3:j4);
        Vm(pq) = Vm(pq) + dx(j5:j6);
    end
    V = Vm .* exp(1j * Va);
    Vm = abs(V);            %% update Vm and Va again in case
    Va = angle(V);          %% we wrapped around with a negative Vm

    %% evalute F(x)
    mis = V .* conj(Ybus * V) - Sbus(Vm);
    F = [   real(mis(pv));
            real(mis(pq));
            imag(mis(pq))   ];
% dxsu(i+1,:) = F;

    %% check for convergence
    normF = norm(F, inf);
    if mpopt.verbose > 1
        fprintf('\n%3d        %10.3e', i, normF);
    end
    if normF < tol
        converged = 1;
        if mpopt.verbose
            fprintf('\nNewton''s method power flow converged in %d iterations.\n', i);
        end
    end
end
% assignin('base','dxsu', dxsu );

if mpopt.verbose
    if ~converged
        fprintf('\nNewton''s method power flow did not converge in %d iterations.\n', i);
    end
end

end

% use this method if an error due to version change occurs
% function [dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V)
% %DSBUS_DV   Computes partial derivatives of power injection w.r.t. voltage.
% %   [DSBUS_DVM, DSBUS_DVA] = DSBUS_DV(YBUS, V) returns two matrices containing
% %   partial derivatives of the complex bus power injections w.r.t voltage
% %   magnitude and voltage angle respectively (for all buses). If YBUS is a
% %   sparse matrix, the return values will be also. The following explains
% %   the expressions used to form the matrices:
% %
% %   S = diag(V) * conj(Ibus) = diag(conj(Ibus)) * V
% %
% %   Partials of V & Ibus w.r.t. voltage magnitudes
% %       dV/dVm = diag(V./abs(V))
% %       dI/dVm = Ybus * dV/dVm = Ybus * diag(V./abs(V))
% %
% %   Partials of V & Ibus w.r.t. voltage angles
% %       dV/dVa = j * diag(V)
% %       dI/dVa = Ybus * dV/dVa = Ybus * j * diag(V)
% %
% %   Partials of S w.r.t. voltage magnitudes
% %       dS/dVm = diag(V) * conj(dI/dVm) + diag(conj(Ibus)) * dV/dVm
% %              = diag(V) * conj(Ybus * diag(V./abs(V)))
% %                                       + conj(diag(Ibus)) * diag(V./abs(V))
% %
% %   Partials of S w.r.t. voltage angles
% %       dS/dVa = diag(V) * conj(dI/dVa) + diag(conj(Ibus)) * dV/dVa
% %              = diag(V) * conj(Ybus * j * diag(V))
% %                                       + conj(diag(Ibus)) * j * diag(V)
% %              = -j * diag(V) * conj(Ybus * diag(V))
% %                                       + conj(diag(Ibus)) * j * diag(V)
% %              = j * diag(V) * conj(diag(Ibus) - Ybus * diag(V))
% %
% %   Example:
% %       [Ybus, Yf, Yt] = makeYbus(baseMVA, bus, branch);
% %       [dSbus_dVm, dSbus_dVa] = dSbus_dV(Ybus, V);
% %
% %   For more details on the derivations behind the derivative code used
% %   in MATPOWER information, see:
% %
% %   [TN2]  R. D. Zimmerman, "AC Power Flows, Generalized OPF Costs and
% %          their Derivatives using Complex Matrix Notation", MATPOWER
% %          Technical Note 2, February 2010.
% %             http://www.pserc.cornell.edu/matpower/TN2-OPF-Derivatives.pdf
% 
% %   MATPOWER
% %   Copyright (c) 1996-2015 by Power System Engineering Research Center (PSERC)
% %   by Ray Zimmerman, PSERC Cornell
% %
% %   $Id: dSbus_dV.m 2644 2015-03-11 19:34:22Z ray $
% %
% %   This file is part of MATPOWER.
% %   Covered by the 3-clause BSD License (see LICENSE file for details).
% %   See http://www.pserc.cornell.edu/matpower/ for more info.
% 
% n = length(V);
% Ibus = Ybus * V;
% 
% if issparse(Ybus)           %% sparse version (if Ybus is sparse)
%     diagV       = sparse(1:n, 1:n, V, n, n);
%     diagIbus    = sparse(1:n, 1:n, Ibus, n, n);
%     diagVnorm   = sparse(1:n, 1:n, V./abs(V), n, n);
% else                        %% dense version
%     diagV       = diag(V);
%     diagIbus    = diag(Ibus);
%     diagVnorm   = diag(V./abs(V));
% end
% 
% dSbus_dVm = diagV * conj(Ybus * diagVnorm) + conj(diagIbus) * diagVnorm;
% dSbus_dVa = 1j * diagV * conj(diagIbus - Ybus * diagV);
%
% end


%% define named indices into bus, gen, branch matrices
% [PQ, PV, REF, NONE, BUS_I, BUS_TYPE, PD, QD, GS, BS, BUS_AREA, VM, ...
%     VA, BASE_KV, ZONE, VMAX, VMIN, LAM_P, LAM_Q, MU_VMAX, MU_VMIN] = idx_bus;
% [F_BUS, T_BUS, BR_R, BR_X, BR_B, RATE_A, RATE_B, RATE_C, ...
%     TAP, SHIFT, BR_STATUS, PF, QF, PT, QT, MU_SF, MU_ST, ...
%     ANGMIN, ANGMAX, MU_ANGMIN, MU_ANGMAX] = idx_brch;
% [GEN_BUS, PG, QG, QMAX, QMIN, VG, MBASE, GEN_STATUS, PMAX, PMIN, ...
%     MU_PMAX, MU_PMIN, MU_QMAX, MU_QMIN, PC1, PC2, QC1MIN, QC1MAX, ...
%     QC2MIN, QC2MAX, RAMP_AGC, RAMP_10, RAMP_30, RAMP_Q, APF] = idx_gen;
