function [ dvPhaseFreq ] = swing2nd( t, voltPhaseFreq, param) %, opt )
%% swing2nd( t, voltPhaseFreq, param )
%
% * Parameters
%   * t : current time, necessary for solver algorithms
%   * voltPhaseFreq : voltage phases and frequencies in a vertical vector
%                    first N entries are phases followed by the frequencies
%   * param     : struct with swing equation parameters
%       * param.inertia : inertia constants of all machines (vert-vector)
%       * param.damping : damping constants of all machines (vert-vector)
%       * param.powerm  : mechanical power  of all machines (vert-vector)
%       * param.powerd  : diagonal (shunt) powers o. a. m.  (vert-vector)
%       * param.Kangle  : coupling angle matrix (matrix)
%       * param.Kmag    : coupling magnitude matrix (matrix)
%
%  * Returns
%   * dvphase   : infinitisimal voltage phase changes
%   * dvfreq    : infinitisimal voltage frequency changes
%
%
% V1.0 mawolff@uos.de 18-Aug-2020
 
%% pseudocode
 
 
%% input handling
% if( nargin < 3 ) 
%    opt = struct(); 
% end
 
%  opt = readOptions( opt );
 
%% code
N = numel(voltPhaseFreq) / 2;
[phasex, phasey] = ...                      % expand phase vectors to matrices
    meshgrid( voltPhaseFreq(1:N), voltPhaseFreq(1:N) );
dphase = phasey-phasex;                         % Theta difference in steady state
angleMat = sin(dphase-param.Kangle);            % make angle matrix for easier calculation
angleMat( logical(eye(size(angleMat))) ) = 0;   % no diagonal elements

dvPhaseFreq(1:N) = voltPhaseFreq( N+(1:N) );    % d(phase) = frequency
dvPhaseFreq(N+(1:N)) = ...                      % d(freq) = swing equation
    1./param.inertia .*( -param.damping .* voltPhaseFreq(N+(1:N)) ...
    + param.powerm(t) - param.powerd ...
    -sum( param.Kmag .* angleMat, 2 ) );

dvPhaseFreq = dvPhaseFreq';
return;
 
end
 
 
% function [ opt ] = readOptions( opt )
% %  % AUTO-GENERATED option handling
% %  if( ~isfield( opt, 'f') )
% %     opt.f = 0;
% %  end
% end
