function [ ] = trajPlot( x, y, ncolors, options )
%% trajPlot( x, y, ncolors, options )
% Plots a trajectory using different colors for time parts. It is also
% possible to set a period and cut the trajectory every time it exceeds it.
%
% * Parameters:
%   * x : The x-coordinate of the timeseries
%   * y : The y-coordinate of the timeseries
%   * (colors) (integer): how many parts with different colors the trajektory
%       should have. (Default: 10)
%   * (opt): options-struct. Possiblities: 
%       * options.cgrad : Color gradients: 
%                     'vrg'  : violett/blue to dark green (more colors than rg)
%                     'blue' : light to dark blue
%                     'rg'   : red to green
%       * options.fig : (figure handle/integer) figure to plot the trajectory in 
%               (otherwise a new one is opened).
%       * options.ax : axes handle, if not given, a new axis will be
%               created
%       * options.per : the period of the x-axis (for example considering a
%               pendulum the angular-axis has the period pi). Default: pi.
%               Use a very large number for non-periodic trajectories.
%       * options.z = []; if given the plot is a 3d plot where z is
%                           constant as given
%       * options.marker = 'none';
%       * options.linestyle = '-';
%       * options.linewidth = 2;
%
% * Returns:
%   * nothing yet
%
%               
% V2.00 mawolff@uos.de 18-Jun-2020

% version history
% V2.00 mawolff@uos.de 18-Jun-2020
% new algorithm for cutting. without xcutmat

%
% TODO: option for disabling border smoothing
% TODO: include jumps
% TODO: opt.axes

%% pseudo code

%% input handling

if( nargin < 4)             % if no options given
    options.f = [];         % create empty option struct
    if( nargin < 3)         % if also no number of traj color segments is given
        ncolors = 10;       % choose 10
    end
end

options = readOptions( options );   % read options or set to default
p = options.per;                % abbreviation for period
epsil = 0.85;                   % TODO 


%% code


x = reshape(x,1,numel(x));      % make sure x and y are row-vectors (1 x N)
y = reshape(y,1,numel(y));      % make sure x and y are row-vectors (1 x N)
% tudu: necessary?
x = x + p;                      % one period offset to fix problems with modulo calculations

N = size(x,2);                  % get number of data points

xnew = nan(1,3*N);
ynew = nan(1,3*N);
cuts = nan(1,3*N);
% cuts(1) = 1;
ncuts = 0;
nocuts = true;

% check for any jumps between two data points 
% jumps = find( abs(diff(x)) > epsil*2*p );
% jmp_i = 1;  % for comparison with other possible cuts
jumps = [];

inserts = 0;        % counter of elements inserted for smoothing borders
fullFi = 1;             % first index of (full) sequence
if( ~isempty(jumps) )
    % TODO 
else
    while( fullFi <= N )
        larger = find( x(fullFi:N) > 2*p, 1, 'first' );     % get index of next value exceeding right border
        smaller = find( x(fullFi:N) < 0, 1, 'first' );    % get index of next value exceeding left border
        
        if( isempty(smaller) && isempty(larger) )     % if no more cuts necessary
            nocuts = ncuts == 0;
            ncuts = ncuts + 1;
            cuts(ncuts) = N+inserts;    % finishing up sequence
            fullLi = N;
            nfi = fullFi + inserts;  
            nli = fullLi + inserts;
            xnew(nfi:nli) = x(fullFi:fullLi);
            ynew(nfi:nli) = y(fullFi:fullLi);
            break;              % end loop
        elseif( isempty( smaller ) )
            icut = larger;
            xcut = 2*p;     % where the new x values are cut by the border, see yins
            xcutinv = 0;    % the other boundary (invers)
        elseif( isempty(larger ) )
            xcut = 0;       % where the new x values are cut by the border, see yins
            xcutinv = 2*pi; % the other boundary (invers)
            icut = smaller;
        elseif( smaller < larger )
            xcut = 0;       % where the new x values are cut by the border, see yins
            xcutinv = 2*pi; % the other boundary (invers)
            icut = smaller;
        else
            xcut = 2*p;     % where the new x values are cut by the border, see yins
            xcutinv = 0;    % the other boundary (invers)
            icut = larger;
            %         [whch, icut] = min([smaller larger]); % get index of next cut in current sequence
        end
        
        fullIcut = icut + fullFi - 1;    % index in full sequence (starting at 1 not i)
        fullLi = fullIcut - 1;    % last value of the sequence (starting at 1)
        
        % insert two points for closing up with border
        yins = dots2Line( [x(fullLi) x(fullIcut)], ...
            [y(fullLi) y(fullIcut)], xcut) ;
        nfi = fullFi + inserts;   % starting index of current sequence including inserted elements
        inserts = inserts + 2;
        nli = fullLi + inserts;
        xnew(nfi:nli) = [ x(fullFi:fullLi) xcut xcutinv ];
        ynew(nfi:nli) = [ y(fullFi:fullLi) yins yins    ];
        
        ncuts = ncuts + 1;
        cuts(ncuts) = nli-1;    % last index of current sequence to plot
        % next sequence starts with same value (therefore 2 inserts)
        if( xcut < xcutinv )    % if cut is at 0
            x = x + 2*p;
        else                    % if cut is at 2p
            x = x - 2*p;
        end
        fullFi = fullLi + 1;
    end
end

xnew = xnew - p;    % reset shift

% =============================== colors  =============================== %
colors = zeros(ncolors,3);              % initialize array with colors for traj part
cgrad = options.cgrad;              % abbreviation to color gradient
if( strcmp(cgrad,'vrg') )               % if user want viored to green colors
    for i = 1:ncolors                   % for every part of the trajectory
        colors(i,:) = ...               % initialize its color
            vioredToGreenGradient(...   % as viored gradient
            i,ncolors);         
    end
elseif( strcmp(cgrad,'rg') )
    for i = 1:ncolors
        colors(i,:) = redToGreenGradient(i,ncolors);
    end
elseif( strcmp(cgrad,'blue') )
    for i = 1:ncolors
        colors(i,:) = lightToDarkBlueGradient(i,ncolors);
    end
elseif( strcmp(cgrad,'green') )
    for i = 1:ncolors
        colors(i,:) = GreenGradient(i,ncolors);
    end
elseif( strcmp(cgrad,'black') )
    for i = 1:(ncolors-1)
        colors(i,:) = blackOnly(i,ncolors);
    end
end

NN = N + inserts;
stepwColor = NN/ncolors;    % how long is one colored part of the trajectory
colorEnds = int32(0:stepwColor:NN);
colorEnds(1) = [];  % remove beginning of first interval, only ends needed
colorEnds = [colorEnds inf];    % last one to prevent exceeding
icolor = 1;
curColorEnd = colorEnds(1);

% =========================== plot trajectory =========================== %
figure(options.fig);
axes(options.ax);   % TODO opt.axes
hold on;
curStart = 1;   % start of current part of trajectory
if( nocuts )    % if trajectory has no cuts due to border cross
    disp('no cuts detected, just separating colors');
    xnew = x - p;
    ynew = y;
end

for i = 1:ncuts
    curEnd = cuts(i);
%     dispsn( 'start index: ', curStart );
%     dispsn( 'final index: ', curEnd   );
    while( curEnd >= curColorEnd )
%         dispsn( 'color start: ', curStart );
%         dispsn( 'color end: ', curColorEnd );
        internPlot( ...
            xnew(curStart:curColorEnd), ...
            ynew(curStart:curColorEnd),...
            colors(icolor,:), options );
        icolor = icolor + 1;
        curStart = curColorEnd;         % start next trajectory part where this one ended (no holes)
        curColorEnd = colorEnds(icolor);
    end
    internPlot( ...
        xnew(curStart:curEnd), ...
        ynew(curStart:curEnd),...
        colors(min(icolor,ncolors),:), options );
    curStart = curEnd + 1;
    % curEnd = cuts(i+1);
end




return;
end

function [ opt ] = readOptions( opt )
    
    if( ~isfield( opt, 'fig') )
        opt.fig = figure;
    end
    if( ~isfield( opt, 'ax') )
        opt.ax = axes;
    end
    if( ~isfield( opt, 'cgrad') )
        opt.cgrad = 'vrg';
    end
    if( ~isfield( opt, 'per') )
        opt.per = pi;
    end
    if( ~isfield( opt, 'z') )
        opt.z = [];
    end
    if( ~isfield( opt, 'marker') )
        opt.marker = 'none';
    end
    if( ~isfield( opt, 'linestyle') )
        opt.linestyle = '-';
    end
    if( ~isfield( opt, 'linewidth') )
        opt.linewidth = 2;
    end
    return;
        
end

function [] = internPlot( x,y,col,opt)
if( isempty( opt.z ) )
    plot(   x, y, ...
        'color',     col,...         % with predefined color,
        'Marker',    opt.marker,...          % given marker style,
        'linestyle', opt.linestyle,...       % given line style and
        'linewidth', opt.linewidth);         % given line width
else
    plot3(   x, y, ...
        ones(numel(x),1)*opt.z,...
        'color',     col,...         % with predefined color,
        'Marker',    opt.marker,...          % given marker style,
        'linestyle', opt.linestyle,...       % given line style and
        'linewidth', opt.linewidth);         % given line width
end

end

function [rgb] = vioredToGreenGradient(step, maxStep)
%% 
% returns the [r g b] vector for a step of an calculation in which the
% first result should be ploted red and the last green. At 1/2 the plot is
% yellow
% Parameters: 
% step: the current step of the calculation
% maxStep: the last step of the calculation. For scaling the rgb.
% Returns:
% rgb: [red green blue]

%%
x = step/maxStep;

og = 1; ug = 0;
a = [0 1/4]; b = [3/4 1]; c = [1/4 1/2]; %d = [1/2 3/4];
m = (b-a)./2+a;
peak = 1.5;

rg = peak - abs( (og-ug)./(c-a).*(x-m) );
rg = min ( rg, og);
rg = max( rg, ug );
rgb = [rg(1) rg(2) max(1-4*x,0)];
return;

end

function [rgb] = redToGreenGradient(step, maxStep)
%% 
% returns the [r g b] vector for a step of an calculation in which the
% first result should be ploted red and the last green. At 1/2 the plot is
% yellow
% Parameters: 
% step: the current step of the calculation
% maxStep: the last step of the calculation. For scaling the rgb.
% Returns:
% rgb: [red green blue]

%%
x = step/maxStep;
red = min([1 -2*x+2]);
green = min([1 2*x]);
rgb = [red green 0];
return;

end

function [rgb] = lightToDarkBlueGradient(step, maxStep)
%% 
% returns the [r g b] vector for a step of an calculation in which the
% first result should be ploted lightblue and the last darkblue. 
% Parameters: 
% step: the current step of the calculation
% maxStep: the last step of the calculation. For scaling the rgb.
% Returns:
% rgb: [red green blue]
%
% V1.0 mawolff@uos.de 9-Feb-2017

%%
x = step/maxStep;
hell = 0.85;
red = -hell*x+hell;
green = -hell*x+hell;
blue = -0.5*x+1;
rgb = [red green blue];
return;

end

function [rgb] = GreenGradient(step, maxStep)
%% 
% returns the [r g b] vector for a step of an calculation in which the
% first result should be ploted red and the last green. At 1/2 the plot is
% yellow
% Parameters: 
% step: the current step of the calculation
% maxStep: the last step of the calculation. For scaling the rgb.
% Returns:
% rgb: [red green blue]

%%
x = (step+2.5*maxStep)/(3.7*maxStep);

og = 1; ug = 0;
a = [0 1/4]; b = [3/4 1]; c = [1/4 1/2]; %d = [1/2 3/4];
m = (b-a)./2+a;
peak = 1.5;

rg = peak - abs( (og-ug)./(c-a).*(x-m) );
rg = min ( rg, og);
rg = max( rg, ug );
rgb = [rg(1) rg(2) max(1-4*x,0)];
return;

end

function [rgb] = blackOnly(~, ~)
%% 
% returns the [r g b] = [ 0 0 0] (black). For unifying only.
% Returns:
% rgb: [red green blue]
%
% V1.0 mawolff@uos.de 9-Feb-2017

%%
rgb = [0 0 0];
return;

end


