function [ coords ] = val2FigCoords( xy, ax )
%% val2FigCoords( xy, ax )
% 
% Calculates coordinates in a figure from coresponding values in the axis. 
%
% * Parameters:
%   * xy: A vector with values from the plot. The length of this vector
%           must be a multiple of two. The result "coords" will have the
%           same length. The vector is read in the way [x1 y1 x2 y2 ... ]
%   * (ax): The axis in which the values are. If not given the current axis
%           will be used.
%
% * Returns:
%   * coords: The vector with the coordinates corresponding to the values
%           given in xy. It is in percent and therefore in [0, 1].
%   
% V1.0 mawolff@uos.de 27-Feb-2017
%

%% code
if( nargin < 2)
    ax = gca;
end
if( mod(numel(xy),2) ~= 0 )
    error('InputError:VectorLengthNotEven', ['The input vector must have an ' ...
            'even number of elements (multiple of 2)']);
end

pos = get(ax, 'position');
xl = get(ax, 'xlim');
yl = get(ax, 'ylim');

xy = reshape(xy,1,numel(xy));   % ensure that input is row vector
coords = zeros(1,numel(xy));    % preallocate result vector

xislog = strcmp(get(ax, 'xscale'), 'log');
yislog = strcmp(get(ax, 'yscale'), 'log');
if( xislog || yislog)           % if axis are logarithmic they must be handled different
                                % such that the scale is linear again
    if( any([xl yl] == 0) )     % if there is is a zero in the axis limits it causes trouble 
        warning(['One of the axis boundaries is zero and '... % with logarithm
            'the corresponding scale is logarithmic. The coordinates '...
            'may be inaccurate']);
        xl(xl==0) = min(xy(1:2:end))/10;
        yl(yl==0) = min(xy(2:2:end))/10;
    end
    if( xislog )
        xl = log10(xl);
        xy(1:2:end) = log10(xy(1:2:end));
    end
    if( yislog )
        yl = log10(yl);
        xy(2:2:end) = log10(xy(2:2:end));
    end
end

for i = 1:numel(xy)/2
    coords(2*i-1) = pos(1)+ pos(3)/(xl(2)-xl(1)) ...    % get x coordinates by rule of three
        * (xy(2*i-1)-xl(1));   
    coords(2*i) = pos(2)+ pos(4)/(yl(2)-yl(1)) ...      % get y coordinates by rule of three
        * (xy(2*i)-yl(1));
end

return;
end

